@@ -23,6 +23,7 @@
 #include "accessdesc.hpp"
 #include "initsysvar.hpp"
 #include "gdljournal.hpp"
+#include "nullgdl.hpp"
 
 //class ProgNode;
 //typedef ProgNode* ProgNodeP;
@@ -85,7 +86,6 @@ private:
     friend class WRAPPED_PRONode;
 
 public: 
-
 //     RetCode returnCode;    
     ProgNodeP GetNULLProgNodeP() const { return NULLProgNodeP;}
 
@@ -116,7 +116,7 @@ public:
     static bool SearchCompilePro(const std::string& pro, bool searchForPro); 
     static int GetFunIx( ProgNodeP);
     static int GetFunIx( const std::string& subName);
-    static int GetProIx( ProgNodeP, bool throwImmediately=false);//const std::string& subName);
+    static int GetProIx( ProgNodeP);//const std::string& subName);
     static int GetProIx( const std::string& subName);
     DStructGDL* ObjectStruct( DObjGDL* self, ProgNodeP mp);
     void SetRootR( ProgNodeP tt, DotAccessDescT* aD, BaseGDL* r, ArrayIndexListT* aL);
@@ -130,7 +130,7 @@ public:
 
 private: 
 
-    static void SetProIx( ProgNodeP f, bool throwImmediately=false); // triggers read/compile
+    static void SetProIx( ProgNodeP f); // triggers read/compile
     static void AdjustTypes( BaseGDL*&, BaseGDL*&);
 
 
@@ -172,9 +172,7 @@ protected:
     BaseGDL** returnValueL; // holding the return value for l_functions
 
     bool interruptEnable;
-
 public:
-
     bool InterruptEnable() const { return interruptEnable;}
     // procedure (searchForPro == true) or function (searchForPro == false)
     static bool CompileFile(const std::string& f, 
@@ -198,11 +196,10 @@ protected:
     static ObjHeapT  objHeap; 
 
     // index for newly allocated heap variables
-    static SizeT objHeapIx;
     static SizeT heapIx;
 
     static EnvStackT  callStack; 
-
+    static bool noInteractive;
     static DLong stepCount;
 
 
@@ -232,10 +229,10 @@ public:
     // the New... functions 'own' their BaseGDL*
     SizeT NewObjHeap( SizeT n=1, DStructGDL* var=NULL)
     {
-        SizeT tmpIx=objHeapIx;
+        SizeT tmpIx=heapIx;
         for( SizeT i=0; i<n; i++)
         objHeap.insert( objHeap.end(),
-            std::pair<SizeT, RefDStructGDL>( objHeapIx++, (DStructGDL*)var));
+            std::pair<SizeT, RefDStructGDL>( heapIx++, (DStructGDL*)var));
         return tmpIx;
     }
     SizeT NewHeap( SizeT n=1, BaseGDL* var=NULL)
@@ -250,9 +247,7 @@ public:
     {
         BaseGDL* del = (*it).second.get();
         objHeap.erase( id); 
-        delete del;
-        // delete (*it).second.get();
-        // objHeap.erase( id);
+        if (!NullGDL::IsNULLorNullGDL(del)) delete del; //avoid destroying !NULL
     }
     static void FreeObjHeap( DObj id)
     {
@@ -262,8 +257,6 @@ public:
             if  ( it != objHeap.end()) 
             { 
                 FreeObjHeapDirect( id, it);
-                // delete (*it).second.get();
-                // objHeap.erase( id);
             }
         }
     }
@@ -271,10 +264,7 @@ public:
     {
         BaseGDL* del = (*it).second.get();
         heap.erase( id); 
-        delete del;
-        // delete (*it).second.get();
-        // // useless because of next: (*it).second.get() = NULL;
-        // heap.erase( id); 
+        if (!NullGDL::IsNULLorNullGDL(del)) delete del; //avoid destroying !NULL
     }
     static void FreeHeap( DPtr id)
     {
@@ -284,8 +274,6 @@ public:
                 if( it != heap.end()) 
                     { 
                         FreeHeapDirect( id, it);
-                        // delete (*it).second.get();
-                        // heap.erase( id); 
                     }
             }
     }
@@ -362,10 +350,8 @@ public:
        }
    }
    static void EnableAllGC() {
-        SizeT nEl = heap.size();
         for( HeapT::iterator it=heap.begin(); it != heap.end(); ++it)
 			it->second.EnableGC(true);
-        nEl = objHeap.size();
         for( ObjHeapT::iterator it=objHeap.begin(); it != objHeap.end(); ++it)
 			it->second.EnableGC(true);
    }
@@ -717,17 +703,18 @@ std::cout << add << " + <ObjHeapVar" << id << ">" << std::endl;
     {
         for( HeapT::iterator it=heap.begin(); it != heap.end(); ++it)
         {
-           delete (*it).second.get();
+           BaseGDL* del = (*it).second.get();
+           if (!NullGDL::IsNULLorNullGDL(del)) delete del; //avoid destroying !NULL
            heap.erase( it->first); 
         }
         for( ObjHeapT::iterator it=objHeap.begin(); it != objHeap.end(); ++it)
         {
-            delete (*it).second.get();
-            objHeap.erase( it->first); 
+           BaseGDL* del = (*it).second.get();
+           if (!NullGDL::IsNULLorNullGDL(del)) delete del; //avoid destroying !NULL
+           heap.erase( it->first);
         }
 // The counters are reset for easier human readability.
        heapIx = 1;
-       objHeapIx = 1;
     }
 
     // name of data
@@ -792,6 +779,7 @@ std::cout << add << " + <ObjHeapVar" << id << ">" << std::endl;
         std::cerr << std::endl;
         
         if( dumpStack) DumpStack( emsg.size() + 1);
+        if (noInteractive) exit(EXIT_SUCCESS); //strangely, IDL exits on error when non interactive with 0 not 1.
     }
     
     static void DumpStack( SizeT w)
@@ -868,6 +856,7 @@ std::cout << add << " + <ObjHeapVar" << id << ">" << std::endl;
             std::cerr << std::left << " " << file;
         }
         std::cerr << std::endl;
+        if (noInteractive) exit(EXIT_SUCCESS);
     }
 
     static void RetAll( RetAllException::ExCode c=RetAllException::NONE)    
@@ -908,9 +897,9 @@ public:
 		return GDLInterpreter::tokenNames;
 	}
 	public:  RetCode  interactive(ProgNodeP _t);
-	public:  RetCode  statement(ProgNodeP _t, bool throwImmediately=false);
-	public:  RetCode  execute(ProgNodeP _t, bool throwImmediately=false);
-	public:  RetCode  statement_list(ProgNodeP _t, bool throwImmediately=false);
+	public:  RetCode  statement(ProgNodeP _t);
+	public:  RetCode  execute(ProgNodeP _t);
+	public:  RetCode  statement_list(ProgNodeP _t);
 	public:  BaseGDL*  call_fun(ProgNodeP _t);
 	public:  BaseGDL**  call_lfun(ProgNodeP _t);
 	public: void call_pro(ProgNodeP _t);
