@@ -96,7 +96,7 @@ public:
 //     if( cleanupIxIn != NULL)
 //       cleanupIx = *cleanupIxIn;
 
-    DLongGDL* isRange = new DLongGDL( dimension(1, BaseGDL::NOZERO));
+    DLongGDL* isRange = new DLongGDL( dimension(1), BaseGDL::NOZERO);
     ixOut.push_back(isRange);
     
     (*isRange)[ 0] = (ix->IsRange()) ? 1 : 0;
@@ -365,11 +365,12 @@ public:
 
   // called after structure is fixed
   ArrayIndexListOneScalarNoAssocT( ArrayIndexVectorT* aIV)
-//     : allIx( NULL)
+      : varIx (static_cast<ArrayIndexScalar*>((*aIV)[0])->GetVarIx())
+//    allIx( NULL)
   {
     nParam = 0;
-    
-    varIx = static_cast<ArrayIndexScalar*>((*aIV)[0])->GetVarIx();
+   // cppchek says: (performance) Variable 'varIx' is assigned in constructor body. Consider performing initialization in initialization list
+//    varIx = static_cast<ArrayIndexScalar*>((*aIV)[0])->GetVarIx();
 
     // ArrayIndexListOneScalarT will do the cleanup
 //     delete (*aIV)[0];
@@ -892,7 +893,7 @@ public:
   { 
     assert( ix.size() == 0);
 
-    DLongGDL* isRange = new DLongGDL( dimension(ixList.size(), BaseGDL::ZERO));
+    DLongGDL* isRange = new DLongGDL( dimension(ixList.size()), BaseGDL::ZERO);
     ixOut.push_back(isRange);
     
     for( SizeT i=0; i<ixList.size(); ++i)
@@ -1100,7 +1101,7 @@ public:
   { 
     assert( ix.size() == 0);
 
-    DLongGDL* isRange = new DLongGDL( dimension(ixList.size(), BaseGDL::ZERO));
+    DLongGDL* isRange = new DLongGDL( dimension(ixList.size()), BaseGDL::ZERO);
     ixOut.push_back(isRange);
     
     for( SizeT i=0; i<ixList.size(); ++i)
@@ -1379,7 +1380,7 @@ public:
 //     if( cleanupIxIn != NULL)
 //       cleanupIx = *cleanupIxIn;
 
-    DLongGDL* isRange = new DLongGDL( dimension(ixList.size(), BaseGDL::NOZERO));
+    DLongGDL* isRange = new DLongGDL( dimension(ixList.size()), BaseGDL::NOZERO);
     ixOut.push_back(isRange);
     
     SizeT pIX = 0;
