@@ -142,7 +142,7 @@ std::istream& BaseGDL::FromStream(std::istream& i)
   return i;
 }
 
-bool BaseGDL::OutOfRangeOfInt() const { return false;}
+//bool BaseGDL::OutOfRangeOfInt() const { return false;}
 
 bool BaseGDL::Scalar() const { return false;}
 bool BaseGDL::StrictScalar() const { return false;}
@@ -291,9 +291,10 @@ bool BaseGDL::ArrayNeverEqual( BaseGDL*)
 }
 
 // for statement compliance (int types , float types scalar only)
-void BaseGDL::ForCheck( BaseGDL**, BaseGDL**)
+bool BaseGDL::ForCheck( BaseGDL**, BaseGDL**)
 {
   throw GDLException("Operation not defined for UNDEF 7.");
+  return false;
 }
 
 bool BaseGDL::ForCondUp( BaseGDL*)
@@ -769,14 +770,65 @@ SizeT MemStats::NumFree = 0;
 SizeT MemStats::HighWater = 0;
 SizeT MemStats::Current = 0;
 
-#if !defined(HAVE_MALLINFO) 
+#if (!defined(HAVE_MALLINFO) && !defined(HAVE_MALLINFO2))
 #  if (!defined(HAVE_MALLOC_ZONE_STATISTICS) || !defined(HAVE_MALLOC_MALLOC_H))
 #    if defined(HAVE_SBRK)
 char* MemStats::StartOfMemory = reinterpret_cast<char*>(::sbrk(0));
 #    endif
 #  endif
+#endif
+  // returns current memory usage and updates the highwater mark
+  void MemStats::UpdateCurrent() 
+  { 
+    // ---------------------------------------------------------------------
+    // based on the codes from:
+    // - the LLVM project (lib/System/Unix/Process.inc) see http://llvm.org/
+    // - the Squid cache project (src/tools.cc) see http://squid-cache.org/
+    // TODO (TOCHECK): Squid considers also gnumalloc.h - ?
+#if defined(HAVE_MALLINFO2)
+    // Docs see below, newer versions of glibc deprecated mallinfo()
+    static struct mallinfo2 mi;
+    mi = mallinfo2();
+    Current = mi.arena+mi.hblkhd;
+#elif defined(HAVE_MALLINFO)
+    // Linux case for example
+    static struct mallinfo mi;
+    mi = mallinfo();
+//         printf("Total non-mmapped bytes (arena):       %d\n", mi.arena);
+//           printf("# of free chunks (ordblks):            %d\n", mi.ordblks);
+//           printf("# of free fastbin blocks (smblks):     %d\n", mi.smblks);
+//           printf("# of mapped regions (hblks):           %d\n", mi.hblks);
+//           printf("Bytes in mapped regions (hblkhd):      %d\n", mi.hblkhd);
+//           printf("Max. total allocated space (usmblks):  %d\n", mi.usmblks);
+//           printf("Free bytes held in fastbins (fsmblks): %d\n", mi.fsmblks);
+//           printf("Total allocated space (uordblks):      %d\n", mi.uordblks);
+//           printf("Total free space (fordblks):           %d\n", mi.fordblks);
+//           printf("Topmost releasable block (keepcost):   %d\n", mi.keepcost);
+      Current = mi.arena+mi.hblkhd; //was mi.uordblks;
+#elif defined(HAVE_MALLOC_ZONE_STATISTICS) && defined(HAVE_MALLOC_MALLOC_H)
+    // Mac OS X case for example
+    static malloc_statistics_t stats;
+    malloc_zone_statistics(malloc_default_zone(), &stats);
+    Current = stats.size_in_use;
+#elif defined(HAVE_SBRK)
+    // Open Solaris case for example
+    static char* EndOfMemory;
+    EndOfMemory = (char*)sbrk(0);
+    Current = EndOfMemory - StartOfMemory;
+#elif defined(_WIN32)
+    // a draft of Windows version (for neccesarry includes consult the LLVM source): 
+    _HEAPINFO hinfo;
+    hinfo._pentry = NULL;
+    Current = 0;
+    while (_heapwalk(&hinfo) == _HEAPOK)
+        if (hinfo._useflag == _USEDENTRY)
+            Current += hinfo._size;
+#else
+    Warning("Cannot get dynamic memory information on this platform (FIXME)");
 #endif
 
+    HighWater = std::max(HighWater, Current);
+  }
 // ---
 
 void GDLDelete( BaseGDL* toDelete)
