@@ -37,7 +37,7 @@
 #ifdef HAVE_MALLOC_MALLOC_H
 #  include <malloc/malloc.h>
 #endif
-#if !defined(HAVE_MALLINFO) 
+#if (!defined(HAVE_MALLINFO) && !defined(HAVE_MALLINFO2))
 #  if (!defined(HAVE_MALLOC_ZONE_STATISTICS) || !defined(HAVE_MALLOC_MALLOC_H))
 #    if defined(HAVE_SBRK)
 #      include <unistd.h>
@@ -228,8 +228,25 @@ const bool IsNonPODType[]={
   false, 	//GDL_ULONG,
   false, 	//GDL_LONG64,
   false 	//GDL_ULONG64
-};	
-
+};
+ const bool IsLongLongType[] = {
+  false, //GDL_UNDEF
+  false, //GDL_BYTE
+  false, //GDL_INT
+  false, //GDL_LONG,	
+  false, //GDL_FLOAT,	
+  true, //GDL_DOUBLE,	
+  false, //GDL_COMPLEX,	
+  false, //GDL_STRING,	
+  false, //GDL_STRUCT,	
+  true, //GDL_COMPLEXDBL,	
+  false, //GDL_PTR,		
+  false, //GDL_OBJ,
+  false, //GDL_UINT,	
+  false, //GDL_ULONG,
+  true, //GDL_LONG64,
+  true //GDL_ULONG64
+ };
   
 } //namespace gdl_type_lookup 
 
@@ -267,16 +284,17 @@ inline bool NumericType( DType t) // Float or Int or Complex
 inline bool ConvertableType( DType t) // everything except Struct, Ptr, Obj
 {
   return gdl_type_lookup::IsConvertableType[ t];
-//   int o = DTypeOrder[ t];
-//   return (o >= 1 && o <= 11);
 }
-
+inline bool LongLongType( DType t) // POD but 64 bits
+{
+  return gdl_type_lookup::IsLongLongType[ t];
+}
 class   BaseGDL;
 class   ArrayIndexListT;
 //class   ExprListT;
 
-struct SpDULong;
-template<class> class Data_;
+//struct SpDULong;
+//template<class> class Data_;
 
 struct ForLoopInfoT;
 
@@ -292,7 +310,7 @@ private:
   // SizeT has architecture-dependant size (32/64 bit)
   static SizeT NumAlloc, NumFree, HighWater, Current;
 
-#if !defined(HAVE_MALLINFO) 
+#if (!defined(HAVE_MALLINFO) && !defined(HAVE_MALLINFO2))
 #  if (!defined(HAVE_MALLOC_ZONE_STATISTICS) || !defined(HAVE_MALLOC_MALLOC_H))
 #    if defined(HAVE_SBRK)
   static char* StartOfMemory;
@@ -333,51 +351,8 @@ public:
   }
 
   // returns current memory usage and updates the highwater mark
-  static void UpdateCurrent() 
-  { 
-    // ---------------------------------------------------------------------
-    // based on the codes from:
-    // - the LLVM project (lib/System/Unix/Process.inc) see http://llvm.org/
-    // - the Squid cache project (src/tools.cc) see http://squid-cache.org/
-    // TODO (TOCHECK): Squid considers also gnumalloc.h - ?
-#if defined(HAVE_MALLINFO)
-    // Linux case for example
-    static struct mallinfo mi;
-    mi = mallinfo();
-//         printf("Total non-mmapped bytes (arena):       %d\n", mi.arena);
-//           printf("# of free chunks (ordblks):            %d\n", mi.ordblks);
-//           printf("# of free fastbin blocks (smblks):     %d\n", mi.smblks);
-//           printf("# of mapped regions (hblks):           %d\n", mi.hblks);
-//           printf("Bytes in mapped regions (hblkhd):      %d\n", mi.hblkhd);
-//           printf("Max. total allocated space (usmblks):  %d\n", mi.usmblks);
-//           printf("Free bytes held in fastbins (fsmblks): %d\n", mi.fsmblks);
-//           printf("Total allocated space (uordblks):      %d\n", mi.uordblks);
-//           printf("Total free space (fordblks):           %d\n", mi.fordblks);
-//           printf("Topmost releasable block (keepcost):   %d\n", mi.keepcost);
-      Current = mi.arena+mi.hblkhd; //was mi.uordblks;
-#elif defined(HAVE_MALLOC_ZONE_STATISTICS) && defined(HAVE_MALLOC_MALLOC_H)
-    // Mac OS X case for example
-    static malloc_statistics_t stats;
-    malloc_zone_statistics(malloc_default_zone(), &stats);
-    Current = stats.size_in_use;
-#elif defined(HAVE_SBRK)
-    // Open Solaris case for example
-    static char* EndOfMemory;
-    EndOfMemory = (char*)sbrk(0);
-    Current = EndOfMemory - StartOfMemory;
-#else
-  /*
-   * // a draft of Windows version (for neccesarry includes consult the LLVM source): 
-   *  _HEAPINFO hinfo;
-   *  hinfo._pentry = NULL;
-   *  Current = 0;
-   *  while (_heapwalk(&hinfo) == _HEAPOK) Current += hinfo._size;
-   */ 
-   // Warning("Cannot get dynamic memory information on this platform (FIXME)");
-#endif
+  static void UpdateCurrent() ;
 
-    HighWater = std::max(HighWater, Current);
-  }
 
 };
 
@@ -516,7 +491,7 @@ public:
   virtual BaseGDL* CShift( DLong d) const; // circular shift
   virtual BaseGDL* CShift( DLong d[MAXRANK]) const; // circular shift multi dim
 
-  virtual bool OutOfRangeOfInt() const;
+//  virtual bool OutOfRangeOfInt() const;
 
   virtual bool Scalar() const;
   virtual bool StrictScalar() const;
@@ -555,7 +530,7 @@ public:
   virtual bool ArrayNeverEqual( BaseGDL*);
   
   // for statement compliance (int types , float types scalar only)
-  virtual void ForCheck( BaseGDL**, BaseGDL** = NULL);
+  virtual bool ForCheck( BaseGDL**, BaseGDL** = NULL);
   virtual bool ForCondUp( BaseGDL*);
   virtual bool ForCondDown( BaseGDL*);
   virtual bool ForAddCondUp( BaseGDL* loopInfo);
