@@ -20,10 +20,10 @@
 
 namespace lib {
 
-  double Gregorian2Julian(struct tm ts);
+  BaseGDL* timestamp(EnvT* e);
+  void timestamptovalues(EnvT* e);
   BaseGDL* systime(EnvT* e);
   BaseGDL* legendre(EnvT* e);
-  BaseGDL* interpol(EnvT* e);
   BaseGDL* gsl_exp(EnvT* e);
 
 } // namespace
