@@ -15,14 +15,22 @@
  *                                                                         *
  ***************************************************************************/
 
-// to be included from datatypes.cpp
-#ifdef INCLUDE_DEFAULT_IO_CPP
 
-//#include "datatypes.hpp"
-//#include "dstructgdl.hpp"
-//#include "arrayindex.hpp".
-//#include "assocdata.hpp"
-//#include "io.hpp"
+#include "includefirst.hpp"
+
+#include "nullgdl.hpp"
+#include "typedefs.hpp"
+//#include "datatypes.hpp" // for friend declaration
+
+#include "dstructgdl.hpp"
+#include "arrayindexlistt.hpp"
+#include "assocdata.hpp"
+#include "dinterpreter.hpp"
+
+// needed with gcc-3.3.2
+#include <cassert>
+
+
 #include "ofmt.hpp"
 #include "gdljournal.hpp"
 #include "list.hpp"
@@ -53,8 +61,13 @@ void SkipWS( istream& is)
 	is.clear();
 	return ;
       }
-  } while( c == ' ' || c == '\t' || c == '\n');
-
+  } while( c == ' ' || c == '\t' || c == '\n' || c == '\r' ); //special check if line is terminated with <CR><LF>:
+#ifndef _WIN32
+  if (c == '\r') {
+      char next=is.get();
+      if (next!='\n') is.unget(); //CRLF case: gobble the \cr of crlf if not WIN32 
+    }
+#endif
   is.unget();
 }
 
@@ -131,6 +144,9 @@ const string ReadElement(istream& is)
 // no skip of WS
 const string ReadComplexElement(istream& is)
 {
+
+  //  cout << " hello ReadComplexElement : " << endl;
+
   SkipWS( is);
   
   string buf;
@@ -230,7 +246,7 @@ istream& operator>>(istream& i, Data_<Sp>& data_)
       if( cEnd == cStart)
 	{
 	  data_[ assignIx]= -1;
-	  Warning("Input conversion error.");
+	  ThrowGDLException("Input conversion error.");
 	}
 	  
       assignIx++;
@@ -240,7 +256,7 @@ istream& operator>>(istream& i, Data_<Sp>& data_)
   return i;
 }
 
-// float
+// float : uses StrToD (character D in string)
 template<> 
 istream& operator>>(istream& i, Data_<SpDFloat>& data_) 
 {
@@ -256,7 +272,7 @@ istream& operator>>(istream& i, Data_<SpDFloat>& data_)
       if( cEnd == cStart)
 	{
 	  data_[ assignIx]= -1;
-	  Warning("Input conversion error.");
+	  ThrowGDLException("Input conversion error.");
 	}
 	  
       assignIx++;
@@ -265,7 +281,7 @@ istream& operator>>(istream& i, Data_<SpDFloat>& data_)
 
   return i;
 }
-// double
+// double : uses StrToD (character D in string)
 template<> 
 istream& operator>>(istream& i, Data_<SpDDouble>& data_) 
 {
@@ -281,7 +297,7 @@ istream& operator>>(istream& i, Data_<SpDDouble>& data_)
       if( cEnd == cStart)
 	{
 	  data_[ assignIx]= -1;
-	  Warning("Input conversion error.");
+	  ThrowGDLException("Input conversion error.");
 	}
       
       assignIx++;
@@ -290,6 +306,7 @@ istream& operator>>(istream& i, Data_<SpDDouble>& data_)
 
   return i;
 }
+
 // complex
 template<> 
 istream& operator>>(istream& i, Data_<SpDComplex>& data_) 
@@ -302,9 +319,11 @@ istream& operator>>(istream& i, Data_<SpDComplex>& data_)
       const string& actLine = ReadComplexElement( i);
       SizeT strLen = actLine.length();
 
+      // cout << "Processing : " << actLine <<endl;
+      // AC 2020 May : since now, always a space between "(" and digit
       if( actLine[ 0] == '(')
-	{
-	  SizeT mid  = actLine.find_first_of(" \t,",1);
+	{	  
+	  SizeT mid  = actLine.find_first_of(" \t,",2);
 	  if( mid >= strLen) mid = strLen;
 	      
 	  string seg1 = actLine.substr( 1, mid-1);
@@ -318,8 +337,11 @@ istream& operator>>(istream& i, Data_<SpDComplex>& data_)
 
 	  if( last <= next)
 	    {
-	      data_[ assignIx]= DComplex(0.0,0.0);
-	      Warning("Imaginary part of complex missing.");
+	      char* cEnd1;
+	      const char* c1=seg1.c_str();
+	      double re = StrToD( c1, &cEnd1);
+	      data_[ assignIx]= DComplex(re,0.0); //this is what IDL does
+//	      Warning("Imaginary part of complex missing."); no warning for IDL
 	    } 
 	  else
 	    {
@@ -334,7 +356,7 @@ istream& operator>>(istream& i, Data_<SpDComplex>& data_)
 	      if( cEnd1 == c1 || cEnd2 == c2)
 		{
 		  data_[ assignIx]= DComplex(0.0,0.0);
-		  Warning("Input conversion error.");
+		  ThrowGDLException("Input conversion error.");
 		}
 	      else
 		{
@@ -354,22 +376,21 @@ istream& operator>>(istream& i, Data_<SpDComplex>& data_)
 	  if( cEnd == cStart)
 	    {
 	      data_[ assignIx]= DComplex(0.0,0.0);
-	      Warning("Input conversion error.");
+	      ThrowGDLException("Input conversion error.");
 	    }
+	  //	  cout << val << endl;
+	  //	  for( long int c=assignIx; c<nTrans; c++)
+	  data_[assignIx] = DComplex(val,0.0);
 	  
-	  for( long int c=assignIx; c<nTrans; c++)
-	    data_[ c] = DComplex(val,0.0);
-	  
-	  // i.seekg( pos); // rewind stream
-	  
-	  return i;
+	  // AC 2020 May unclear why we need that ... 
+	  // i.seekg( pos); // rewind stream	  
+	  //	  return i;
 	}
 	  
       assignIx++;
       nTrans--;
     }
 
-
   return i;
 }
 // double complex
@@ -384,9 +405,10 @@ istream& operator>>(istream& i, Data_<SpDComplexDbl>& data_)
       const string& actLine = ReadComplexElement( i);
       SizeT strLen = actLine.length();
 
+      // AC 2020 May : since now, always a space between "(" and digit
       if( actLine[ 0] == '(')
 	{
-	  SizeT mid  = actLine.find_first_of(" \t,",1);
+	  SizeT mid  = actLine.find_first_of(" \t,",2);
 	  if( mid >= strLen) mid = strLen;
 	      
 	  string seg1 = actLine.substr( 1, mid-1);
@@ -401,7 +423,7 @@ istream& operator>>(istream& i, Data_<SpDComplexDbl>& data_)
 	  if( last <= next)
 	    {
 	      data_[ assignIx]= DComplexDbl(0.0,0.0);
-	      Warning("Imaginary part of complex missing.");
+//	      Warning("Imaginary part of complex missing.");
 	    } 
 	  else
 	    {
@@ -416,7 +438,7 @@ istream& operator>>(istream& i, Data_<SpDComplexDbl>& data_)
 	      if( cEnd1 == c1 || cEnd2 == c2)
 		{
 		  data_[ assignIx]= DComplexDbl(0.0,0.0);
-		  Warning("Input conversion error.");
+		  ThrowGDLException("Input conversion error.");
 		}
 	      else
 		{
@@ -436,22 +458,21 @@ istream& operator>>(istream& i, Data_<SpDComplexDbl>& data_)
 	  if( cEnd == cStart)
 	    {
 	      data_[ assignIx]= DComplexDbl(0.0,0.0);
-	      Warning("Input conversion error.");
+	      ThrowGDLException("Input conversion error.");
 	    }
 	  
-	  for( long int c=assignIx; c<nTrans; c++)
-	    data_[ c] = DComplexDbl(val,0.0);
+	  //	  for( long int c=assignIx; c<nTrans; c++)
+	  data_[ assignIx] = DComplexDbl(val,0.0);
 	  
+	  // AC 2020 May unclear why we need that ... 
 	  // i.seekg( pos); // rewind stream
-	  
-	  return i;
+	  //	  return i;
 	}
-	  
+
       assignIx++;
       nTrans--;
     }
 
-
   return i;
 }
 
@@ -1226,8 +1247,9 @@ ostream& Data_<SpDString>::ToStream(ostream& o, SizeT w, SizeT* actPosPtr)
   SizeT length;
   if( this->dim.Rank() == 0)
     {
-      length = (*this)[0].length();
-      o << CheckNL( w, actPosPtr, length) << (*this)[0];
+      length = (*this)[0].length();  //app
+      // this is correct but gives bug #2876161 : if (length ==0) o << '\n'; else  
+        o << CheckNL( w, actPosPtr, length) << (*this)[0];
       return o;
     }
 
@@ -1729,15 +1751,15 @@ istream& Data_<SpDByte>::Read( istream& os, bool swapEndian, bool compress, XDR
     os.read( buf, 4 );
     xdrmem_create( xdrs, &buf[0], 4, XDR_DECODE );
     short int length = 0;
-    if ( !xdr_short( xdrs, &length ) ) throw GDLIOException( "Problem reading XDR file." );
+    if ( !xdr_short( xdrs, &length ) ) {free( buf ); throw GDLIOException( "Problem reading XDR file." );}
     xdr_destroy( xdrs );
     free( buf );
     if ( length <= 0 ) return os;
 
     int bufsize = 4 * ((length - 1) / 4 + 1);
-    buf = (char *) calloc( length, sizeof (char) );
+    buf = (char *) calloc( bufsize, sizeof (char) );
     os.read( &buf[0], bufsize );
-    if ( !os.good( ) ) throw GDLIOException( "Problem reading XDR file." ); //else we are correctly aligned for next read!
+    if ( !os.good( ) ) {free( buf ); throw GDLIOException( "Problem reading XDR file." );} //else we are correctly aligned for next read!
     //do it by ourselves, faster and surer!
     if ( bufsize < nChar ) nChar = bufsize; //truncate eventually
     for ( SizeT i = 0; i < nChar; i++ ) ( *this )[i] = buf[i];
@@ -1785,7 +1807,7 @@ bool compress, XDR *xdrs ) {
         (*this)[i].clear();
       } else {
         int bufsize = 4 + 4 * ((length - 1) / 4 + 1) ;
-        buf = (char *) calloc( length, sizeof (char) );
+        buf = (char *) calloc( bufsize, sizeof (char) );
         os.read( &buf[0], bufsize );
         if ( !os.good( ) ) throw GDLIOException( "Problem reading XDR file." ); //else we are correctly aligned for next read!
         (*this)[i].assign( &buf[4], length );
@@ -1812,7 +1834,9 @@ bool compress, XDR *xdrs ) {
         } else {
           os.read( &vbuf[0], nChar );
         }
-        (*this)[i].assign( &vbuf[0], nChar );
+        //we have read nChar but this (a std::string) may be filled with ASCII NULL chars. These MUST be removed if we want the rest of GDL behave correctly on string manipulations (strlen, where etc...)
+        while (vbuf[nChar-1]==0 && nChar>1) {nChar--;}
+	(*this)[i].assign( &vbuf[0], nChar );
       }
     }
   }
@@ -1858,6 +1882,4 @@ BaseGDL* DStructGDL::AssocVar( int lun, SizeT offset)
   return new Assoc_<DStructGDL >( lun, this, offset);
 }
 
-//#include "instantiate_templates.hpp"
-
-#endif
+#include "instantiate_templates.hpp"
