@@ -17,10 +17,6 @@
  ***************************************************************************/
 #include "includefirst.hpp"
 
-#ifdef HAVE_LIBWXWIDGETS
-#include <wx/filedlg.h>
-#include <wx/msgdlg.h>
-#endif
 
 #include "envt.hpp"
 #include "dialog.hpp"
@@ -33,15 +29,10 @@ namespace lib {
 #ifdef HAVE_LIBWXWIDGETS
   BaseGDL* wxwidgets_exists(EnvT* e)
   {
-    return new DLongGDL(1);
+    return new DIntGDL(1);
   }
 
-  BaseGDL* dialog_pickfile_wxwidgets(EnvT* e)
-  {
-    if (!GDLWidget::wxIsStarted()){
-      if( ! wxInitialize( ) ) e->Throw("Unable to initialize wxWidgets");
-      GDLWidget::SetWxStarted();
-    }
+  BaseGDL* dialog_pickfile_wxwidgets(EnvT* e) {
     /*
       results = DIALOG_PICKFILE_WXWIDGETS(DEFAULT_EXTENSION=default_extension, $
       DIRECTORY=directory, DIALOG_PARENT=dialog_parent, $
@@ -53,6 +44,10 @@ namespace lib {
       TITLE=title)
     */
 
+#ifdef HAVE_LOCALE_H
+    setlocale(LC_ALL, "C");
+#endif
+
     bool isdefault_extension = false;
     bool isdirectory = false;
     bool isdialog_parent = false;
@@ -335,18 +330,22 @@ namespace lib {
 	      if ((*basenamestrgdl)[0].find(".") == string::npos)
 		(*res)[i] += "." + defaultextstr;
 	    }
+#ifdef _WIN32 // if posixpaths desired, convert them here.
+    if (lib::posixpaths) {
+        char * ptr = (char *) (*res)[i].c_str();
+        for(int i=0;ptr[i] != 0;i++) if(ptr[i] == '\\') ptr[i] = '/';
+    }
+#endif          
 	  }
       }
     return res;
   }
 
-  BaseGDL* dialog_message_wxwidgets(EnvT* e)
-  {
-    
-    if (!GDLWidget::wxIsStarted()){
-      if( ! wxInitialize( ) ) e->Throw("Unable to initialize wxWidgets");
-      GDLWidget::SetWxStarted();
-    }
+  BaseGDL* dialog_message_wxwidgets(EnvT* e) {
+
+#ifdef HAVE_LOCALE_H
+    setlocale(LC_ALL, "C");
+#endif
 
     DStringGDL* messagestr;
     bool iscancel = false;
@@ -477,7 +476,7 @@ namespace lib {
     else                         return new DStringGDL("Cancel");
   }
 #else
-  BaseGDL* wxwidgets_exists(EnvT* e) { return new DLongGDL(0); };
+  BaseGDL* wxwidgets_exists(EnvT* e) { return new DIntGDL(0); };
   BaseGDL* dialog_pickfile_wxwidgets(EnvT* e) { ThrowGDLException("wxWidgets is not available!"); return 0; }
   BaseGDL* dialog_message_wxwidgets(EnvT* e) { ThrowGDLException("wxWidgets is not available!"); return 0; }
 #endif
