@@ -32,6 +32,7 @@
 #include "dinterpreter.hpp"
 #include "gdljournal.hpp"
 #include "gdleventhandler.hpp"
+#include "basic_pro_jmg.hpp"
 
 #ifdef USE_MPI
 #include "mpi.h"
@@ -58,12 +59,11 @@ bool historyIntialized = false;
 // instantiation of static data
 GDLInterpreter::HeapT     GDLInterpreter::heap; 
 GDLInterpreter::ObjHeapT  GDLInterpreter::objHeap; 
-SizeT                     GDLInterpreter::objHeapIx;
 SizeT                     GDLInterpreter::heapIx;
-EnvStackT             GDLInterpreter::callStack;
-DLong                   GDLInterpreter::stepCount;
-
-ProgNode GDLInterpreter::NULLProgNode;
+EnvStackT                 GDLInterpreter::callStack;
+DLong                     GDLInterpreter::stepCount;
+bool                      GDLInterpreter::noInteractive; // To exit on error or stop in line execution mode (gdl -e do_something)
+ProgNode                  GDLInterpreter::NULLProgNode;
 ProgNodeP GDLInterpreter::NULLProgNodeP = &GDLInterpreter::NULLProgNode;
 
 void LibInit(); // defined in libinit.cpp
@@ -75,7 +75,6 @@ DInterpreter::DInterpreter(): GDLInterpreter()
   //    heap.push_back(NULL); // init heap index 0 (used as NULL ptr)
   //    objHeap.push_back(NULL); // init heap index 0 (used as NULL ptr)
   interruptEnable = true;
-  objHeapIx=1; // map version (0 is NULL ptr)
   heapIx=1;    // map version (0 is NULL ptr)
   returnValue  = NULL;
   returnValueL = NULL;
@@ -464,31 +463,80 @@ int GDLInterpreter::GetFunIx( const string& subName)
   return funIx;
 }
 
-void GDLInterpreter::SetProIx( ProgNodeP f, bool throwImmediately)
+void GDLInterpreter::SetProIx( ProgNodeP f)
 {
   if( f->proIx == -1)
-    f->proIx=GetProIx(f, throwImmediately);//->getText());
+    f->proIx=GetProIx(f);//->getText());
 }
 
-int GDLInterpreter::GetProIx( ProgNodeP f, bool throwImmediately)
+int GDLInterpreter::GetProIx(ProgNodeP f)
 {
   string subName = f->getText();
-  int proIx=ProIx(subName);
-  if( proIx == -1)
-    {
-      // trigger reading/compiling of source file
-      /*bool found=*/ SearchCompilePro(subName, true);
-	  
-      proIx=ProIx(subName);
-    if (throwImmediately){  
-      if( proIx == -1)
-	{
-	  throw GDLException(f,"Procedure not found: "+subName,true,false);
-	}
+  int proIx = ProIx(subName);
+  if (proIx == -1) {
+    // trigger reading/compiling of source file
+    /*bool found=*/ SearchCompilePro(subName, true);
+
+    proIx = ProIx(subName);
+
+    //eliminate the simple case
+    if (proIx != -1) return proIx;
+#ifdef 	AUTO_PRINT_EXPR
+    //noInteractive: throw
+    if (noInteractive) throw GDLException(f, "Procedure not found: " + subName, true, false);
+    //attempts an implied print. All this should be done in the ANTLR stuff of course.
+    //We are here because the text is interpreted as a procedure. It is not (otherwise it would have been found),
+    //but it could just be one or a series of variable names, such as in "a=dist(3) & b=findgen(2) & a,b"
+    //If it is not a variable, doing "print,a" will produce an error "PRINT: Variable is undefined: A". This is not what
+    //we want, we want "% Procedure not found: A". So we "print" only if all the names are indeed bona fide variables.
+    //gather types of siblings. if they are not all "ref", do not tempt anything
+    EnvStackT& callStack = ProgNode::interpreter->CallStack();
+    DLong curlevnum = callStack.size();
+    if (curlevnum > 1) throw GDLException(f, "Procedure not found: " + subName, true, false);
+    DSubUD* pro = static_cast<DSubUD*> (callStack[curlevnum - 1]->GetPro());
+    bool ok = true;
+    ProgNodeP test = f;
+    std::string what = test->getText();
+    int xI = pro->FindVar(what);
+    if (xI == -1) {
+      BaseGDL** varPtr = pro->GetCommonVarPtr(what);
+      if (varPtr == NULL) ok = false;
     }
+    while (ok) {
+      test = test->GetNextSibling();
+      if (!test) break;
+      string type = test->getText();
+      string varName = test->GetFirstChild()->getText();
+      ok = (type == "ref"); //only simple variables.
+      if (ok) { //test it is a REAL variable.
+        xI = pro->FindVar(varName);
+        if (xI == -1) {
+          BaseGDL** varPtr = pro->GetCommonVarPtr(varName);
+          if (varPtr == NULL) ok = false;
+        }
+      }
+      if (!ok) break;
+      what += "," + test->GetFirstChild()->getText(); //for "ref" firstChild contains the name to be printed
     }
+    if (ok) { //only simple things like "a,b,c"
+      try {
+        ProgNode::interpreter->executeLine.clear(); // clear EOF (for executeLine)
+        ProgNode::interpreter->executeLine.str("print,/implied_print," + what);
+        std::istream execute_me(ProgNode::interpreter->executeLine.rdbuf());
+        ProgNode::interpreter->ExecuteLine(&execute_me, 0);
+        ProgNode::interpreter->SetRetTree(f->GetLastSibling()->GetNextSibling());
+        return proIx;
+      } catch (GDLException& e) {
+        throw GDLException(f, "Procedure not found: " + subName, true, false);
+      }
+    } else throw GDLException(f, "Procedure not found: " + subName, true, false);
+#else      
+      throw GDLException(f, "Procedure not found: " + subName, true, false);
+#endif    
+  }
   return proIx;
 }
+
 int GDLInterpreter::GetProIx( const string& subName)
 {
   int proIx=ProIx(subName);
@@ -1296,7 +1344,7 @@ char* DInterpreter::NoReadline( const string& prompt)
           th.join();
           return NULL;
         }
-#ifdef WIN32
+#ifdef _WIN32
         Sleep(10);
 #else
         usleep(10);
@@ -1329,7 +1377,7 @@ string DInterpreter::GetLine()
 {
   clog << flush; cout << flush;
 
-#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
+#if defined(HAVE_LIBREADLINE)
   int edit_input = SysVar::Edit_Input() && isatty(0);
 #endif
 
@@ -1342,7 +1390,7 @@ string DInterpreter::GetLine()
 
     lineEdit = true;
 
-#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
+#if defined(HAVE_LIBREADLINE)
     
     if( edit_input != 0)
       cline = readline(const_cast<char*>(actualPrompt.c_str()));
@@ -1378,7 +1426,7 @@ string DInterpreter::GetLine()
   } while( line == "" 
 	|| line[0] == ';'); // skip also comment lines (bug #663)
   
-#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
+#if defined(HAVE_LIBREADLINE)
   // SA: commented out to comply with IDL behaviour- allowing to 
   //     set the history-file length only in the startup file
   //if( edit_input > 20)
@@ -1535,8 +1583,10 @@ void DInterpreter::ExecuteFile( const string& file)
 	}
       catch( RetAllException& retAllEx)
 	{
-	  runCmd = (retAllEx.Code() == RetAllException::RUN);
-	  if( !runCmd) throw;
+        
+        //Do nothing, as we are at Main. A RETALL or RESET etc shoul not be harmful. 
+//	  runCmd = (retAllEx.Code() == RetAllException::RUN );
+//	  if( !runCmd) throw;
 	}
       //       catch( exception& e)
       // 	{
@@ -1645,6 +1695,7 @@ RetCode DInterpreter::InterpreterLoop(const string& startup,
   if (statement.length() > 0) {
     // execute single statement and exit (a new-line is added to statement in gdl.cpp)
     // (e.g. $ gdl -e "print, 'hello world'")
+    noInteractive=true;
     istringstream iss(statement, ios_base::out);
     try {
       ExecuteLine(&iss);
@@ -1659,13 +1710,14 @@ RetCode DInterpreter::InterpreterLoop(const string& startup,
     batch_files.clear(); // not needed anymore...
   }
 
-#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
+#if defined(HAVE_LIBREADLINE)
 
   // initialize readline (own version - not pythons one)
   // in includefirst.hpp readline is disabled for python_module
   // http://www.delorie.com/gnu/docs/readline/rlman.html
   char rlName[] = "GDL";
   rl_readline_name = rlName;
+  rl_outstream = stderr;
   //Our handler takes too long
   //when editing the command line with ARROW keys. (bug 562). (used also in gdl.cpp)
   //but... without it we have no graphics event handler! FIXME!!!
@@ -1687,10 +1739,10 @@ RetCode DInterpreter::InterpreterLoop(const string& startup,
   if (homeDir != NULL) {
     string pathToGDL_history;
     pathToGDL_history = homeDir;
-    AppendIfNeeded(pathToGDL_history, "/");
+    AppendIfNeeded(pathToGDL_history, lib::PathSeparator());
     pathToGDL_history = pathToGDL_history + ".gdl";
     string history_filename;
-    AppendIfNeeded(pathToGDL_history, "/");
+    AppendIfNeeded(pathToGDL_history, lib::PathSeparator());
     history_filename = pathToGDL_history + "history";
     if (debug) cout << "History file name: " << history_filename << endl;
 
@@ -1760,6 +1812,7 @@ RetCode DInterpreter::InterpreterLoop(const string& startup,
         ResetObjects();
         ResetHeap();
         if (fullResetCmd) {
+          lib::ResetDLLs();
           PurgeContainer(libFunList);
           PurgeContainer(libProList);
         }
@@ -1823,7 +1876,9 @@ RetCode DInterpreter::InterpreterLoop(const string& startup,
       }
     }    catch (exception& e) {
       cerr << "InterpreterLoop: Exception: " << e.what() << endl;
-    }    catch (...) {
+    }    catch (GDLException &e ) {
+      Warning(e.getMessage());
+    }   catch (...) {
       cerr << "InterpreterLoop: Unhandled Error." << endl;
     }
   }
