@@ -243,8 +243,45 @@ public:
     out=number;
 
     return noOverflow;
-  } 
+  }
+  
+  bool Text2Number( DByte& out, int base)
+  {
+    bool noOverflow = true;
 
+    DByte number=0;
+
+    for(unsigned i=0; i < text.size(); ++i)
+      {
+	char c=text[i];
+	if( c >= '0' && c <= '9')
+	  {
+	    c -= '0';
+	  }
+	else if( c >= 'a' &&  c <= 'f')
+	  {
+	    c -= 'a'-10;
+	  }
+	else 
+	  {
+	    c -= 'A'-10;
+	  }
+
+	DInt newNumber = base * number + c;
+
+	// check for overflow
+	if( newNumber > 255)
+	  {
+	    noOverflow = false;
+	  }
+
+	number=newNumber;
+      }
+    out=number;
+
+    return noOverflow;
+  }
+  
   void Text2Byte(int base);
   // promote: use Long if number to large
   void Text2Int(int base, bool promote=false);
