@@ -59,7 +59,13 @@ DStructDesc* FindInStructList(StructListT v, const string& s)
   if( f == v.end()) return NULL;
   return *f;
 }
-
+DStructDesc* FindObjectInStructList(StructListT v, const string& s)
+{
+  StructListT::iterator f=find_if(v.begin(),v.end(),DStruct_eq(s));
+  if( f == v.end()) return NULL;
+  if (((*f)->FunList().size() + (*f)->ProList().size()) == 0) return NULL;
+  return *f;
+}
   bool DStructBase::ContainsStringPtrObject()
   {
    for( SizeT t=0; t<tags.size(); ++t)
@@ -77,12 +83,13 @@ DStructDesc* FindInStructList(StructListT v, const string& s)
 
 void DUStructDesc::AddTag( const string& tagName, const BaseGDL* data)
 {
+  string TN = StrUpCase( tagName);  // prevent non-capitalized chars.
   for( SizeT i=0; i < tNames.size(); i++)
-    if( tNames[i] == tagName)
-      throw GDLException(tagName+" is already defined "
+    if( tNames[i] == TN)
+      throw GDLException(TN+" is already defined "
 			 "with a conflicting definition");
   
-  tNames.push_back(tagName);
+  tNames.push_back(  TN);
   Add( data->GetTag());
 }
 
@@ -100,7 +107,18 @@ void DStructDesc::AddParent( DStructDesc* p)
   }
 }
 
-// copy appropiate member subroutines from fun and pro lists
+void DStructDesc::AddParentListOnly( DStructDesc* p)
+{
+  parent.push_back(p);
+  OperatorList* parentOperatorList = p->GetOperatorList(); 
+  if( parentOperatorList != NULL)
+  {
+    assert( this->operatorList == NULL); // GDL_OBJECT can only be inherited once
+    operatorList = new OperatorList(*parentOperatorList);
+  }
+}
+
+// copy appropriate member subroutines from fun and pro lists
 void DStructDesc::SetupOperators()
 {
   assert( this->operatorList != NULL);
@@ -159,9 +177,9 @@ void DStructDesc::AssureIdentical( DStructDesc* d)
       if( tags[i]->Type() == GDL_STRUCT)
 	{
 	  SpDStruct* castLeft= 
-	    dynamic_cast<SpDStruct*>(tags[i]);
+	    static_cast<SpDStruct*>(tags[i]);
 	  SpDStruct* castRight= 
-	    dynamic_cast<SpDStruct*>(d->tags[i]);
+	    static_cast<SpDStruct*>(d->tags[i]);
 	  DStructDesc* leftD=castLeft->Desc();
 	  DStructDesc* rightD=castRight->Desc();
 	  
@@ -203,9 +221,9 @@ bool operator==(const DStructDesc& left,
       if( left.tags[i]->Type() == GDL_STRUCT)
 	{
 	  SpDStruct* castLeft= 
-	    dynamic_cast<SpDStruct*>(left.tags[i]);
+	    static_cast<SpDStruct*>(left.tags[i]);
 	  SpDStruct* castRight= 
-	    dynamic_cast<SpDStruct*>(right.tags[i]);
+	    static_cast<SpDStruct*>(right.tags[i]);
 	  DStructDesc* leftD=castLeft->Desc();
 	  DStructDesc* rightD=castRight->Desc();
 	  
