@@ -45,16 +45,38 @@ protected:
   // and DStringGDL (considers actual string sizes)
   SizeT nBytes = tags.back()->NBytes();
 
-  // alignment 
+//  2-May-2019 GVJ
+//  Align data in structures according to data lengthin accord with IDL).
+/********************
 #ifdef USE_EIGEN
+ (This forced all data to be aligned on 16-bytes.  There was no perceptible perfomance gain,
+ nor was any penalty found for simply using 2-byte alignment.
+
+  const int alignmentInBytes = 2; // there was no performance difference.
+
   assert( sizeof( char*) <= 16); 
   const int alignmentInBytes = 16; // set to multiple of 16 >= sizeof( char*)
 #else
   const int alignmentInBytes = sizeof( char*);
 #endif
+******************/
+
+  const int alignmentInBytes = sizeof( char*);
+
+// The old way:
+/******
   SizeT exceed = nBytes % alignmentInBytes;
   if( exceed > 0)
 	nBytes += alignmentInBytes - exceed;
+*****/
+// replacement:
+  SizeT Align = (nBytes < alignmentInBytes)? nBytes: alignmentInBytes;
+  SizeT initOffset = tagOffset.back();
+  SizeT Oddbytes = initOffset % Align;
+  if(Oddbytes > 0) { 
+	  tagOffset.pop_back();
+	  tagOffset.push_back( initOffset + (Align-Oddbytes));
+  }
 
   // valid tagOffset (used by NBytes())
   tagOffset.push_back( tagOffset.back() + nBytes);
@@ -242,14 +264,86 @@ public:
   }
 
   void AddParent( DStructDesc*);
+  void AddParentListOnly( DStructDesc*); //for restore
 
-  void GetParentNames( std::vector< std::string>& pNames) const
-  {
-    SizeT nParents=parent.size();
-    for( SizeT i=0; i<nParents; ++i)
-      {
-	pNames.push_back( parent[i]->Name());
-      }
+  /**
+  *  @brief  fills a std::vector of strings with all the object's direct ancestors names
+  *  @param  pNames the std::vector of strings returned filled.
+  *
+  *  fills the @pNames std::vector of strings with all the object's direct ancestors names
+  */
+  void GetParentNames(std::vector< std::string>& pNames) const {
+   SizeT nParents = parent.size();
+   for (SizeT i = 0; i < nParents; ++i) {
+    pNames.push_back(parent[i]->Name());
+   }
+ }
+   /**
+  *  @brief  fills a std::set of strings with all the object's direct ancestors names
+  *  @param  pNames the std::set of strings returned filled.
+  *
+  *  fills the @pNames std::set (ordered) of strings with all the object's direct ancestors names
+  */
+  void GetParentNames(std::set< std::string>& pNames) const {
+   SizeT nParents = parent.size();
+   for (SizeT i = 0; i < nParents; ++i) {
+    pNames.insert(parent[i]->Name());
+   }
+ }
+  /**
+  *  @brief  fills a std::vector of strings with all the object's ancestors names (direct, inherited)
+  *  @param  aaNames the vector of strings returned filled.
+  *
+  *  fills the @aaNames vector of strings with all the object's parents names
+  */  
+  void GetAllAncestorsNames(std::vector< std::string>& aaNames) const {
+   SizeT nParents = parent.size();
+   for (SizeT i = 0; i < nParents; ++i) {
+    aaNames.push_back(parent[i]->Name());
+   }
+   //parents of parents...
+   for (SizeT i = 0; i < nParents; ++i) {
+      parent[i]->GetAllAncestorsNames(aaNames);
+   }
+   //remove dupes?
+  }
+  
+ /**
+  *  @brief  fills a std::set of strings with all the object's ancestors names (direct, inherited)
+  *  @param  aaNames the vector of strings returned filled.
+  *
+  *  fills the @aaNames vector of strings with all the object's parents names
+  */  
+  void GetAllAncestorsNames(std::set< std::string>& aaNames) const {
+   SizeT nParents = parent.size();
+   for (SizeT i = 0; i < nParents; ++i) {
+    aaNames.insert(parent[i]->Name());
+   }
+   //parents of parents...
+   for (SizeT i = 0; i < nParents; ++i) {
+      parent[i]->GetAllAncestorsNames(aaNames);
+   }
+  }
+  /**
+  *  @brief  fills a std::set of strings with all the object's non-direct ancestors
+  *  @param  aNames the vector of strings returned filled.
+  *
+  *  fills the @aNames std::set of strings with all the object's non-direct ancestors
+  */  
+  void GetAncestorsNames(std::set< std::string>& aNames) const {
+   std::vector<DStructDesc*>direct;
+   SizeT nParents = parent.size();
+   for (SizeT i = 0; i < nParents; ++i) {
+    direct.push_back(parent[i]);
+   }
+   //parents of parents...
+   for (SizeT i = 0; i < nParents; ++i) {
+      direct[i]->GetAllAncestorsNames(aNames);
+   }
+  }
+  
+  int GetNumberOfParents(){
+   return parent.size();
   }
 
   void AssureIdentical( DStructDesc* d);
@@ -278,6 +372,7 @@ public:
 };
 
 DStructDesc* FindInStructList(StructListT, const std::string& name);
+DStructDesc* FindObjectInStructList(StructListT, const std::string& name);
 
 // actually holding the data **********************************************
 // class DStructGDL see dstructgdl.hpp
