@@ -86,13 +86,13 @@ namespace lib {
 #endif
   }
 
-  BaseGDL* gshhg_exists( EnvT* e )
+  BaseGDL* shapelib_exists( EnvT* e )
   {
-#ifdef USE_GSHHS
-    //    e->Message( "GDL was compiled with support for GSHHG" );
+#ifdef USE_SHAPELIB
+    //    e->Message( "GDL was compiled with support for Shapefile format." );
     return new DIntGDL(1);
 #else
-    //e->Message( "GDL was compiled without support for GSHHG" );
+    //e->Message( "GDL was compiled without support for Shapefile format." );
     return new DIntGDL(0);
 #endif
   }
@@ -160,33 +160,15 @@ namespace lib {
 #endif
   }
 
-  BaseGDL* proj4_exists( EnvT* e )
+  BaseGDL* proj_exists( EnvT* e )
   {
-#if defined(USE_LIBPROJ4)
+#if defined(USE_LIBPROJ)
     return new DIntGDL(1);
 #else
     return new DIntGDL(0);
 #endif
   }
   
-  BaseGDL* proj4new_exists( EnvT* e )
-  {
-#if defined(USE_LIBPROJ4_NEW)
-    return new DIntGDL(1);
-#else
-    return new DIntGDL(0);
-#endif
-  }
-
-  BaseGDL* pslib_exists( EnvT* e )
-  {
-#if defined(USE_PSLIB)
-    return new DIntGDL(1);
-#else
-    return new DIntGDL(0);
-#endif
-  }
-
   BaseGDL* python_exists( EnvT* e )
   {
 #if defined(USE_PYTHON)
@@ -225,7 +207,7 @@ namespace lib {
 
   BaseGDL* x11_exists( EnvT* e )
   {
-#if defined(USE_X)
+#if defined(HAVE_X)
     return new DIntGDL(1);
 #else
     return new DIntGDL(0);
@@ -234,12 +216,29 @@ namespace lib {
   
   BaseGDL* dsfmt_exists(EnvT* e)
   {
-    //dSFMT (random generator) is used AT THE MOMENT only if Eigen is present, and only if useDSFMTAcceleration == true
+    //dSFMT (random generator) is now used by default if Eigen:: but also can be controlled by -noDSFMT .
 #ifdef USE_EIGEN
     return new DIntGDL(useDSFMTAcceleration == true);
 #else
     return new DIntGDL(0);
 #endif
   }
+  BaseGDL* expat_exists(EnvT* e)
+  {
+#ifdef USE_EXPAT
+    return new DIntGDL(1);
+#else
+    return new DIntGDL(0);
+#endif
+  }
+
+  BaseGDL* mpi_exists(EnvT* e)
+  {
+#ifdef USE_MPI
+    return new DIntGDL(1);
+#else
+    return new DIntGDL(0);
+#endif
+  }  
 }
  // namespace
