@@ -24,7 +24,7 @@ namespace lib {
  BaseGDL* fftw_exists(EnvT *e);
  BaseGDL* glpk_exists(EnvT *e);
  BaseGDL* grib_exists(EnvT *e);
- BaseGDL* gshhg_exists(EnvT *e);
+ BaseGDL* shapelib_exists(EnvT *e);
  BaseGDL* hdf5_exists(EnvT *e);
  BaseGDL* hdf_exists(EnvT *e);
  BaseGDL* magick_exists(EnvT *e);
@@ -32,8 +32,7 @@ namespace lib {
  BaseGDL* ncdf4_exists(EnvT *e);
  BaseGDL* openmp_exists(EnvT *e);
  BaseGDL* pnglib_exists(EnvT *e);
- BaseGDL* proj4_exists(EnvT *e);
- BaseGDL* proj4new_exists(EnvT *e);
+ BaseGDL* proj_exists(EnvT *e);
  BaseGDL* pslib_exists(EnvT *e);
  BaseGDL* python_exists(EnvT *e);
  BaseGDL* tiff_exists(EnvT *e);
@@ -42,6 +41,8 @@ namespace lib {
   // only exception wxwidgets_exists in "dialop.hpp"
  BaseGDL* x11_exists(EnvT *e);
  BaseGDL* dsfmt_exists(EnvT *e);
+ BaseGDL* expat_exists(EnvT *e);
+ BaseGDL* mpi_exists(EnvT* e);
 
 } // namespace
 
