@@ -190,23 +190,13 @@ void GDLGStream::DefaultCharSize() {
   int Y_CH_SIZE = s->TagIndex("Y_CH_SIZE");
   int X_PX_CM = s->TagIndex("X_PX_CM");
   int Y_PX_CM = s->TagIndex("Y_PX_CM");
-  //  int NAME = s->TagIndex("NAME");
-  //  DString name = (*static_cast<DStringGDL*>(d->GetTag(NAME, 0)))[0];
-  ////values must be those that plplot think are good. Most of the time they are not.
-  //  if (name == "PS" || name=="SVG") schr( 2.5, 1.0, 2.5);
-  //  else {
-  //#if defined(_WIN32)
-  //    schr(2.1, 1.4, 2.1);  // from 1.5, 1.0 2014/09/18 //This is a feature of windows --- or of windows plplot -- to be confirmed.
-  //#else
   DLong chx = (*static_cast<DLongGDL*> (d->GetTag(X_CH_SIZE, 0)))[0];
   DLong chy = (*static_cast<DLongGDL*> (d->GetTag(Y_CH_SIZE, 0)))[0];
   DFloat xpxcm = (*static_cast<DFloatGDL*> (d->GetTag(X_PX_CM, 0)))[0];
   DFloat ypxcm = (*static_cast<DFloatGDL*> (d->GetTag(Y_PX_CM, 0)))[0];
-  DFloat xchsizemm = chx * CM_IN_MM / xpxcm;
-  DFloat linespacingmm = chy * CM_IN_MM / ypxcm;
+  DFloat xchsizemm = GetPlplotFudge() * chx * CM_IN_MM / xpxcm;
+  DFloat linespacingmm = GetPlplotFudge() * chy * CM_IN_MM / ypxcm;
   schr(xchsizemm, 1.0, linespacingmm);
-  //#endif
-  //  }
 }
   void GDLGStream::RenewPlplotDefaultCharsize(PLFLT newMmSize)
   {
@@ -225,6 +215,7 @@ void GDLGStream::DefaultCharSize() {
     theDefaultChar.scale=1.0;
     theDefaultChar.mmsx=pls->chrht; //millimeter
     theDefaultChar.mmsy=pls->chrht;
+    theDefaultChar.fudge=GetPlplotFudge();
     theDefaultChar.ndsx=mm2ndx(theDefaultChar.mmsx); //normalized device
     theDefaultChar.ndsy=mm2ndy(theDefaultChar.mmsy);
     theDefaultChar.dsy=theDefaultChar.ndsy*thePage.height;
@@ -341,8 +332,8 @@ void GDLGStream::GetGeometry( long& xSize, long& ySize)
     xSize = (*static_cast<DLongGDL*>(SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("X_SIZE"), 0)))[0];
     ySize = (*static_cast<DLongGDL*>(SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("Y_SIZE"), 0)))[0];
   } else {
-    xSize = xleng;
-    ySize = yleng;
+  xSize = xleng;
+  ySize = yleng;
   }
   if (xSize<1.0||ySize<1) //plplot gives back crazy values! z-buffer for example!
   {
@@ -360,6 +351,8 @@ void GDLGStream::GetGeometry( long& xSize, long& ySize)
 {
   bool debug = false;
   static char errmsg[] = "No such font:   ";
+  static std::string begin_unicode="#[";
+  static std::string end_unicode="]";
   static const size_t errmsglen = strlen(errmsg);
   static double fact[]={1.,0.9,0.666,0.5,0.45,0.33,0.2};
   double base=1.0;
@@ -679,8 +672,8 @@ void GDLGStream::GetGeometry( long& xSize, long& ySize)
             case 'C' : // tick sign
               out += " "; break;
               break;
-            default : 
-              out.append(in, i, 1);
+            default :
+              if ((unsigned char)in[i] > 127) { out+=begin_unicode; out+=to_string((unsigned char)in[i]); out+=end_unicode; } else out.append(in, i, 1); //use unicode
               break;
           }
           break;
@@ -784,7 +777,7 @@ void GDLGStream::GetGeometry( long& xSize, long& ySize)
             case 'z' : out += "#(2305)"; break; // ?
             case 'm' : out += "#(899)";  break; // smallest circle - dot
             default : 
-              out.append(in, i, 1);
+              if ((unsigned char)in[i] > 127) { out+=begin_unicode; out+=to_string((unsigned char)in[i]); out+=end_unicode; } else out.append(in, i, 1); //use unicode
               break;
           }
           break;
@@ -824,7 +817,7 @@ void GDLGStream::GetGeometry( long& xSize, long& ySize)
             case '"' : out += "#(2831)"; break; case '^' : case '~' : out += "#(2931)"; break; // [yu]
             case 127u : case '_' : out += "#(2832)"; break; case ';' : out += "#(2932)"; break; // [ya]
             default : 
-              out.append(in, i, 1);
+              if ((unsigned char)in[i] > 127) { out+=begin_unicode; out+=to_string((unsigned char)in[i]); out+=end_unicode; } else out.append(in, i, 1); //use unicode
               break;
           }
           break;
@@ -860,9 +853,9 @@ void GDLGStream::GetGeometry( long& xSize, long& ySize)
             case 'I' : case 'i' :
             case 'O' : case 'o' :
             case 'P' : case 'p' :
-              out += "#g";
+              out += "#g"; out.append(in, i, 1); break;
             default : 
-              out.append(in, i, 1);
+              if ((unsigned char)in[i] > 127) { out+=begin_unicode; out+=to_string((unsigned char)in[i]); out+=end_unicode; } else out.append(in, i, 1); //use unicode
               break;
           }
           break;
@@ -886,13 +879,13 @@ void GDLGStream::GetGeometry( long& xSize, long& ySize)
             case 190u : out += "#(271)"; break; // 3/4
             case 215u : out += "#(846)"; break; // cross sign
             case 223u : out += "#fs#gb"; out += internalFontCodes[curr_fnt]; break; // beta script
-            default : 
-              out.append(in, i, 1);
+            default :
+              if ((unsigned char)in[i] > 127) { out+=begin_unicode; out+=to_string((unsigned char)in[i]); out+=end_unicode; } else out.append(in, i, 1); //use unicode
               break;
           }
           break;
         default : // simply pass the char
-          out.append(in, i, 1);
+          if ((unsigned char)in[i] > 127) { out+=begin_unicode; out+=to_string((unsigned char)in[i]); out+=end_unicode; } else out.append(in, i, 1); //use unicode
           break;
       }
       curr_fnt = next_fnt;
@@ -907,7 +900,7 @@ void GDLGStream::GetGeometry( long& xSize, long& ySize)
 retrn:
   activeFontCodeNum = curr_fnt;
   if (stringLength) *stringLength=0;
-  if (debug) cout << "GDLGStream::TranslateFormatCodes(\"" << in << "\") = \"" << out << "\"" << endl;  
+  cout << "ERROR: GDLGStream::TranslateFormatCodes(\"" << in << "\") = \"" << out << "\"" << endl;  
   return ""; 
 }
 
@@ -986,6 +979,9 @@ void GDLGStream::vpor(PLFLT xmin, PLFLT xmax, PLFLT ymin, PLFLT ymax )
 void GDLGStream::wind( PLFLT xmin, PLFLT xmax, PLFLT ymin, PLFLT ymax )
 {
   if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"wind(): setting x[%f:%f],y[%f:%f] (world) \n",xmin,xmax,ymin,ymax);
+  //silly test to protect against plplot warnings ... side effects unkonwn.
+  if (xmin==xmax) {xmin=0; xmax=1;}
+  if (ymin==ymax) {ymin=0; ymax=1;}
   plstream::wind(xmin, xmax, ymin, ymax);
   theBox.wx1=xmin;
   theBox.wx2=xmax;
