@@ -23,6 +23,9 @@
 // #define GDL_DEBUG_WIDGETS
 // #define GDL_DEBUG_WIDGETS_COLORIZE
 
+// use "plain menubars" instead of 'taskbars used as menubars'. taskbars permit to change font in bar, put pixmaps instead of text, and will work
+// on OSX. So we choose normally to undefine this 
+// #define PREFERS_MENUBAR 1
 #include <wx/wx.h>
 #include <wx/app.h>
 #include <wx/panel.h>
@@ -72,11 +75,14 @@
 #define DONOTALLOWSTRETCH 0
 #define ALLOWSTRETCH 1
 #define FRAME_ALLOWSTRETCH 1
+#define DEFAULT_TREE_IMAGE_SIZE 16
+
 #ifdef __WXMSW__
   #define NEWLINECHARSIZE 2  //length of <cr><nl>
 #else
   #define NEWLINECHARSIZE 1  //length of <nl> 
 #endif
+
 #define gdlSIZE_EVENT_HANDLER wxSizeEventHandler(gdlwxFrame::OnSizeWithTimer) //filter mouse events (manual resize) to avoid too many updtes for nothing
 #define gdlSIZE_IMMEDIATE_EVENT_HANDLER wxSizeEventHandler(gdlwxFrame::OnSize) 
 typedef DLong WidgetIDT;
@@ -85,7 +91,17 @@ static int    widgetTypeList[]={0,1,2,3,4,5,6,7,8,9,10,11,12,13,14};
 static bool handlersInited=false; //handlers of graphic formats for bitmaps (magick).
 
 enum { WINDOW_TIMER = -2*wxID_HIGHEST, RESIZE_TIMER, RESIZE_PLOT_TIMER }; //negative values, should not clash with our (positive) widget ids.
-
+enum { 
+  TREE_BITMAP_ITEM = 0,
+  TREE_BITMAP_ITEM_SELECTED = 0,
+  TREE_BITMAP_FOLDER,
+  TREE_BITMAP_FOLDER_OPEN,
+  TREE_BITMAP_END
+};
+enum { 
+  gdlWxTree_UNCHECKED = 0,
+  gdlWxTree_CHECKED
+};
 class DStructGDL;
 
 // thread safe deque
@@ -108,7 +124,7 @@ public:
     return front;
   }
   // for all regular events
-  void Push( DStructGDL* ev)
+  void PushBack( DStructGDL* ev)
   {
     dq.push_back( ev);
   }
@@ -198,7 +214,7 @@ public:
  WidgetEventInfo(wxEventType t_, wxObjectEventFunction f_, wxWindow* w_) : t(t_), f(f_), w(w_) {
  }
 };
-#if __WXMSW__ 
+#ifndef __WXMAC__
 // main App class
 class wxAppGDL: public wxApp
 {
@@ -216,7 +232,7 @@ public:
 //the event processing stops immediately considering that the event had been already processed
 //(for the former return value) or that it is not going to be processed at all (for the latter one).
 };
-//wxDECLARE_APP(wxAppGDL); //wxAppGDL is equivalent to wxGetApp()
+wxDECLARE_APP(wxAppGDL); //wxAppGDL is equivalent to wxGetApp()
 #endif
 
 // GDL versions of wxWidgets controls =======================================
@@ -260,7 +276,7 @@ public:
   event->SetEventObject(this);
   // only for wWidgets > 2.9 (takes ownership of event)
   //     this->QueueEvent( event);
-  //this->AddPendingEvent( *event); // copies event
+//  this->AddPendingEvent(*event); // copies event
   this->OnShowRequest(*event); // JP Apr 2015: Should block the main thread until the window opens,
   //              so that the following WIDGET_INFO can properly read
   //              the window's properties.
@@ -377,8 +393,6 @@ public:
   static void UnInit(); // global GUI desinitialization in case it is useful (?)
   static bool wxIsStarted(){return (wxIsOn);}
   static void SetWxStarted(){wxIsOn=true;}
-  static bool AreWxHandlersOk(){return (handlersOk);}
-  static void SetWxHandlersOk(){handlersOk=true;}
   static void UnsetWxStarted(){gdl_lastControlId=0;/* not possible: wxWidgets library does not survive wxUniitiailze() ... wxIsOn=false; handlersOk=false;*/}
   static int  GDLNewControlId(){
    gdl_lastControlId++;
@@ -578,7 +592,7 @@ public:
   bool IsValid(){return valid;}
   void SetUnValid(){valid=false;}
   void SetValid(){valid=true;}
-  bool IsDynamicResize(){return ((dynamicResize>0)|| (dynamicResize==0 && !GetRealized())); }
+  bool IsDynamicResize(){return ((dynamicResize>0)|| (dynamicResize==0 && !IsRealized())); }
   void SetDynamicResize(){if (dynamicResize > -1) dynamicResize=1;}
   void UnsetDynamicResize(){if (dynamicResize > -1) dynamicResize=0;}
   void authorizeDynamicResize(){dynamicResize=1;}
@@ -620,8 +634,13 @@ public:
   virtual DLong NChildren() const { return 0;}
   DLong GetSibling();
   virtual DLongGDL* GetChildrenList() const {return new DLongGDL(0);}
-  virtual void SetXmanagerActiveCommand() {}
-  virtual bool GetXmanagerActiveCommand() const { return false;}
+  
+  //returns a list of IDs of all the widgets starting at me and below.
+  DLongGDL* GetAllHeirs();
+  
+  virtual void SetXmanagerActiveCommand() {std::cerr<<"XMANAGER ACTIVE COMMAND on a not-top widget, please report."<<std::endl;}
+  
+  bool GetXmanagerActiveCommand();
 
   void SetEventPro( const DString& ePro) { eventPro = StrUpCase( ePro);}
   const DString& GetEventPro() const { return eventPro;};
@@ -652,11 +671,11 @@ public:
   wxScrolled<wxPanel>* GetPanel() { return widgetPanel;}
 
   bool GetManaged() const { return managed;}
-  bool GetRealized(); 
+  bool IsRealized(); 
   void SetManaged( bool manval){managed = manval;}
   virtual void SetSensitive( bool value);
   bool GetSensitive();
-  virtual DLong GetTheSibling(DLong){return 0;}
+  virtual DLong GetTheSiblingOf(DLong myIdx){return 0;}
   virtual void SetFocus();
 
   int  GetExclusiveMode() const { return exclusiveMode;}
@@ -697,35 +716,25 @@ public:
   bool DisableSizeEvents(gdlwxFrame* &tlbFrame,WidgetIDT &id);
   static void EnableSizeEvents(gdlwxFrame* &tlbFrame,WidgetIDT &id);
 
-  void SendWidgetTimerEvent(DDouble secs) {
-  WidgetIDT* id = new WidgetIDT(widgetID);
-  int millisecs = floor(secs * 1000.0);
-  if (theWxWidget) { //we nee a handle on a wxWindow object...
-   wxWindow* w=dynamic_cast<wxWindow*>(theWxWidget);
-   assert (w!=NULL);
-   w->GetEventHandler()->SetClientData(id);
-   if (m_windowTimer==NULL) {
-    m_windowTimer=new wxTimer(w->GetEventHandler(),widgetID);
-   }
-#ifdef GDL_DEBUG_WIDGETS
-   std::cerr<<"sending event,"<<widgetID<<","<<m_windowTimer<<std::endl;
-#endif
-   m_windowTimer->StartOnce(millisecs);
-  }
- }
+  void SendWidgetTimerEvent(DDouble secs);
+  virtual void SetButtonWidget( bool onOff){}
+  void ClearEvents();
 };
 
 class GDLWidgetContainer: public GDLWidget
 {
 protected:
   std::deque<WidgetIDT> children;
+  bool xfree;
+  bool yfree;
 public:
   GDLWidgetContainer( WidgetIDT parentID, EnvT* e, ULong eventFlags_=0);
 
   ~GDLWidgetContainer();
   
   virtual bool IsContainer() const { return true;}
-  
+  bool xFree() {return xfree;}
+  bool yFree() {return yfree;}
 //Realize a Container==> realise first all children. Vertically-stored widgets in a base widget must be reordered, done in overriding GDLWidgetBase::OnRealize.
   void OnRealize();
 
@@ -756,13 +765,13 @@ public:
     return ret;   
   }
   //returns next in list or 0
-  DLong GetTheSibling(DLong childIx)
+  DLong GetTheSiblingOf(DLong myIdx)
   {
     DLong size=children.size( );
     if (size<1) return 0;
     DLong ret=0;
     for (int i=0; i< size-1; ++i) {
-     if (children[i] == childIx) {
+     if (children[i] == myIdx) {
       ret=children[i+1];
       break;
      }
@@ -806,12 +815,7 @@ public:
   virtual bool IsNormalBase() const { return false;} 
   virtual bool IsTabbedBase() const { return false;} 
   void SetWidgetSize(DLong sizex, DLong sizey) final;
-  void ClearEvents()
-  {
-  if (!this->GetXmanagerActiveCommand( ))  eventQueue.Purge();
-  else readlineEventQueue.Purge(); 
-  }
-  
+ 
   void NullWxWidget() { theWxWidget = NULL;}
   
   WidgetIDT GetLastRadioSelection() const { return lastRadioSelection;}                         
@@ -875,7 +879,7 @@ public:
  bool IsTopBase() const final{
   return true;
  }
- bool IsRealized() { return realized; }
+ bool IsTopLevelRealized() { return realized; }
  gdlwxFrame* GetTopFrame() {
   return topFrame;
  }
@@ -982,6 +986,7 @@ public:
 class wxButtonGDL: public wxButton
 {
   wxMenu* popupMenu;
+  wxPoint position;
 public: 
   wxButtonGDL(wxFont font, wxWindow *parent, 
           wxWindowID id, 
@@ -994,6 +999,7 @@ public:
       wxButton(parent,id,label,pos,size,style,validator,name){
       this->SetFont(font);
       popupMenu=new wxMenu();
+      position=this->GetClientRect().GetBottomLeft();
       Connect(id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(wxButtonGDL::OnButton));
       Connect(id, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(wxButtonGDL::OnButton));
     }
@@ -1041,19 +1047,19 @@ public:
   ~GDLWidgetButton();
   // for WIDGET_CONTROL
 
-  void SetButtonWidget( bool onOff)
+  virtual void SetButtonWidget( bool onOff)
   {
     if( theWxWidget != NULL)
     {
       switch( buttonType) {
         case RADIO: {	  
-          SetButton( onOff);
+          SetRadioButton( onOff);
           wxRadioButton* radioButton = dynamic_cast<wxRadioButton*>(theWxWidget);
           radioButton->SetValue(onOff);
           break;
         }
         case CHECKBOX: {
-          SetButton( onOff);
+          SetRadioButton( onOff);
           wxCheckBox* checkBox = dynamic_cast<wxCheckBox*>(theWxWidget);
           checkBox->SetValue(onOff);
           break;
@@ -1064,7 +1070,7 @@ public:
   }
   virtual void SetButtonWidgetLabelText( const DString& value_ ) {std::cerr<<"SetButtonWidgetLabelText() ID="<<widgetID <<" error, please check!"<<std::endl;} ;//code in gdlwidget
   virtual void SetButtonWidgetBitmap( wxBitmap* bitmap_ );//code in gdlwidget
-  void SetButton( bool onOff)
+  void SetRadioButton( bool onOff)
   {
     buttonState = onOff;
   }
@@ -1072,7 +1078,6 @@ public:
   {
     return buttonState;
   }
-  
   bool IsButton() const final { return true;} 
   bool IsBitmapButton() const {return ( buttonType==POPUP_BITMAP || buttonType==BITMAP);}
  };
@@ -1115,13 +1120,13 @@ public:
     return ret;   
   }
   //returns next in list or 0
-  DLong GetTheSibling(DLong childIx)
+  DLong GetTheSiblingOf(DLong myIdx)
   {
     DLong size=children.size( );
     if (size<1) return 0;
     DLong ret=0;
     for (int i=0; i< size-1; ++i) {
-     if (children[i] == childIx) {
+     if (children[i] == myIdx) {
       ret=children[i+1];
       break;
      }
@@ -1161,8 +1166,8 @@ public:
 #else
  wxToolBarToolBase* entry;
  GDLWidgetMenuBarButton(WidgetIDT parentID, EnvT* e, DStringGDL* value, DULong eventflags, wxBitmap* bitmap_=NULL, DStringGDL* buttonTooltip = NULL);
- wxSize computeWidgetSize(); //not a real menubar: buttons may have a different fontsize.
 #endif
+ wxSize computeWidgetSize(); //not a real menubar: buttons may have a different fontsize.
  ~GDLWidgetMenuBarButton();
  void SetSensitive(bool value);
  void SetButtonWidgetLabelText( const DString& value_ );
@@ -1181,13 +1186,25 @@ class GDLWidgetMenuEntry: public GDLWidgetButton
 { 
   bool addSeparatorAbove;
   wxMenuItem* the_sep;
+  bool checkedState;
 public:
- GDLWidgetMenuEntry( WidgetIDT parentID, EnvT* e, DStringGDL* value, DULong eventflags, bool hasSeparatorAbove=false, wxBitmap* bitmap=NULL);
+ GDLWidgetMenuEntry( WidgetIDT parentID, EnvT* e, DStringGDL* value, DULong eventflags, bool hasSeparatorAbove=false, wxBitmap* bitmap=NULL, bool checked_type=false);
  ~GDLWidgetMenuEntry();
  bool IsEntry() const final {return true;}
  void SetSensitive(bool value);
  void SetButtonWidgetLabelText( const DString& value_ );
  void SetButtonWidgetBitmap( wxBitmap* bitmap );
+  bool GetButtonSet() const
+  {
+    return checkedState;
+  }
+  virtual void SetButtonWidget(bool onOff) final {
+    checkedState = onOff;
+    wxMenuItem* mi = static_cast<wxMenuItem*> (theWxWidget);
+    if (mi->GetKind() == wxITEM_CHECK || mi->GetKind() == wxITEM_RADIO) {
+      mi->Check(checkedState);
+    }
+  }
 };
 
 //specialized for Context Menu Base
@@ -1300,14 +1317,18 @@ class GDLWidgetText: public GDLWidget
   std::string lastValue;
   bool noNewLine;
   bool editable;
-  int maxlinelength;
+  int maxlinelength; //the size of the longest line in the widgets current value.
   int nlines;
   bool wrapped;
+  bool multiline;
+  wxSize textSize; //the current size, in characters, not owing for the character sizes and spurious decorations as wxWidgets adds by itself
+  wxSize initialSize; //memory of the initial (realized) textSize just after creation.
 public:
   GDLWidgetText( WidgetIDT parentID, EnvT* e, DStringGDL* value, DULong eventflags, bool noNewLine,
 		 bool editable);
   ~GDLWidgetText();
   
+  bool IsMultiline(){return multiline;}
   bool IsEditable(){return editable;}
   void SetEditable(bool v){ editable=v;}
   void ChangeText( DStringGDL* value, bool noNewLine=false);
@@ -1456,13 +1477,13 @@ public:
     return ret;   
   } 
   //returns next in list or 0
-  DLong GetTheSibling(DLong childIx)
+  DLong GetTheSiblingOf(DLong myIdx)
   {
     DLong size=children.size( );
     if (size<1) return 0;
     DLong ret=0;
     for (int i=0; i< size-1; ++i) {
-     if (children[i] == childIx) {
+     if (children[i] == myIdx) {
       ret=children[i+1];
       break;
      }
@@ -1626,18 +1647,22 @@ public:
 
 
 // tree widget **************************************************
-class wxTreeCtrlGDL: public wxTreeCtrl
-{  
-  wxWindowID GDLWidgetTableID;
+class wxTreeCtrlGDL: public wxTreeCtrl {
+  wxWindowID GDLWidgetTreeID;
+  wxWindowID draggedGDLWidgetID;
+  int KeyModifier; //Shift, Control... set during a drag
 public:
+
   wxTreeCtrlGDL(wxWindow *parent, wxWindowID id = wxID_ANY,
-               const wxPoint& pos = wxDefaultPosition,
-               const wxSize& size = wxDefaultSize,
-               long style = wxTR_DEFAULT_STYLE,
-               const wxValidator &validator = wxDefaultValidator,
-               const wxString& name = wxTreeCtrlNameStr)
+    const wxPoint& pos = wxDefaultPosition,
+    const wxSize& size = wxDefaultSize,
+    long style = wxTR_DEFAULT_STYLE,
+    const wxValidator &validator = wxDefaultValidator,
+    const wxString& name = wxTreeCtrlNameStr)
           :wxTreeCtrl( parent, id, pos, size, style, wxDefaultValidator , name ),
-          GDLWidgetTableID(id)
+          GDLWidgetTreeID(id),
+          draggedGDLWidgetID(0),
+          KeyModifier(0)
           {
 //            Connect(GDLWidgetTableID, wxEVT_COMMAND_TREE_ITEM_ACTIVATED, wxTreeEventHandler(wxTreeCtrlGDL::OnItemActivated));
 //            Connect(GDLWidgetTableID, wxEVT_COMMAND_TREE_ITEM_ACTIVATED,wxTreeEventHandler(wxTreeCtrlGDL::OnItemActivated));
@@ -1646,36 +1671,61 @@ public:
 //            Connect(GDLWidgetTableID, wxEVT_COMMAND_TREE_ITEM_COLLAPSED,wxTreeEventHandler(wxTreeCtrlGDL::OnItemCollapsed));
 //            Connect(GDLWidgetTableID, wxEVT_COMMAND_TREE_ITEM_EXPANDED,wxTreeEventHandler(wxTreeCtrlGDL::OnItemExpanded));
 //            Connect(GDLWidgetTableID, wxEVT_COMMAND_TREE_SEL_CHANGED,wxTreeEventHandler(wxTreeCtrlGDL::OnItemSelected));
-          }
-//necessary to define the destructor otherwise compiler will try to find the bind event table for destruction event!
+  }
+  //necessary to define the destructor otherwise compiler will try to find the bind event table for destruction event!
   ~wxTreeCtrlGDL(){}
-void OnItemActivated(wxTreeEvent & event);
-void OnItemCollapsed(wxTreeEvent & event);
-void OnItemExpanded(wxTreeEvent & event);
-void OnBeginDrag(wxTreeEvent & event);
-void OnItemDropped(wxTreeEvent & event);
-void OnItemSelected(wxTreeEvent & event);
+  void SetCurrentModifier(int x, bool remove){ //We have to remap the wxWidgets events
+    switch(x){
+      case WXK_SHIFT:
+        if (remove) KeyModifier &= ~(1); else KeyModifier |= 1; break; //bit 1 
+      case WXK_ALT:
+        if (remove) KeyModifier &= ~(1<<3); else KeyModifier |= (1<<3); break; //bit 4
+      case WXK_NUMLOCK:
+        if (remove) KeyModifier &= ~(1<<2); else KeyModifier |= (1<<2); break; //bit 3
+      case WXK_CONTROL:
+        if (remove) KeyModifier &= ~(1<<1); else KeyModifier |= (1<<1); break; //bit 2
+        
+    }
+  }
+  int GetCurrentModifier(){return KeyModifier;}
+  void OnTreeKeyDown(wxKeyEvent & event);
+  void OnTreeKeyUp(wxKeyEvent & event);
+  void OnItemActivated(wxTreeEvent & event);
+  void OnItemCollapsed(wxTreeEvent & event);
+  void OnItemStateClick(wxTreeEvent & event);
+  void OnItemExpanded(wxTreeEvent & event);
+  void OnDrag(wxTreeEvent & event);
+  void OnDrop(wxTreeEvent & event);
+  void OnItemSelected(wxTreeEvent & event);
+  void SetDragged(wxWindowID that) {draggedGDLWidgetID=that;}
+  wxWindowID GetDragged(){ return draggedGDLWidgetID;}
+  DECLARE_EVENT_TABLE()
 };
 
 class wxTreeItemDataGDL : public wxTreeItemData {
   public:
     WidgetIDT widgetID;
+    wxTreeCtrlGDL* myTree;
 
-    wxTreeItemDataGDL(WidgetIDT id) : widgetID(id) {}
+  wxTreeItemDataGDL(WidgetIDT id, wxTreeCtrlGDL* myTree_) : widgetID(id), myTree(myTree_) {}
+  WidgetIDT GetWidgetID(){return widgetID;}
+  wxTreeCtrlGDL* GetTree(){return myTree;}
 };
 
 class GDLWidgetTree: public GDLWidget
 {
-bool droppable  ; 
-bool draggable  ; 
+int droppable  ; 
+int draggable  ;
+bool has_checkbox;
 bool expanded;
 bool folder;
-int buttonImageId;
-int imageId;
-WidgetIDT selectedID;
+bool noBitmaps;
+bool mask;
+bool multiple; //can select multiple
 wxTreeItemId treeItemID;
 wxTreeItemDataGDL* treeItemData;
-WidgetIDT rootID;
+GDLWidgetTree* myRoot;
+DString dragNotify;
 
 public:
 GDLWidgetTree( WidgetIDT p, EnvT* e, BaseGDL* value_, DULong eventFlags
@@ -1685,26 +1735,60 @@ GDLWidgetTree( WidgetIDT p, EnvT* e, BaseGDL* value_, DULong eventFlags
 ,bool expanded_
 ,bool folder_
 ,DLong treeindex
+,DString &dragNotify_
 );
 		 
 ~GDLWidgetTree();
 
   bool IsTree() const final { return true;}
-  bool IsDraggable() {return draggable;}
-  bool IsDroppable() {return droppable;}
-  bool IsFolder() {return folder;}
-  bool IsExpanded() {return expanded;}
-  void DoExpand(){
-    wxTreeCtrlGDL * me = dynamic_cast<wxTreeCtrlGDL*>(theWxWidget);
-    if (me) me->Expand(treeItemID);
+  //DRAG
+  int GetDraggableValue() {return draggable;}
+  bool GetDragability();
+  void SetDragability(DLong val){draggable=val;}
+  //DRAGNOTIFY
+  DString GetDragNotifyValue(){return dragNotify;}
+  void SetDragNotify(DString &s){Warning("SET_DRAG_NOTIFY is currently ignored by GDL, FIXME."); dragNotify=s;}
+  //DROP
+  int  GetDroppableValue(){return droppable;}
+  bool GetDropability();
+  void SetDropability(DLong val){droppable=val;}
+  bool IsExpanded() {
+    return expanded;
   }
-  WidgetIDT GetRootID(){ return rootID;}
-  void SetSelectedID( WidgetIDT id){selectedID=id;}
-  WidgetIDT GetSelectedID(){ return selectedID;}
+  void SetExpanded(bool b){expanded=b;}
+
+  void DoExpand(bool what);
+  void Select(bool select);
+  void SetTreeIndex(DLong where);
+  GDLWidgetTree* GetMyRootGDLWidgetTree(){ return myRoot;}
+  WidgetIDT IsSelectedID();
+  WidgetIDT IsDragSelectedID();
+  DLongGDL* GetAllSelectedID();
+  DLongGDL* GetAllDragSelectedID();
   DInt GetTreeIndex();
   wxTreeItemId GetItemID(){ return treeItemID;}
+  void SetItemID( wxTreeItemId id){treeItemID=id;}
   void SetValue(DString val);
-};
+  void OnRealize();
+  void SetFolder(){folder=true;}
+  bool IsFolder(){return folder;}
+  void SetMask(bool b){mask=b;}
+  bool HasMask(){return mask;}
+  DByteGDL* ReturnBitmapAsBytes();
+  bool HasCheckBox(){return has_checkbox;}
+  void CheckItem(bool b){treeItemData->myTree->SetItemState(treeItemID,b);treeItemData->myTree->Refresh();}
+  bool IsUsingBitmaps(){return noBitmaps;}
+  void SetBitmap(wxBitmap* bitmap);
+  void SetVisible() {treeItemData->myTree->EnsureVisible(treeItemID);}
+  bool IsChecked(){if (has_checkbox) return treeItemData->myTree->GetItemState(treeItemID); else return false;
+  }
+
+  DLong NChildren() const;
+  WidgetIDT GetChild(DLong childIx) const final;
+  DLongGDL* GetChildrenList() const;
+  DLong GetTheSiblingOf(DLong myIx) final;
+  DLong Sibling();
+ };
 
 
 
