@@ -58,6 +58,39 @@
 // Besides, the misuse of event.Skip() can pose severe problems (blank widgets, etc...) since (quoting documentation) 
 // "Sizers rely on size events to function correctly. Therefore, in a sizer-based layout, do not forget to call Skip
 // on all size events you catch (and don't catch size events at all when you don't need to).
+BEGIN_EVENT_TABLE(wxTreeCtrlGDL, wxTreeCtrl)
+    EVT_TREE_BEGIN_DRAG(wxID_ANY, wxTreeCtrlGDL::OnDrag)
+//    EVT_TREE_BEGIN_RDRAG(wxID_ANY, wxTreeCtrlGDL::OnBeginRDrag)
+    EVT_TREE_END_DRAG(wxID_ANY, wxTreeCtrlGDL::OnDrop)
+//    EVT_TREE_BEGIN_LABEL_EDIT(wxID_ANY, wxTreeCtrlGDL::OnBeginLabelEdit)
+//    EVT_TREE_END_LABEL_EDIT(wxID_ANY, wxTreeCtrlGDL::OnEndLabelEdit)
+//    EVT_TREE_DELETE_ITEM(wxID_ANY, wxTreeCtrlGDL::OnDeleteItem)
+//    EVT_TREE_GET_INFO(wxID_ANY, wxTreeCtrlGDL::OnGetInfo)
+//    EVT_TREE_SET_INFO(wxID_ANY, wxTreeCtrlGDL::OnSetInfo)
+    EVT_TREE_ITEM_EXPANDED(wxID_ANY, wxTreeCtrlGDL::OnItemExpanded)
+//    EVT_TREE_ITEM_EXPANDING(wxID_ANY, wxTreeCtrlGDL::OnItemExpanding)
+    EVT_TREE_ITEM_COLLAPSED(wxID_ANY, wxTreeCtrlGDL::OnItemCollapsed)
+//    EVT_TREE_ITEM_COLLAPSING(wxID_ANY, wxTreeCtrlGDL::OnItemCollapsing)
+
+    EVT_TREE_SEL_CHANGED(wxID_ANY, wxTreeCtrlGDL::OnItemSelected)
+//    EVT_TREE_SEL_CHANGING(wxID_ANY, wxTreeCtrlGDL::OnSelChanging)
+    EVT_KEY_DOWN(wxTreeCtrlGDL::OnTreeKeyDown)
+    EVT_KEY_UP(wxTreeCtrlGDL::OnTreeKeyUp)
+    EVT_TREE_ITEM_ACTIVATED(wxID_ANY, wxTreeCtrlGDL::OnItemActivated)
+    EVT_TREE_STATE_IMAGE_CLICK(wxID_ANY, wxTreeCtrlGDL::OnItemStateClick)
+
+//    // so many different ways to handle right mouse button clicks...
+//    EVT_CONTEXT_MENU(wxTreeCtrlGDL::OnContextMenu)
+//    // EVT_TREE_ITEM_MENU is the preferred event for creating context menus
+//    // on a tree control, because it includes the point of the click or item,
+//    // meaning that no additional placement calculations are required.
+//    EVT_TREE_ITEM_MENU(wxID_ANY, wxTreeCtrlGDL::OnItemMenu)
+//    EVT_TREE_ITEM_RIGHT_CLICK(wxID_ANY, wxTreeCtrlGDL::OnItemRClick)
+//
+//    EVT_RIGHT_DOWN(wxTreeCtrlGDL::OnRMouseDown)
+//    EVT_RIGHT_UP(wxTreeCtrlGDL::OnRMouseUp)
+//    EVT_RIGHT_DCLICK(wxTreeCtrlGDL::OnRMouseDClick)
+END_EVENT_TABLE()
 
 DEFINE_EVENT_TYPE(wxEVT_SHOW_REQUEST)
 DEFINE_EVENT_TYPE(wxEVT_HIDE_REQUEST)
@@ -108,6 +141,7 @@ void wxTextCtrlGDL::OnChar(wxKeyEvent& event ) {
       widg->InitTag( "OFFSET", DLongGDL( this->GetInsertionPoint() ) );
       widg->InitTag( "CH", DByteGDL( 10 ) );
       GDLWidget::PushEvent( baseWidgetID, widg );
+      if (txt->IsMultiline()) event.Skip( );
       return;
       }
       event.Skip( );
@@ -277,7 +311,7 @@ void wxButtonGDL::OnButton( wxCommandEvent& event)
 #if (GDL_DEBUG_ALL_EVENTS || GDL_DEBUG_BUTTON_EVENTS)
   wxMessageOutputStderr().Printf(_T("in gdlMenuButton::OnButton: %d\n"),event.GetId());
 #endif
- this->PopupMenu(dynamic_cast<wxMenu*>(popupMenu));
+ this->PopupMenu(dynamic_cast<wxMenu*>(popupMenu),position);
  event.Skip();
 }
 
@@ -314,7 +348,7 @@ void gdlwxFrame::OnMenu( wxCommandEvent& event)
 #if (GDL_DEBUG_ALL_EVENTS || GDL_DEBUG_BUTTON_EVENTS)
   wxMessageOutputStderr().Printf(_T("in gdlwxFrame::OnMenu: %d\n"),event.GetId());
 #endif
-
+//if widget is a checked menu, wxWidgets should not change its checked state all by itself. How to avoird this?  
   WidgetIDT baseWidgetID = GDLWidget::GetIdOfTopLevelBase( event.GetId());
   // create GDL event struct
   DStructGDL*  widgbut = new DStructGDL( "WIDGET_BUTTON");
@@ -352,7 +386,7 @@ void gdlwxFrame::OnRadioButton( wxCommandEvent& event)
 
     GDLWidgetButton* widget = static_cast<GDLWidgetButton*>(GDLWidget::GetWidget( lastSelection));
     assert(widget!=NULL);
-    widget->SetButton( false);
+    widget->SetRadioButton( false);
     GDLWidget::PushEvent( baseWidgetID, widgbut);
   }
     
@@ -368,7 +402,7 @@ void gdlwxFrame::OnRadioButton( wxCommandEvent& event)
 
   GDLWidget* widget = GDLWidget::GetWidget( event.GetId());
   assert(widget->IsButton());
-  static_cast<GDLWidgetButton*>(widget)->SetButton( true);
+  static_cast<GDLWidgetButton*>(widget)->SetRadioButton( true);
   GDLWidget::PushEvent( baseWidgetID, widgbut);
 }
 
@@ -383,7 +417,7 @@ void gdlwxFrame::OnCheckBox( wxCommandEvent& event)
   WidgetIDT baseWidgetID = GDLWidget::GetIdOfTopLevelBase( event.GetId());
   GDLWidget* widget = GDLWidget::GetWidget( event.GetId());
   assert(widget->IsButton());
-  static_cast<GDLWidgetButton*>(widget)->SetButton( selectValue);
+  static_cast<GDLWidgetButton*>(widget)->SetRadioButton( selectValue);
   
   // create GDL event struct
   DStructGDL*  widgbut = new DStructGDL( "WIDGET_BUTTON");
@@ -498,14 +532,14 @@ void gdlwxFrame::OnListBoxDoubleClicked( wxCommandEvent& event)
   OnListBoxDo( event, 2);
 }
 
-//this is necessary to reproduce IDL's behaviour
+//this is necessary to reproduce IDL's behaviour.
 void gdlwxFrame::OnTextMouseEvents( wxMouseEvent& event)
 {
 #if (GDL_DEBUG_ALL_EVENTS || GDL_DEBUG_TEXT_EVENTS)
   wxMessageOutputStderr().Printf(_T("in gdlwxFrame::OnTextMouseEvents: %d\n"),event.GetId());
 #endif
-  GDLWidgetText* txt = static_cast<GDLWidgetText*>(GDLWidget::GetWidget( event.GetId()));
-  if( txt == NULL)
+  GDLWidgetText* txtWidget = static_cast<GDLWidgetText*>(GDLWidget::GetWidget( event.GetId()));
+  if( txtWidget == NULL)
   {
 #if (GDL_DEBUG_ALL_EVENTS || GDL_DEBUG_KBRD_EVENTS)
     wxMessageOutputStderr().Printf(_T("GDLWidget == NULL: %d\n"),event.GetId());
@@ -514,27 +548,43 @@ void gdlwxFrame::OnTextMouseEvents( wxMouseEvent& event)
     return;
   }
   DStructGDL* widg;
-  bool report = txt->HasEventType( GDLWidget::EV_ALL );
-  bool edit = txt->IsEditable( );
+  bool report = txtWidget->HasEventType( GDLWidget::EV_ALL );
+  bool edit = txtWidget->IsEditable( );
   WidgetIDT baseWidgetID = GDLWidget::GetIdOfTopLevelBase( event.GetId( ) );
   if ( report ) {
-    wxTextCtrl* textCtrl = dynamic_cast<wxTextCtrl*>(txt->GetWxWidget());
-    if( textCtrl == NULL)
-    {
-      event.Skip();
-      std::cerr<<"No wxWidget!"<<std::endl; return; // happens on construction 
-    }
+    wxTextCtrl* textCtrl = dynamic_cast<wxTextCtrl*>(txtWidget->GetWxWidget());
+    if( textCtrl == NULL) {std::cerr<<"gdlwxFrame::OnTextMouseEvents() : No wxWidget!"<<std::endl;  event.Skip( ); return;}
+    
+    long from,to;
     //signal entry (even if offset is bad)
-    if ( event.ButtonDown(wxMOUSE_BTN_LEFT) ) {
+    if ( event.ButtonDown(wxMOUSE_BTN_LEFT)) {
+      //Do not report, as the selection (wxWidgets side) has not been made yet, it will be done AFTER event.skip
+      //We report only the true selection, made with ButtonUp
+//      widg = new DStructGDL( "WIDGET_TEXT_SEL");
+//      widg->InitTag("ID", DLongGDL( event.GetId()));
+//      widg->InitTag("TOP", DLongGDL( baseWidgetID));
+//      widg->InitTag("HANDLER", DLongGDL( baseWidgetID ));
+//      widg->InitTag("TYPE", DIntGDL( 3)); // selection
+//      from=textCtrl->GetInsertionPoint();
+//      widg->InitTag("OFFSET", DLongGDL( from ));
+//      widg->InitTag("LENGTH", DLongGDL( 0 ));
+//      GDLWidget::PushEvent( baseWidgetID, widg);
+      // NOTE: "Process a wxEVT_LEFT_DOWN event. The handler of this event should normally call event.Skip()
+      // to allow the default processing to take place as otherwise the window under mouse wouldn't get the focus. "
+      event.Skip( );
+    } else if ( event.ButtonUp(wxMOUSE_BTN_LEFT)) {
       widg = new DStructGDL( "WIDGET_TEXT_SEL");
       widg->InitTag("ID", DLongGDL( event.GetId()));
       widg->InitTag("TOP", DLongGDL( baseWidgetID));
       widg->InitTag("HANDLER", DLongGDL( baseWidgetID ));
       widg->InitTag("TYPE", DIntGDL( 3)); // selection
-      widg->InitTag("OFFSET", DLongGDL( 0 ));
-      widg->InitTag("LENGTH", DLongGDL( 0 ));
+      textCtrl->GetSelection(&from,&to);
+      widg->InitTag("OFFSET", DLongGDL( from ));
+      widg->InitTag("LENGTH", DLongGDL( to-from ));
       GDLWidget::PushEvent( baseWidgetID, widg);
-      event.Skip( );
+      // NOTE: "Process a wxEVT_LEFT_DOWN event. The handler of this event should normally call event.Skip()
+      // to allow the default processing to take place as otherwise the window under mouse wouldn't get the focus. "
+      if (event.ButtonDown(wxMOUSE_BTN_LEFT)) event.Skip( );
     }
     return;
     //middle button (paste) should be filtered. Here we just avoid the paste pollutes a not-edit widget (without creating a GDL event).
@@ -1101,7 +1151,7 @@ void gdlwxFrame::OnEnterWindow( wxMouseEvent &event ) {
   if( widget == NULL) {
      event.Skip();
      return;
-  }
+    }
   if ( widget->GetEventFlags() & GDLWidget::EV_TRACKING ) {
     WidgetIDT baseWidgetID = GDLWidget::GetIdOfTopLevelBase( event.GetId( ) );
     DStructGDL* widgtracking = new DStructGDL( "WIDGET_TRACKING" );
@@ -1110,7 +1160,8 @@ void gdlwxFrame::OnEnterWindow( wxMouseEvent &event ) {
     widgtracking->InitTag( "HANDLER", DLongGDL( baseWidgetID ) );
     widgtracking->InitTag( "ENTER", DIntGDL( 1 ) ); 
     GDLWidget::PushEvent( baseWidgetID, widgtracking );
-  } else event.Skip();
+  }
+ event.Skip();
 }
 
 void gdlwxFrame::OnLeaveWindow( wxMouseEvent &event ) {
@@ -1130,7 +1181,8 @@ void gdlwxFrame::OnLeaveWindow( wxMouseEvent &event ) {
     widgtracking->InitTag( "HANDLER", DLongGDL( baseWidgetID ) );
     widgtracking->InitTag( "ENTER", DIntGDL( 0 ) ); 
     GDLWidget::PushEvent( baseWidgetID, widgtracking );
-  } else event.Skip();
+  }
+ event.Skip();
 }
 
 void gdlwxFrame::OnKBRDFocusChange( wxFocusEvent &event ) {
@@ -1368,9 +1420,6 @@ void gdlwxGraphicsPanel::OnPlotWindowSize(wxSizeEvent &event)
 }
 
 void gdlwxDrawPanel::OnMouseMove( wxMouseEvent &event ) {
-#if (GDL_DEBUG_ALL_EVENTS || GDL_DEBUG_MOVE_EVENTS)
-  wxMessageOutputStderr().Printf(_T("in gdlwxDrawPanel::OnMouseMove: %d\n"),event.GetId());
-#endif
   DULong eventFlags=myWidgetDraw->GetEventFlags();
 
   if ( eventFlags & GDLWidget::EV_MOTION ) {
@@ -1383,7 +1432,10 @@ void gdlwxDrawPanel::OnMouseMove( wxMouseEvent &event ) {
     widgdraw->InitTag( "X", DLongGDL( event.GetX() ) );
     widgdraw->InitTag( "Y", DLongGDL( drawSize.y-event.GetY()  ) );
     GDLWidget::PushEvent( baseWidgetID, widgdraw );
-  }  else event.Skip(); //normal end of event processing!
+  } else event.Skip(); //normal end of event processing!
+#if (GDL_DEBUG_ALL_EVENTS || GDL_DEBUG_MOVE_EVENTS)
+  wxMessageOutputStderr().Printf(_T("in gdlwxDrawPanel::OnMouseMove: ID=%d, coords=(%d,%d)\n"),event.GetId(),event.GetX(),event.GetY());
+#endif
 }
 
 void gdlwxDrawPanel::OnMouseDown( wxMouseEvent &event ) {
@@ -1523,7 +1575,7 @@ void gdlwxDrawPanel::OnKey( wxKeyEvent &event ) {
       default:
         widgdraw->InitTag( "TYPE", DIntGDL( 5 ) ); //normal key
         widgdraw->InitTag( "KEY", DLongGDL( 0 ) );
-        widgdraw->InitTag( "CH", DByteGDL( event.GetRawKeyCode() & 0xFF ) );
+        widgdraw->InitTag( "CH", DByteGDL( event.GetKeyCode() & 0xFF ) );
         widgdraw->InitTag( "MODIFIERS", DLongGDL( event.GetModifiers() ) );
         GDLWidget::PushEvent( baseWidgetID, widgdraw );
     }
@@ -1797,7 +1849,20 @@ void  wxGridGDL::OnTableRangeSelection(wxGridRangeSelectEvent & event){
 //}
 //
 
-
+void wxTreeCtrlGDL::OnTreeKeyDown(wxKeyEvent & event){
+#if (GDL_DEBUG_ALL_EVENTS || GDL_DEBUG_OTHER_EVENTS)
+  wxMessageOutputStderr().Printf(_T("in gdlTreeCtrl::OnTreeKeyDown: %d\n"),event.GetId());
+#endif  
+  wxTreeCtrlGDL* me=dynamic_cast<wxTreeCtrlGDL*>(event.GetEventObject());
+  me->SetCurrentModifier(event.GetKeyCode(), false); //add
+}
+void wxTreeCtrlGDL::OnTreeKeyUp(wxKeyEvent & event){
+#if (GDL_DEBUG_ALL_EVENTS || GDL_DEBUG_OTHER_EVENTS)
+  wxMessageOutputStderr().Printf(_T("in gdlTreeCtrl::OnTreeKeyUp: %d\n"),event.GetId());
+#endif  
+  wxTreeCtrlGDL* me=dynamic_cast<wxTreeCtrlGDL*>(event.GetEventObject());
+  me->SetCurrentModifier(event.GetKeyCode(), true); //remove
+}
 void wxTreeCtrlGDL::OnItemActivated(wxTreeEvent & event){
 #if (GDL_DEBUG_ALL_EVENTS || GDL_DEBUG_OTHER_EVENTS)
   wxMessageOutputStderr().Printf(_T("in gdlTreeCtrl::OnItemActivated: %d\n"),event.GetId());
@@ -1810,125 +1875,169 @@ void wxTreeCtrlGDL::OnItemActivated(wxTreeEvent & event){
     DStructGDL* treeselect = new DStructGDL( "WIDGET_TREE_SEL");
     treeselect->InitTag("ID", DLongGDL( dynamic_cast<wxTreeItemDataGDL*>(me->GetItemData(event.GetItem()))->widgetID ));
     treeselect->InitTag("TOP", DLongGDL( baseWidgetID));
-    treeselect->InitTag("HANDLER", DLongGDL( GDLWidgetTableID ));
+    treeselect->InitTag("HANDLER", DLongGDL( GDLWidgetTreeID ));
     treeselect->InitTag("TYPE", DIntGDL(0)); // 0
     treeselect->InitTag("CLICKS",DLongGDL(2));
     // insert into structList
     GDLWidget::PushEvent( baseWidgetID, treeselect );
     event.Skip();
+    me->Refresh();
 }
 
 
 void wxTreeCtrlGDL::OnItemSelected(wxTreeEvent & event){
 #if (GDL_DEBUG_ALL_EVENTS || GDL_DEBUG_OTHER_EVENTS)
-  wxMessageOutputStderr().Printf(_T("in gdlTreeCtrl::OnItemActivated: %d\n"),event.GetId());
+  wxMessageOutputStderr().Printf(_T("in gdlTreeCtrl::OnItemSelected: %d\n"),event.GetId());
 #endif
   WidgetIDT baseWidgetID = GDLWidget::GetIdOfTopLevelBase( event.GetId( ) );
 //get GDLWidgetTree ID which was passed as wxTreeItemData at creation to identify
 //the GDL widget that received the event
     wxTreeCtrlGDL* me=dynamic_cast<wxTreeCtrlGDL*>(event.GetEventObject());
     WidgetIDT selected=dynamic_cast<wxTreeItemDataGDL*>(me->GetItemData(event.GetItem()))->widgetID;
-    GDLWidgetTree* tree= static_cast<GDLWidgetTree*>(GDLWidget::GetWidget(dynamic_cast<wxTreeItemDataGDL*>(me->GetItemData(event.GetItem()))->widgetID));
+    GDLWidgetTree* tree= static_cast<GDLWidgetTree*>(GDLWidget::GetWidget(selected));
     //inform root widget it is selected
-    GDLWidgetTree* root=static_cast<GDLWidgetTree*>(GDLWidget::GetWidget(tree->GetRootID()));
-    root->SetSelectedID(selected);
-    
+    GDLWidgetTree* root=tree->GetMyRootGDLWidgetTree();
     DStructGDL* treeselect = new DStructGDL( "WIDGET_TREE_SEL");
     treeselect->InitTag("ID", DLongGDL( selected ));
     treeselect->InitTag("TOP", DLongGDL( baseWidgetID));
-    treeselect->InitTag("HANDLER", DLongGDL( GDLWidgetTableID ));
+    treeselect->InitTag("HANDLER", DLongGDL( GDLWidgetTreeID ));
     treeselect->InitTag("TYPE", DIntGDL(0)); // 0
     treeselect->InitTag("CLICKS",DLongGDL(1));
     // insert into structList
     GDLWidget::PushEvent( baseWidgetID, treeselect );
     event.Skip();
+    me->Refresh();
 }
 
-void wxTreeCtrlGDL::OnBeginDrag(wxTreeEvent & event){
-  //largely useful protection!!!
-  if (!event.GetItem().IsOk()) {    event.Skip(); return;}
-
+void wxTreeCtrlGDL::OnDrag(wxTreeEvent & event){
 #if (GDL_DEBUG_ALL_EVENTS || GDL_DEBUG_OTHER_EVENTS)
-  wxMessageOutputStderr().Printf(_T("in gdlTreeCtrl::OnBeginDrag: %d\n"),event.GetId());
+  wxMessageOutputStderr().Printf(_T("in gdlTreeCtrl::OnDrag: %d "),event.GetId());
 #endif
-
-//needed to explicitly authorize dragging.
-//get GDLWidgetTree ID which was passed as wxTreeItemData at creation to identify
-//the GDL widget that received the event
+  if (!event.GetItem().IsOk()) {    event.Skip(); return;}
+  
     wxTreeCtrlGDL* me=dynamic_cast<wxTreeCtrlGDL*>(event.GetEventObject());
-    GDLWidgetTree* item = static_cast<GDLWidgetTree*>(GDLWidget::GetWidget(dynamic_cast<wxTreeItemDataGDL*>(me->GetItemData(event.GetItem()))->widgetID));
-    if (item->IsDraggable()) event.Allow();
-        event.Skip();
-
+    WidgetIDT selected=dynamic_cast<wxTreeItemDataGDL*>(me->GetItemData(event.GetItem()))->widgetID;
+    GDLWidgetTree* tree= static_cast<GDLWidgetTree*>(GDLWidget::GetWidget(selected));
+      
+    if (tree->GetDragability()) {//was set or inherited from one ancestor that was set
+      me->SetDragged(selected);
+      event.Allow();
+#if (GDL_DEBUG_ALL_EVENTS || GDL_DEBUG_OTHER_EVENTS)
+  wxMessageOutputStderr().Printf(_T("allowed.\n"),event.GetId());
+#endif
+  return;
+    }  
+#if (GDL_DEBUG_ALL_EVENTS || GDL_DEBUG_OTHER_EVENTS)
+  wxMessageOutputStderr().Printf(_T("canceled.\n"),event.GetId());
+#endif
+  event.Skip();
 }
 
-void wxTreeCtrlGDL::OnItemDropped(wxTreeEvent & event){
-  if (!event.GetItem().IsOk()) {    event.Skip(); return;}
+void wxTreeCtrlGDL::OnDrop(wxTreeEvent & event){
 
 #if (GDL_DEBUG_ALL_EVENTS || GDL_DEBUG_OTHER_EVENTS)
-  wxMessageOutputStderr().Printf(_T("in gdlTreeCtrl::OnItemDropped: %d\n"),event.GetId());
+  wxMessageOutputStderr().Printf(_T("in gdlTreeCtrl::OnDrop: %d\n"),event.GetId());
 #endif
+  if (!event.GetItem().IsOk()) {    event.Skip(); return;}
+
     WidgetIDT baseWidgetID = GDLWidget::GetIdOfTopLevelBase( event.GetId( ) );
     wxTreeCtrlGDL* me=dynamic_cast<wxTreeCtrlGDL*>(event.GetEventObject());
-    GDLWidgetTree* item = static_cast<GDLWidgetTree*>(GDLWidget::GetWidget(dynamic_cast<wxTreeItemDataGDL*>(me->GetItemData(event.GetItem()))->widgetID));
+    WidgetIDT selected=dynamic_cast<wxTreeItemDataGDL*>(me->GetItemData(event.GetItem()))->widgetID;
+    GDLWidgetTree* item = static_cast<GDLWidgetTree*>(GDLWidget::GetWidget(selected));
+    GDLWidgetTree* root=item->GetMyRootGDLWidgetTree();
 
-    if (item->IsDroppable()) {
+    if (item->GetDropability()) { //was set or inherited from one ancestor that was set
  //get GDLWidgetTree ID which was passed as wxTreeItemData at creation to identify
 //the GDL widget that received the event
       DStructGDL* treedrop = new DStructGDL( "WIDGET_DROP");
       treedrop->InitTag("ID", DLongGDL( item->GetWidgetID()  )); //ID of the destination
       treedrop->InitTag("TOP", DLongGDL( baseWidgetID));
-      treedrop->InitTag("HANDLER", DLongGDL( GDLWidgetTableID ));
-      treedrop->InitTag("DRAG_ID", DLongGDL( item->GetRootID())); // ID of the source TREE
-      treedrop->InitTag("POSITION",DIntGDL(2)); //ALWAYS 2 IT SEEMS THAT wxW cannot do better? !   1 above 2 on 3 below destination widget
+      treedrop->InitTag("HANDLER", DLongGDL( GDLWidgetTreeID ));
+      treedrop->InitTag("DRAG_ID", DLongGDL( GDLWidgetTreeID )); // ID of the source TREE
+      treedrop->InitTag("POSITION",item->IsFolder()?DIntGDL(2):DIntGDL(3)); //   1 above 2 on 3 below destination widget
       treedrop->InitTag("X",DLongGDL(event.GetPoint().x)); //x and Y coord of position wrt lower left corner of destination tree widget
       treedrop->InitTag("Y",DLongGDL(event.GetPoint().y));
-      treedrop->InitTag("MODIFIERS",DIntGDL(0)); //mask with 1 shift 2 control 4 caps lock 8 alt
+      treedrop->InitTag("MODIFIERS",DIntGDL(me->GetCurrentModifier())); //mask with 1 shift 2 control 4 caps lock 8 alt
       // insert into structList
       GDLWidget::PushEvent( baseWidgetID, treedrop );
   }
+    //unset dragged
+    me->SetDragged(0);
+
   event.Skip();
+    me->Refresh();
 
 }
 void wxTreeCtrlGDL::OnItemExpanded(wxTreeEvent & event){
-  if (!event.GetItem().IsOk()) {    event.Skip(); return;}
 #if (GDL_DEBUG_ALL_EVENTS || GDL_DEBUG_OTHER_EVENTS)
   wxMessageOutputStderr().Printf(_T("in gdlTreeCtrl::OnItemExpanded: %d\n"),event.GetId());
 #endif
-    WidgetIDT baseWidgetID = GDLWidget::GetIdOfTopLevelBase( event.GetId( ) );
+  if (!event.GetItem().IsOk()) {    event.Skip(); return;}
+
+      WidgetIDT baseWidgetID = GDLWidget::GetIdOfTopLevelBase( event.GetId( ) );
 //get GDLWidgetTree ID which was passed as wxTreeItemData at creation to identify
 //the GDL widget that received the event
     wxTreeCtrlGDL* me=dynamic_cast<wxTreeCtrlGDL*>(event.GetEventObject());
-    
+    WidgetIDT whoID=dynamic_cast<wxTreeItemDataGDL*>(me->GetItemData(event.GetItem()))->widgetID;
+    GDLWidgetTree* who=static_cast<GDLWidgetTree*>(GDLWidget::GetWidget(whoID));
+    who->SetExpanded(true);
     DStructGDL* treeexpand = new DStructGDL( "WIDGET_TREE_EXPAND");
-    treeexpand->InitTag("ID", DLongGDL( dynamic_cast<wxTreeItemDataGDL*>(me->GetItemData(event.GetItem()))->widgetID ));
+    treeexpand->InitTag("ID", DLongGDL( whoID ));
     treeexpand->InitTag("TOP", DLongGDL( baseWidgetID));
-    treeexpand->InitTag("HANDLER", DLongGDL( GDLWidgetTableID ));
+    treeexpand->InitTag("HANDLER", DLongGDL( GDLWidgetTreeID ));
     treeexpand->InitTag("TYPE", DIntGDL(1)); // 1
     treeexpand->InitTag("EXPAND",DLongGDL(1)); //1 expand
     // insert into structList
     GDLWidget::PushEvent( baseWidgetID, treeexpand );
     event.Skip();
+    me->Refresh();
 }
 void wxTreeCtrlGDL::OnItemCollapsed(wxTreeEvent & event){
-  if (!event.GetItem().IsOk()) {    event.Skip(); return;}
 #if (GDL_DEBUG_ALL_EVENTS || GDL_DEBUG_OTHER_EVENTS)
   wxMessageOutputStderr().Printf(_T("in gdlTreeCtrl::OnItemCollapsed: %d\n"),event.GetId());
 #endif
-    WidgetIDT baseWidgetID = GDLWidget::GetIdOfTopLevelBase( event.GetId( ) );
+  if (!event.GetItem().IsOk()) {    event.Skip(); return;}
+
+      WidgetIDT baseWidgetID = GDLWidget::GetIdOfTopLevelBase( event.GetId( ) );
 //get GDLWidgetTree ID which was passed as wxTreeItemData at creation to identify
 //the GDL widget that received the event
     wxTreeCtrlGDL* me=dynamic_cast<wxTreeCtrlGDL*>(event.GetEventObject());
-    
+    WidgetIDT whoID=dynamic_cast<wxTreeItemDataGDL*>(me->GetItemData(event.GetItem()))->widgetID;
+    GDLWidgetTree* who=static_cast<GDLWidgetTree*>(GDLWidget::GetWidget(whoID));
+    who->SetExpanded(false);    
     DStructGDL* treeexpand = new DStructGDL( "WIDGET_TREE_EXPAND");
-    treeexpand->InitTag("ID", DLongGDL( dynamic_cast<wxTreeItemDataGDL*>(me->GetItemData(event.GetItem()))->widgetID ));
+    treeexpand->InitTag("ID", DLongGDL( whoID ) );
     treeexpand->InitTag("TOP", DLongGDL( baseWidgetID));
-    treeexpand->InitTag("HANDLER", DLongGDL( GDLWidgetTableID ));
+    treeexpand->InitTag("HANDLER", DLongGDL( GDLWidgetTreeID ));
     treeexpand->InitTag("TYPE", DIntGDL(1)); // 1
     treeexpand->InitTag("EXPAND",DLongGDL(0)); //0 collapse
     // insert into structList
     GDLWidget::PushEvent( baseWidgetID, treeexpand );
     event.Skip();
+    me->Refresh();
 }
+ void wxTreeCtrlGDL::OnItemStateClick(wxTreeEvent & event){
+#if (GDL_DEBUG_ALL_EVENTS || GDL_DEBUG_OTHER_EVENTS)
+  wxMessageOutputStderr().Printf(_T("in gdlTreeCtrl::OnItemCollapsed: %d\n"),event.GetId());
+#endif
+  if (!event.GetItem().IsOk()) {    event.Skip(); return;}
+  // change checked state first
+  this->SetItemState(event.GetItem(),wxTREE_ITEMSTATE_NEXT);
   
+      WidgetIDT baseWidgetID = GDLWidget::GetIdOfTopLevelBase( event.GetId( ) );
+//get GDLWidgetTree ID which was passed as wxTreeItemData at creation to identify
+//the GDL widget that received the event
+    wxTreeCtrlGDL* me=dynamic_cast<wxTreeCtrlGDL*>(event.GetEventObject());
+    
+    DStructGDL* treeechecked = new DStructGDL( "WIDGET_TREE_CHECKED");
+    treeechecked->InitTag("ID", DLongGDL( dynamic_cast<wxTreeItemDataGDL*>(me->GetItemData(event.GetItem()))->widgetID ));
+    treeechecked->InitTag("TOP", DLongGDL( baseWidgetID));
+    treeechecked->InitTag("HANDLER", DLongGDL( GDLWidgetTreeID ));
+    treeechecked->InitTag("TYPE", DIntGDL(2)); // 2
+    treeechecked->InitTag("STATE",DLongGDL(this->GetItemState(event.GetItem()))); //on off
+    // insert into structList
+    GDLWidget::PushEvent( baseWidgetID, treeechecked );
+    event.Skip();
+    me->Refresh();
+ }  
 #endif
