@@ -23,9 +23,18 @@
 #include "objects.hpp"
 #include "graphicsdevice.hpp"
 
-#include "devicewin.hpp"
+#if _WIN32
+#include "otherdevices/devicewin.hpp"
+#endif
+  
+#ifdef HAVE_LIBWXWIDGETS 
 #include "devicewx.hpp"
-#include "devicex.hpp"
+#endif
+  
+#ifdef HAVE_X
+#include "otherdevices/devicex.hpp"
+#endif
+  
 #include "deviceps.hpp"
 #include "devicesvg.hpp"
 #include "devicez.hpp"
@@ -140,10 +149,10 @@ bool GraphicsDevice::ExistDevice( const string& device, int &index)
   for( int i=0; i<size; i++)
     {
       if( deviceList[ i]->Name() == device)
-	{
-	  index=i;
-	  return true;
-	}
+    {
+      index=i;
+      return true;
+    }
     }
   return false;
 }
@@ -154,13 +163,13 @@ bool GraphicsDevice::SetDevice( const string& device)
   for( int i=0; i<size; i++)
     {
       if( deviceList[ i]->Name() == device)
-	{
-	  actDevice=deviceList[ i];
-	  // update !D
-	  SysVar::SetD( actDevice->DStruct());
-	  
-	  return true;
-	}
+    {
+      actDevice=deviceList[ i];
+      // update !D
+      SysVar::SetD( actDevice->DStruct());
+      
+      return true;
+    }
     }
   return false;
 }
@@ -171,9 +180,9 @@ DStructGDL* GraphicsDevice::GetDeviceStruct( const string& device)
   for( int i=0; i<size; i++)
     {
       if( deviceList[ i]->Name() == device)
-	{
-	  return deviceList[ i]->DStruct();
-	}
+    {
+      return deviceList[ i]->DStruct();
+    }
     }
   return NULL;
 }
@@ -183,65 +192,62 @@ void GraphicsDevice::Init()
 
   DefineDStructDesc();
 
+  GraphicsDevice* current_device=NULL;
+  
   // 4 devices types without surprise !
   deviceList.push_back( new DeviceNULL());
   deviceList.push_back( new DevicePS());
   deviceList.push_back( new DeviceSVG());
   deviceList.push_back( new DeviceZ());
   
-  // if GDL_USE_WX (or switch --use-wx) , and has wxWidgets, the wxWidgets device becomes 'X' or 'WIN' depending on machine,
+  std::string defaultDeviceName=std::string("X"); //what we expect the plot device to be
+#ifdef _WIN32
+  defaultDeviceName=std::string("WIN"); //ALWAYS WIN on WINDOWS!
+#elif __APPLE__
+  if (usePlatformDeviceName) defaultDeviceName=std::string("MAC");
+#endif
+  
+  // if GDL_DISABLE_WX_PLOTS (or switch --no-use-wx ) IS NOT PRESENT , and has wxWidgets, the wxWidgets device becomes 'X' or 'WIN' depending on machine,
   // no other device is defined.
   if (useWxWidgetsForGraphics) {
 #ifdef HAVE_LIBWXWIDGETS
-    //start wxWidgets here instead of first call of a widget function.
-      if( ! wxInitialize( ) ) ThrowGDLException("Unable to initialize wxWidgets");
-      GDLWidget::SetWxStarted();
+    current_device=new DeviceWX(defaultDeviceName);    //define wxWidgets 'plot' as either X..
+    actGUIDevice =current_device;      // GuiDevice is same as X or WIN in this case
+#else //not linked with wxWidgets: plot is either X or WIN depending on platform
 #ifdef HAVE_X
-    deviceList.push_back( new DeviceWX("X"));
-#else
-#ifdef _WIN32
-    deviceList.push_back( new DeviceWX("WIN"));
+    current_device=new DeviceX(defaultDeviceName); //X on unix, MAC on mac... is it necessary ?
+#elif _WIN32
+    current_device=NULL; 
 #endif
-#endif  
-#else
-#ifdef HAVE_X
-    deviceList.push_back( new DeviceX());
+    actGUIDevice = NULL; //no wxWidgets at all, because on Windows without wX, or everywhere else without wX and X11.
 #endif
-#ifdef _WIN32
-    deviceList.push_back( new DeviceWIN());
-#endif
-#endif
-  } else {
+  } else {  //wxWidgets will *NOT*be used for plot windows, unless there is nothing else left.
 #ifdef HAVE_LIBWXWIDGETS
-    deviceList.push_back( new DeviceWX()); //traditional use, device will be called "MAC"
-#endif
+    if (useWxWidgets) { //wxWidgets is present but not as default plot system.
+    actGUIDevice = new DeviceWX();  //even if wx is not used for plots, it will be used for widget_draw. The corresponding name will be (unsupported) "WX".
+    deviceList.push_back(actGUIDevice); // do not forget to add it to list!
+    }
+#endif   
 #ifdef HAVE_X
-    deviceList.push_back( new DeviceX());
+    current_device= new DeviceX(defaultDeviceName); //X on unix & Mac , MAC on mac if --MAC switch asked for.
+#else 
+    current_device=NULL; //prepare to the worst..
+    //but, hey, may be wxWidgets is here?
+#ifdef HAVE_LIBWXWIDGETS
+    if (useWxWidgets) current_device= new DeviceWX(defaultDeviceName);    //define wxWidgets 'plot' as either X or WIN and use it despite the orders.
 #endif
-#ifdef _WIN32
-    deviceList.push_back( new DeviceWIN());
 #endif
   }
-  // we try to set X, WIN or WX as default 
-  // (and NULL if X11 system (Linux, OSX, Sun) but without X11 at compilation)
-#if defined(HAVE_X) // Check X11 first
-  if( !SetDevice( "X")) 
-#elif defined(_WIN32) // If Windows enable WinGCC driver 
-    if( !SetDevice( "WIN")) 
-#elif defined (HAVE_LIBWXWIDGETS) // Finally check WX
-      if (!SetDevice("MAC"))
-#else
-	if( !SetDevice( "NULL")) 
-#  endif
-#  if !defined (HAVE_X) && !defined (HAVE_LIBWXWIDGETS) && !defined (_WIN32)
-	  {
-	  }
-#  else
-  {
+//nothing should prevent gdl to be used without 'direct' graphics ?
+  if (current_device == NULL) {
+    defaultDeviceName.assign("NULL"); //the NULL device. handy.
+  } else deviceList.push_back(current_device); //push the 'PLOT' device.
+  if (iAmANotebook) defaultDeviceName.assign("SVG"); 
+  if( !SetDevice(defaultDeviceName) ){
     cerr << "Error initializing graphics." << endl;
-    exit( EXIT_FAILURE);
+    exit(EXIT_FAILURE);
   }
-#  endif
+  
 // GDL (at least with device X and Wx) handle equally any types of screens,
 // with an equivalent depth of 24 (tested and true). So there is no
 // need to return any depth, color number etc that would be anything
@@ -253,38 +259,14 @@ void GraphicsDevice::Init()
     unsigned bckTag=pStruct->Desc()->TagIndex("BACKGROUND");
     (*static_cast<DLongGDL*>(pStruct->GetTag(colorTag, 0)))[0]=16777215;
     (*static_cast<DLongGDL*>(pStruct->GetTag(bckTag, 0)))[0]=0;
-#ifdef HAVE_LIBWXWIDGETS
-  // some X error message suggested this call
-#ifdef HAVE_X
-  //  XInitThreads();
-#endif
-#endif
-  int index=0;
-  // setting the GUI dev. There is only ONE possibility: the wxWidgets.
-  // depending on how we build GDL, wxWidgets may replace X and WIN entirely, or not.
-  // If not, it will be called "MAC" (temporarily).
-  // If yes (replaces either X or WIN) it will be called 'X' or 'WIN' accordingly.
-  if (ExistDevice( "MAC", index)) { //wxWidgets present, in concurrence with others.
-    actGUIDevice = deviceList[index];
-  } else if (ExistDevice( "X", index)) {
-#ifdef HAVE_LIBWXWIDGETS    
-    actGUIDevice = deviceList[index]; //true existing device
-#else
-    actGUIDevice = deviceList[0];  //will be a fake GUI and procedures will complain.
-#endif
-  } else if (ExistDevice("WIN", index)) {
-#ifdef HAVE_LIBWXWIDGETS  
-    actGUIDevice = deviceList[index]; //true existing device
-#else
-    actGUIDevice = deviceList[0];  //will be a fake GUI and procedures will complain.
-#endif
-  } else {
-    actGUIDevice = deviceList[0];  //will be a fake GUI and procedures will complain.
-  }
 }
 
 void GraphicsDevice::DestroyDevices()
 {
+    
+#ifdef HAVE_LIBWXWIDGETS
+  if (useWxWidgets) GDLWidget::UnInit();    // un-initialize widget system
+#endif
   PurgeContainer( deviceList);
   actDevice = NULL;
 }
@@ -419,39 +401,20 @@ void GraphicsMultiDevice::SetActWin(int wIx) {
 // process user deleted windows
 // should be done in a thread
 
-void GraphicsMultiDevice::TidyWindowsList() {
-  int wLSize = winList.size();
+void GraphicsMultiDevice::TidyWindowsList(bool dodelete) {
+ int wLSize = winList.size();
 
   for (int i = 0; i < wLSize; i++) if (winList[i] != NULL && !winList[i]->GetValid()) {
-    
-    //general purpose winlist cleaning with destruction of "closed" plstreams and (eventually) associated widgets:
-    //in case winList groups X11 streams (or WIN streams) *and* wxWidgets streams (GDL_USE_WX="NO") the following
-    //permits to delete the widget_draw also, not only the plplot stream.
-#ifdef HAVE_LIBWXWIDGETS
-    if (dynamic_cast<GDLWXStream*> (winList[i]) != NULL) {
-      GDLDrawPanel* panel = NULL;
-      panel = dynamic_cast<GDLDrawPanel*> (static_cast<GDLWXStream*> (winList[i])->GetGDLDrawPanel());
-      //test if stream is associated to graphic window or widget_draw. If graphic, destroy directly TLB widget.
-      GDLWidgetDraw *draw = panel->GetGDLWidgetDraw();
-      if (draw) {
-        //parent of panel may be a GDLFrame. If frame is actually made by the WOpen function, destroy everything.
-        GDLWidgetBase* container = NULL;
-        container = static_cast<GDLWidgetBase*> (draw->GetTopLevelBaseWidget(draw->WidgetID()));
-        if (container && container->IsGraphicWindowFrame()) container->SelfDestroy();
-        else delete draw;
-      } else delete winList[i];
-    } else
-#endif     
-    delete winList[i];
+    if (dodelete) delete winList[i];
     winList[i] = NULL;
     oList[i] = 0;
-  }
+ }
   // set new actWin IF NOT VALID ANY MORE
   if (actWin < 0 || actWin >= wLSize || winList[actWin] == NULL || !winList[actWin]->GetValid()) {
     std::vector< long>::iterator mEl = std::max_element(oList.begin(), oList.end()); // most recently created
     if (*mEl == 0) { // no window open
       SetActWin(-1); //sets    oIx = 1;
-    } else SetActWin(GraphicsDevice::GetDevice()->GetNonManagedWidgetActWin(false)); //get first non-managed window. false is needed. 
+    } else SetActWin(GraphicsDevice::GetDevice()->GetNonManagedWidgetActWin(false)); //get first non-managed window. false is absolutely needed (forever loop). 
   }
 }
 
@@ -478,12 +441,11 @@ void GraphicsMultiDevice::EventHandler() {
     if (winList[i] != NULL)
       winList[i]->EventHandler();
 
-  // TidyWindowsList(); //removing it here removes a lot of loops but needs TidyWindowsList() to be called at all places needed.
+ // TidyWindowsList(); //removing it here removes a lot of loops but needs TidyWindowsList() to be called at all places needed.
 }
 
 bool GraphicsMultiDevice::WDelete(int wIx) {
-  if( wIx >= 0 && winList[ wIx] != NULL) {winList[ wIx]->SetValid(false);
-    TidyWindowsList();//if WDelete is always called with a valid wIx and is always a window, not a widget, this should be OK.
+  if( wIx >= 0 && winList[ wIx] != NULL) {winList[ wIx]->SetValid(false); TidyWindowsList();//if WDelete is always called with a valid wIx and is always a window, not a widget, this should be OK.
     return true;
   } else return false;
 }
@@ -521,14 +483,15 @@ bool GraphicsMultiDevice::WShow(int ix, bool show, int iconic) {
   int wLSize = winList.size();
   if (ix >= wLSize || ix < 0 || winList[ix] == NULL) return false;
 
-  if (iconic!=-1) { //iconic asked. do nothing else.
-    if (iconic==1) IconicWin(ix); else DeIconicWin(ix);
+  if (iconic != -1) { //iconic asked. do nothing else.
+    if (iconic == 1) IconicWin(ix);
+    else DeIconicWin(ix);
+    UnsetFocus();
     return true;
+  } else {
+    if (show) RaiseWin(ix); else LowerWin(ix);
   }
-  
-  if (show) RaiseWin(ix);  else LowerWin(ix);
-
-  //UnsetFocus();
+  UnsetFocus();
 
   return true;
 }
@@ -598,7 +561,7 @@ bool GraphicsMultiDevice::SetBackingStore(int value) {
 bool GraphicsMultiDevice::Hide() //used as a substitute for /PIXMAP in DEVICE
 {
   TidyWindowsList();
-  winList[ actWin]->UnMapWindow();
+  winList[ actWin]->UnMapWindowAndSetPixmapProperty();
   return true;
 }
 
@@ -611,40 +574,17 @@ int GraphicsMultiDevice::ActWin() {
   return actWin;
 }
 
-int GraphicsMultiDevice::GetNonManagedWidgetActWin(bool doTidyWindowList) { //for case of "WSET,-1" or cases where the program has to find
+int GraphicsMultiDevice::GetNonManagedWidgetActWin(bool doTidyWindowList) { 
+  //for case of "WSET,-1" or cases where the program has to find
   //by itself the first available window: this window must not be an active managed widget.
   //where the returned actwin is the first NON-MANAGED-WIDGET_DRAW available.
 
   if (doTidyWindowList) TidyWindowsList(); //bool is used in cas this function is called from... TidyWindowList itself
 
-#ifndef HAVE_LIBWXWIDGETS
   for (int i = 0; i < winList.size(); i++) if (winList[i] != NULL ) {
-    if (!(winList[i]->IsPixmapWindow())) {return i;}//first non-pixmap window.
+    if (winList[i]->IsPlot() && !(winList[i]->IsPixmapWindow())) {return i;}//first non-pixmap window.
   }
   return -1;
-#else
-  for (int i = 0; i < winList.size(); i++) if (winList[i] != NULL ) {
-    if (!(winList[i]->IsPixmapWindow())) {//forget a pixmap window.
-      if (dynamic_cast<GDLWXStream*> (winList[i]) == NULL) {
-        //found non-widget window, probably an X11 one. Will return it if it is not a PIXMAP window
-        if (!(winList[i]->IsPixmapWindow())) return i;
-      } else { //special case of a graphic window which is either a draw widget (needs to be not managed) 
-        // or a wxWindow "normal window" (in which case we return it immediately).
-        GDLDrawPanel* panel = NULL;
-        panel = dynamic_cast<GDLDrawPanel*> (static_cast<GDLWXStream*> (winList[i])->GetGDLDrawPanel());
-        //test if stream is associated to graphic window or widget_draw. If graphic, return. If widget, return if not managed.
-        GDLWidgetDraw *draw = panel->GetGDLWidgetDraw();
-        if (draw) { //panel. Managed?
-          //parent of panel may be a GDLFrame. 
-          GDLWidgetBase* container = NULL;
-          container = static_cast<GDLWidgetBase*> (draw->GetTopLevelBaseWidget(draw->WidgetID()));
-          if (container && !(container->GetManaged())) return i; //first non-managed draw widget
-        } else return i; //a wxWidgets "plot" (as opposed to) "widget" bona fide window to be returned.
-      } 
-    }
-  }
-  return -1; //nothing found
-#endif     
 }
 
 bool GraphicsMultiDevice::CopyRegion(DLongGDL* me) {
