@@ -21,12 +21,20 @@
 #include "datatypes.hpp"
 #include "envt.hpp"
 
-/** 
+/**
  * needs the hdf5 library, obtainable from http://hdf.ncsa.uiuc.edu/HDF5/
  */
 #define H5_USE_16_API 1
+#include <H5version.h>
 #include "hdf5.h"
 
+/*
+// We stay in 1.8 version, using the "1" old interface
+#define H5Dopen H5Dopen1
+#define H5Gopen H5Gopen1
+#define H5Ewalk H5Ewalk1
+*/
+
 namespace lib {
 
   /**
@@ -40,6 +48,7 @@ namespace lib {
   BaseGDL* h5d_open_fun( EnvT* e);
   BaseGDL* h5d_read_fun( EnvT* e);
   BaseGDL* h5d_get_space_fun( EnvT* e);
+  BaseGDL* h5s_get_simple_extent_ndims_fun( EnvT* e);
   BaseGDL* h5s_get_simple_extent_dims_fun( EnvT* e);
   void h5f_close_pro( EnvT* e);
   void h5d_close_pro( EnvT* e);
