@@ -22,7 +22,7 @@
 namespace SysVar
 {
   //  extern unsigned int pathIx, dIx;
-
+  DFloat* GetSC();
   // returns a StrArr with the path to search
   const StrArr& GDLPath();
   void SetGDLPath( const DString& newPath);
@@ -39,7 +39,6 @@ namespace SysVar
 
   // returns !DIR
   const DString& Dir();
-  const DString& GshhsDir();
 
   // updates !STIME
   void UpdateSTime();
@@ -68,6 +67,10 @@ namespace SysVar
 
   // returns !MAP
   DStructGDL* Map();
+  // returns !FANCY
+  DIntGDL* GetFancy();
+  // returns !GDL
+  DStructGDL* GDLconfig();
 
   // returns !CPU
   DStructGDL* Cpu();
