@@ -36,6 +36,7 @@
 #include "math_fun_gm.hpp"
 #include "math_fun_ng.hpp"
 #include "plotting.hpp"
+#include "sorting.hpp"
 
 #include "file.hpp"
 
@@ -48,6 +49,8 @@
 #include "linearprogramming.hpp"
 #include "saverestore.hpp"
 
+#include "terminfo.hpp"
+
 #ifdef USE_PYTHON
 #  include "gdlpython.hpp"
 #endif
@@ -55,10 +58,6 @@
 #include "grib.hpp"
 #include "semshm.hpp"
 
-// for extensions
-#include "new.hpp"
-
-
 using namespace std;
 
 // declare here other library init functions
@@ -80,7 +79,7 @@ void LibInit()
   LibInit_ac();
   LibInit_gm();
   LibInit_ng(); 
-  LibInit_jp();
+  if (useWxWidgets) LibInit_jp(); //when called, useWxWidgets as already been set to false if wxWidgets was not initializing.
   LibInit_exists();
 
   const char KLISTEND[] = "";
@@ -89,7 +88,9 @@ void LibInit()
   const string restoreWarnKey[]={"NO_COMPILE", "RELAXED_STRUCTURE_ASSIGNMENT", "RESTORED_OBJECTS" , KLISTEND};
   new DLibPro(lib::gdl_restore,string("RESTORE"),1,restoreKey,restoreWarnKey);
   
-  const string saveKey[]={ "FILENAME","DESCRIPTION","VERBOSE","VARIABLES", "ALL", "COMM", "COMPRESS", "SYSTEM_VARIABLES",KLISTEND};
+  const string saveKey[]={ "FILENAME","DESCRIPTION","VERBOSE","VARIABLES", "ALL", "COMM", "COMPRESS", "SYSTEM_VARIABLES"
+    ,"XDR" //obsolete
+    ,KLISTEND};
   const string saveWarnKey[]={"EMBEDDED","ROUTINES", KLISTEND};
   new DLibPro(lib::gdl_save,string("SAVE"),-1,saveKey,saveWarnKey);
 
@@ -108,10 +109,17 @@ void LibInit()
   new DLibFun(lib::scope_varfetch_value,string("SCOPE_VARFETCH"),-1,scope_varfetchKey,scope_varfetchWarnKey);
   const string scope_tracebackKey[]={"STRUCTURE","SYSTEM", KLISTEND};
   new DLibFunRetNew(lib::scope_traceback,string("SCOPE_TRACEBACK"),0,scope_tracebackKey);
+  const string scope_varnameKey[] = {"COMMON", "COUNT", "LEVEL", KLISTEND};
+  new DLibFun (lib::scope_varname_fun, string ("SCOPE_VARNAME"), -1, scope_varnameKey);
+
 
   const string cpuKey[]={ "RESET","RESTORE","TPOOL_MAX_ELTS", "TPOOL_MIN_ELTS",
 					"TPOOL_NTHREADS","VECTOR_ENABLE",KLISTEND};
-  new DLibPro(lib::cpu,string("CPU"),0,cpuKey);
+  new DLibPro(lib::cpu_pro,string("CPU"),0,cpuKey);
+  
+// removed, see comments in basic_pro.cpp 
+//  const string gdlconfigKey[]={"MAP_QUALITY","GDL_NO_DSFMT","GDL_USE_WX",KLISTEND};
+//  new DLibPro(lib::gdl_config_pro,string("GDL_CONFIG"),0,gdlconfigKey);
 
   const string get_kbrdKey[]={"ESCAPE","KEY_NAME",KLISTEND};
   new DLibFunRetNew(lib::get_kbrd,string("GET_KBRD"),1,NULL,get_kbrdKey);
@@ -119,19 +127,25 @@ void LibInit()
   const string svdcKey[]={"COLUMN","ITMAX","DOUBLE",KLISTEND};
   new DLibPro(lib::svdc,string("SVDC"),4,svdcKey);
 
-  new DLibFunRetNew(lib::temporary,string("TEMPORARY"),1);
+  new DLibFunRetNew(lib::temporary_fun,string("TEMPORARY"),1);
+  
+  new DLibFunRetNew(lib::terminal_size_fun,string("TERMINAL_SIZE"),2);
 
   const string routine_infoKey[]={"FUNCTIONS","SYSTEM","DISABLED","ENABLED",
 				  "PARAMETERS","SOURCE", KLISTEND};
   new DLibFunRetNew(lib::routine_info,string("ROUTINE_INFO"),1,routine_infoKey);
 
+  new DLibFunRetNew(lib::routine_name_fun,string("ROUTINE_NAME_INTERNALGDL"),1);
+  new DLibFunRetNew(lib::routine_dir_fun,string("ROUTINE_DIR"),1);
+
+
+  const string spawnKey[]={ "COUNT","EXIT_STATUS","NOSHELL","NULL_STDIN","PID","STDERR","UNIT", //All platforms
+  "FORCE", "MACCREATOR", "NOCLISYM", "NOLOGNAME", "NOTIFY", //obsoleted and silently ignored.  
 #ifdef _WIN32
-//Please note that NOWAIT and HIDE are WINDOWS-Reserved Keywords.
-  const string spawnKey[] = { "COUNT", "EXIT_STATUS", "PID",
-	  "SH", "NOSHELL", "UNIT", "HIDE", "NOWAIT", KLISTEND };
+	  "HIDE", "LOG_OUTPUT", "NOWAIT",  KLISTEND};
+
 #else
-  const string spawnKey[]={ "COUNT","EXIT_STATUS","PID",
-			    "SH","NOSHELL","UNIT",KLISTEND};
+  "NOTTYRESET","SH",  KLISTEND};
 #endif
  
   new DLibPro(lib::spawn_pro,string("SPAWN"),3,spawnKey);
@@ -147,6 +161,7 @@ void LibInit()
 			       "SWAP_IF_LITTLE_ENDIAN",
 			       "NTOHL","NTOHS","HTONL","HTONS",
 			       "FTOXDR","DTOXDR","XDRTOF","XDRTOD",
+             "DTOGFLOAT","GFLOATTOD", // obsoleted 2  keywords only on the VMS platform
 			       KLISTEND};
   new DLibPro(lib::byteorder,string("BYTEORDER"),-1,byteorderKey);
 
@@ -176,8 +191,10 @@ void LibInit()
                                  "TEST_SYMLINK",
                                  "DIRECTORY","SYMLINK","REGULAR","ZERO_LENGTH",
                                  KLISTEND};
-  new DLibFunRetNew(lib::file_search,string("FILE_SEARCH"),3,file_searchKey);
-
+  new DLibFunRetNew(lib::file_search,string("FILE_SEARCH"),2,file_searchKey); //was 3: is NOT IDL, check this special GDL possibilty ? still useful?
+  const string findfileKey[]={"COUNT",KLISTEND}; //Obsolete but very much used version of file_search! Use C++ code ABSOLUTELY!!
+  new DLibFunRetNew(lib::file_search,string("FINDFILE"),1,findfileKey);
+  
   const string file_expand_pathKey[]={KLISTEND};
   new DLibFunRetNew(lib::file_expand_path,string("FILE_EXPAND_PATH"),1,file_expand_pathKey);
 
@@ -214,6 +231,9 @@ void LibInit()
 			       "NAMED_PIPE","SOCKET","SYMLINK","NOEXPAND_PATH","DANGLING_SYMLINK",KLISTEND};
   new DLibFunRetNew(lib::file_test,string("FILE_TEST"),1,file_testKey);
 
+  const string path_sepKey[]={"PARENT_DIRECTORY","SEARCH_PATH",KLISTEND};
+  new DLibFunRetNew(lib::path_sep,string("PATH_SEP"),0,path_sepKey);
+  
   const string file_basenameKey[]={"FOLD_CASE",KLISTEND};
   new DLibFunRetNew(lib::file_basename,string("FILE_BASENAME"),2,file_basenameKey);
 
@@ -257,6 +277,8 @@ void LibInit()
 
   const string sortKey[]={"L64",KLISTEND};
   new DLibFunRetNew(lib::sort_fun,string("SORT"),1,sortKey,NULL,true);
+  const string gdlsortKey[]={"L64","QUICK","MERGE","RADIX","INSERT","AUTO",KLISTEND}; //,"CHECK"
+  new DLibFunRetNew(lib::gdl_sort_fun,string("GDL_SORT"),1,gdlsortKey,NULL,true);
 
   const string medianKey[]={"EVEN","DOUBLE","DIMENSION",KLISTEND};
   new DLibFunRetNew(lib::median,string("MEDIAN"),2,medianKey);
@@ -303,38 +325,37 @@ void LibInit()
   
   const string memoryKey[]={"CURRENT","HIGHWATER","NUM_ALLOC",
     "NUM_FREE","STRUCTURE","L64",KLISTEND};
-  new DLibFunRetNew(lib::memory, string("MEMORY"), 1, memoryKey, NULL);
+  new DLibFunRetNew(lib::memory_fun, string("MEMORY"), 1, memoryKey, NULL);
 
   // printKey, readKey and stringKey are closely associated
   // as the same functions are called "FORMAT" till "MONTH"
   // must be the first four keywords. The inner print_os function is BASED on this ORDER!
   //NOTE THAT AM_PM, DAYS_OF_WEEK and MONTHS are silently ignored!!!
-  //implied print is a feature introduced in idl8.3 and shared by print/printf, string and fix(type=7)
   #define COMMONKEYWORDSFORSTRINGFORMATTING "FORMAT","AM_PM","DAYS_OF_WEEK","MONTHS"
-  const string printKey[]={COMMONKEYWORDSFORSTRINGFORMATTING, "STDIO_NON_FINITE",KLISTEND};
-  const string impliedprintKey[]={COMMONKEYWORDSFORSTRINGFORMATTING, "STDIO_NON_FINITE","IMPLIED_PRINT",KLISTEND};
-  //At the moment, print accepts silently "IMPLIED_PRINT" as this is used in autoprint feature.
-  new DLibPro(lib::print,string("PRINT"),-1,impliedprintKey);
-  //but PRINTF issues a warning, as it is not yet supported
-  const string printfWarnKey[]={"IMPLIED_PRINT",KLISTEND};
-  new DLibPro(lib::printf,string("PRINTF"),-1,printKey,printfWarnKey);
+  //implied print is a feature introduced in idl8.3 and shared by print/printf, string and fix(type=7) and used in print_os()
+  #define COMMONKEYWORDSFORSTRINGFORMATTINGWITHIMPLIEDPRINT COMMONKEYWORDSFORSTRINGFORMATTING,"IMPLIED_PRINT"
+  const string printOsKey[]={COMMONKEYWORDSFORSTRINGFORMATTINGWITHIMPLIEDPRINT, "STDIO_NON_FINITE",KLISTEND};
+  const string printKey[]={COMMONKEYWORDSFORSTRINGFORMATTINGWITHIMPLIEDPRINT, "STDIO_NON_FINITE","REWRITE"/*obsoleted in 5.3*/,KLISTEND};
+  new DLibPro(lib::print,string("PRINT"),-1,printKey);
+  new DLibPro(lib::printf,string("PRINTF"),-1,printOsKey);
   // allow printing (of expressions) with all keywords 
   // (easier to implement this way)
-  new DLibPro(lib::stop,string("STOP"),-1,printKey); 
+  new DLibPro(lib::stop,string("STOP"),-1,printOsKey); 
 
-  const string readKey[]={COMMONKEYWORDSFORSTRINGFORMATTING,"PROMPT",KLISTEND};
+  const string readKey[]={COMMONKEYWORDSFORSTRINGFORMATTING,"PROMPT"
+    ,"KEY_ID","KEY_MATCH","KEY_VALUE" //obsoleted in 5.3
+    ,KLISTEND};
   new DLibPro(lib::read,string("READ"),-1,readKey);
   new DLibPro(lib::readf,string("READF"),-1,readKey);
 
   const string readsKey[]={COMMONKEYWORDSFORSTRINGFORMATTING,	   KLISTEND}; // no PROMPT
   new DLibPro(lib::reads,string("READS"),-1,readsKey);
 
-  const string stringKey[]={COMMONKEYWORDSFORSTRINGFORMATTING,"PRINT",KLISTEND};
-  const string stringWarnKey[]={"IMPLIED_PRINT",KLISTEND};
-  new DLibFun(lib::string_fun,string("STRING"),-1,stringKey,stringWarnKey);
+  const string stringKey[]={COMMONKEYWORDSFORSTRINGFORMATTINGWITHIMPLIEDPRINT,"PRINT",KLISTEND};
+  new DLibFun(lib::string_fun,string("STRING"),-1,stringKey);
 
-  const string defsysvKey[]={"EXISTS",KLISTEND};
-  new DLibPro(lib::defsysv,string("DEFSYSV"),3,defsysvKey); 
+  const string defsysvKey[]={"EXISTS","TEST",KLISTEND};
+  new DLibPro(lib::defsysv,string("DEFSYSV"),4,defsysvKey); 
 
   const string heap_gcKey[]={"PTR","OBJ","VERBOSE",KLISTEND};
   new DLibPro(lib::heap_gc,string("HEAP_GC"),0,heap_gcKey); 
@@ -360,9 +381,9 @@ void LibInit()
   new DLibFunRetNew(lib::complexarr,string("COMPLEXARR"),MAXRANK,arrKey,NULL,true);
   new DLibFunRetNew(lib::dcomplexarr,string("DCOMPLEXARR"),MAXRANK,arrKey,NULL,true);
 
-  const string ptrArrKey[]={"NOZERO","ALLOCATE_HEAP",KLISTEND};
+  const string ptrArrKey[]={"ALLOCATE_HEAP",KLISTEND};
   new DLibFunRetNew(lib::ptrarr,string("PTRARR"),MAXRANK,ptrArrKey);
-  new DLibFunRetNew(lib::objarr,string("OBJARR"),MAXRANK,arrKey);
+  new DLibFunRetNew(lib::objarr,string("OBJARR"),MAXRANK);
 
   const string ptr_newKey[]={"NO_COPY","ALLOCATE_HEAP",KLISTEND};
   new DLibFunRetNew(lib::ptr_new,string("PTR_NEW"),1,ptr_newKey);
@@ -409,12 +430,15 @@ void LibInit()
 			  "SWAP_ENDIAN","SWAP_IF_BIG_ENDIAN",
 			  "SWAP_IF_LITTLE_ENDIAN" /*12*/,
 			  "VAX_FLOAT","WIDTH","XDR", "BLOCK",
-			  "NOAUTOMODE","BINARY","STREAM",
-			  KLISTEND};
-  const string openWarnKey[]={"INITIALSIZE","EXTENDSIZE",KLISTEND}; // VAX only
-  new DLibPro(lib::openr,string("OPENR"),3,openKey,openWarnKey);
-  new DLibPro(lib::openw,string("OPENW"),3,openKey,openWarnKey);
-  new DLibPro(lib::openu,string("OPENU"),3,openKey,openWarnKey);
+			  "NOAUTOMODE","BINARY"
+       ,"STREAM"// obsolete VMS only
+       ,"DEFAULT","INITIALSIZE","EXTENDSIZE","FIXED","FORTRAN"// obsolete VMS only
+       ,"KEYED","LIST","NONE","PRINT","SEGMENTED","SHARED","SUBMIT","SUPERSED"// obsolete VMS only
+       ,"TRUNCATE_ON_CLOSE","UDF_BLOCK","VARIABLE"// obsolete VMS only
+       ,KLISTEND};
+  new DLibPro(lib::openr,string("OPENR"),3,openKey); //3 as a third arg, "record_length", VMS only, was once possible.
+  new DLibPro(lib::openw,string("OPENW"),3,openKey);
+  new DLibPro(lib::openu,string("OPENU"),3,openKey);
   new DLibPro(lib::get_lun,string("GET_LUN"),1);
 
   const string socketKey[]={"ERROR","GET_LUN","STDIO",
@@ -435,9 +459,12 @@ void LibInit()
   const string free_lunWarnKey[]={"EXIT_STATUS",KLISTEND};
   new DLibPro(lib::free_lun,string("FREE_LUN"),-1,free_lunKey,free_lunWarnKey);
 
-  const string writeuKey[]={"TRANSFER_COUNT",KLISTEND};
+  const string writeuKey[]={"TRANSFER_COUNT","REWRITE"/*obsolete*/,KLISTEND};
+  const string readuKey[]={"TRANSFER_COUNT"
+    ,"KEY_ID", "KAY_MATCH", "KEY_VALUE" // obsoleted in 5.3
+    ,KLISTEND};
   new DLibPro(lib::writeu,string("WRITEU"),-1,writeuKey);
-  new DLibPro(lib::readu,string("READU"),-1,writeuKey);
+  new DLibPro(lib::readu,string("READU"),-1,readuKey);
 
   const string resolve_routineWarnKey[]={"SKIP_EXISTING",KLISTEND};
   const string resolve_routineKey[]={"NO_RECOMPILE","IS_FUNCTION","EITHER","COMPILE_FULL_FILE","QUIET",KLISTEND};
@@ -466,9 +493,8 @@ void LibInit()
   new DLibFunRetNew(lib::dcomplex_fun,string("DCOMPLEX"),MAXRANK+2,NULL,NULL,true);
 */
 // that's apparently the desired bahaviour, see bug no. 3151760
-  const string fixKey[]={"TYPE","PRINT",KLISTEND};
-  const string fixWarnKey[]={"IMPLIED_PRINT",KLISTEND};
-  new DLibFun(lib::fix_fun,string("FIX"),10,fixKey,fixWarnKey);
+  const string fixKey[]={"TYPE","PRINT","IMPLIED_PRINT",KLISTEND}; //Fix never uses IMPLIED_PRINT, but called print_os does.
+  new DLibFun(lib::fix_fun,string("FIX"),10,fixKey);
 
   new DLibFun(lib::uint_fun,string("UINT"),10,NULL,NULL);
   new DLibFun(lib::long_fun,string("LONG"),10,NULL,NULL);
@@ -516,13 +542,14 @@ void LibInit()
   // retConstant: check definition of the rounding functions if they depend 
   // from some sys var (defining a round mode) 
   // (probably nobody rounds a constant anyway)
-  new DLibFunRetNew(lib::round_fun,string("ROUND"),1,roundKey);
-  const string ceilfloorKey[]={"L64",KLISTEND};
-  new DLibFunRetNew(lib::ceil_fun,string("CEIL"),1,ceilfloorKey);
-  new DLibFunRetNew(lib::floor_fun,string("FLOOR"),1,ceilfloorKey);
+  const string roundceilfloorKey[]={"L64",KLISTEND};
+  new DLibFunRetNew(lib::round_fun,string("ROUND"),1,roundceilfloorKey);
+  new DLibFunRetNew(lib::ceil_fun,string("CEIL"),1,roundceilfloorKey);
+  new DLibFunRetNew(lib::floor_fun,string("FLOOR"),1,roundceilfloorKey);
 
   new DLibFunDirect(lib::conj_fun,string("CONJ"));
   new DLibFunDirect(lib::imaginary_fun,string("IMAGINARY"));
+  new DLibFunDirect(lib::real_part_fun,string("REAL_PART"));
 
   const string strcompressKey[]={"REMOVE_ALL",KLISTEND};
   new DLibFunRetNew(lib::strcompress,string("STRCOMPRESS"),1,strcompressKey,NULL,true);
@@ -619,9 +646,13 @@ void LibInit()
       "COLOR","GET_PAGE_SIZE","GET_SCREEN_SIZE","INCHES","WINDOW_STATE","SCALE_FACTOR", 
       "XOFFSET", "YOFFSET", "ENCAPSULATED", "GET_GRAPHICS_FUNCTION", 
       "SET_GRAPHICS_FUNCTION", "CURSOR_STANDARD", "CURSOR_ORIGINAL",
-      "CURSOR_CROSSHAIR","RETAIN", "BITS_PER_PIXEL", 
+      "CURSOR_CROSSHAIR",
+      "CURSOR_IMAGE", //WIN,X
+      "CURSOR_MASK",  //WIN,X
+      "CURSOR_XY",  //WIN,X
+      "RETAIN", "BITS_PER_PIXEL", 
       "GET_WINDOW_POSITION","GET_PIXEL_DEPTH","GET_VISUAL_DEPTH","GET_VISUAL_NAME",
-      "GET_WRITE_MASK", "COPY","GET_FONTNAMES","SET_FONT","GET_CURRENT_FONT","GET_FONTNUM",
+      "GET_WRITE_MASK", "COPY","GET_FONTNAMES","SET_FONT","FONT","GET_CURRENT_FONT","GET_FONTNUM", //"FONT" is idem SET_FONT before IDL5.3
       "SET_PIXEL_DEPTH", //Z
       "SET_CHARACTER_SIZE", //(all)
       KLISTEND
@@ -634,10 +665,8 @@ void LibInit()
   "BOOK", //PS
   "BYPASS_TRANSLATION", //(WIN,X)
   "CMYK", //PS
-  "CURSOR_IMAGE", //WIN,X
-  "CURSOR_MASK",  //WIN,X
-  "CURSOR_XY",  //WIN,X
   "DIRECT_COLOR", //X
+  "DEPTH", //VMS OBSOLETE
   //"EJECT", (HP)
   //"ENCODING", (CGM)
   "FLOYD", //(PCL,X)
@@ -1006,8 +1035,7 @@ void LibInit()
   const string binomialcoefKey[] = {"DOUBLE", KLISTEND };
   new DLibFunRetNew(lib::binomialcoef, string("IMSL_BINOMIALCOEF"), 2, binomialcoefKey);
 
-  // SA: GRIB format support based on the ECMWF GRIB_API package (IDL does not support it yet)
-  // GRIBAPI_ prefix is used in order to (hopefully) prevent future incompatibilities with IDL
+  // SA: GRIB format support using ECMWF ECCODES package (developed prior to IDL GRIB support)
   // -----------------------------------------------------------------------------------------
   // GRIB: file related
   new DLibFunRetNew(lib::grib_open_file_fun, string("GRIBAPI_OPEN_FILE"), 1); 
