@@ -75,7 +75,7 @@ void LibInit_ac()
 
 #endif
 
-  const string spl1Key[]={"YP0","YPN_1","DOUBLE","HELP",KLISTEND};
+  const string spl1Key[]={"YP0","YPN_1","YP1","DOUBLE","HELP",KLISTEND}; //YP1 is old value for YP0
   new DLibFunRetNew(lib::spl_init_fun,string("SPL_INIT"),2,spl1Key);
   const string spl2Key[]={"DOUBLE","HELP",KLISTEND};
   new DLibFunRetNew(lib::spl_interp_fun,string("SPL_INTERP"),4,spl2Key);
