@@ -200,7 +200,8 @@ void LibInit_cl()
   const string magick_readKey[]={"RGB","SUB_RECT","MAP",KLISTEND};
   new DLibFunRetNew(lib::magick_read,string("MAGICK_READ"),1,magick_readKey);
   new DLibFunRetNew(lib::magick_readindexes,string("MAGICK_READINDEXES"),1);
-  new DLibPro(lib::magick_readcolormapRGB,string("MAGICK_READCOLORMAPRGB"),4);
+  const string magick_readcolormapKey[]={"BACKGROUND_COLOR",KLISTEND}; //0
+  new DLibPro(lib::magick_readcolormapRGB,string("MAGICK_READCOLORMAPRGB"),4,magick_readcolormapKey);
   const string magick_writeKey[]={"RGB",KLISTEND};
   new DLibPro(lib::magick_write,string("MAGICK_WRITE"),2,magick_writeKey);
   new DLibPro(lib::magick_writefile,string("MAGICK_WRITEFILE"),3);
@@ -234,7 +235,8 @@ void LibInit_cl()
 
   //hmm
   new DLibPro(lib::magick_display,string("MAGICK_DISPLAY"),1);
-
+  new DLibFunRetNew(lib::magick_type,string("MAGICK_TYPE"),1);
+  new DLibFunRetNew(lib::magick_class,string("MAGICK_CLASS"),1);
   // SA: support fot query/ping routines
   const string magick_pingKey[] = {"INFO", "CHANNELS", "DIMENSIONS", "HAS_PALETTE", "IMAGE_INDEX",
     "NUM_IMAGES", "PIXEL_TYPE",/* "SUPPORTED_READ", "SUPPORTED_WRITE", */"TYPE",KLISTEND};
