@@ -41,16 +41,15 @@
 #include "magick_cl.hpp"
 #include "graphicsdevice.hpp"
 
-#define GDL_DEBUG
-//#undef GDL_DEBUG
-
-// If Magick has not been initialized, do it here, instead of initilizing it in the main program (speedup and avoid strange backtraces)
+// If Magick has not been initialized, do it here, instead of initializing it in the main program (speedup and avoid strange backtraces)
 // Also warn about limitations due to local implementation of Magick library.
 //We should do more, by example hat octave people do in their code (they circumvent some other limitations)
 #define START_MAGICK  \
     if (notInitialized ) { \
       notInitialized = false; \
-        Magick::InitializeMagick(NULL); }
+        Magick::InitializeMagick(NULL);  \
+        if ( QuantumDepth < 16) fprintf(stderr, "%% INFO: the %s library support only 8 bit images.\n", MagickPackageName);\
+              }
 
 namespace lib {
 
@@ -58,18 +57,18 @@ namespace lib {
   using namespace antlr;
   using namespace Magick;
 
-  Image *gImage;
+  Image* gImage[40];
   unsigned int gValid[40];
   unsigned int gCount = 0;
   static bool notInitialized = true;
+  static const std::string imageTypeList[]={"BilevelType","GrayscaleType","GrayscaleMatteType","PaletteType","PaletteMatteType","TrueColorType","TrueColorMatteType","ColorSeparationType","ColorSeparationMatteType","OptimizeType","UnknownType"};
+  static const std::string imageClassList[]={"UndefinedClass","DirectClass","PseudoClass"};
 
-  __attribute__((constructor)) static void init(void) {
-    START_MAGICK;
-    gImage = new Image[40];
-  }
-
-  __attribute__((destructor)) static void destroy(void) {
-    delete[] gImage;
+  __attribute__ ((destructor)) static void destruct(void) { // This is GCC only. Possible memory leak with other compilers.
+    int i;
+    for (i = 0; i < 40; ++i)
+      if (gValid[i] == 1)
+        delete gImage[i];
   }
 
   void magick_setup() {
@@ -77,22 +76,17 @@ namespace lib {
     for (i = 0; i < 40; ++i) gValid[i] = 0;
   }
 
-  Image& magick_image(EnvT *e, unsigned int mid) {
+  Image *magick_image(EnvT *e, unsigned int mid) {
     if (gValid[mid] == 0) e->Throw("invalid ID.");
-
     return gImage[mid];
   }
 
-  unsigned int magick_image(EnvT* e, Image &imImage) {
+  unsigned int magick_image(EnvT* e, Image *imImage) {
     unsigned int mid = magick_id();
     gImage[mid] = imImage;
     return mid;
   }
 
-  void magick_replace(EnvT* e, unsigned int mid, Image &imImage) {
-    gImage[mid] = imImage;
-  }
-
   unsigned int magick_id(void) {
     unsigned int i, val;
     val = gCount;
@@ -117,14 +111,14 @@ namespace lib {
 
       if (filename.length() == 0) e->Throw("Void file Name");
 
-      Image a;
+      Image *a = new Image();
       try {
-        a.read(filename);
+        a->read(filename);
       } catch (WarningCoder &warning_) {
         cerr << warning_.what() << endl;
       }
-      if ((a.rows() * a.columns()) == 0) e->Throw("Error reading image dimensions!");
-      a.flip();
+      if ((a->rows() * a->columns()) == 0) e->Throw("Error reading image dimensions!");
+      a->flip(); //necessary 
       unsigned int mid;
       mid = magick_image(e, a);
       return new DUIntGDL(mid);
@@ -133,7 +127,7 @@ namespace lib {
     }
     return NULL; //pacify -Wreturn-type
   }
-
+//  a=magick_ping(name,type,"INFO", "CHANNELS", "DIMENSIONS", "HAS_PALETTE", "IMAGE_INDEX","NUM_IMAGES", "PIXEL_TYPE",/* "SUPPORTED_READ", "SUPPORTED_WRITE", */"TYPE")
   BaseGDL * magick_ping(EnvT* e) {
     // TODO!
     //if (e->KeywordPresent("SUPPORTED_READ") || e->KeywordPresent("SUPPORTED_WRITE"))
@@ -149,11 +143,11 @@ namespace lib {
       if (filename.length() == 0) return new DLongGDL(0);
 
       WordExp(filename);
-
-      Image a;
+      
+      Image *a = new Image;
       try {
-        a.ping(filename);
-        //a.read(filename);
+        a->ping(filename);
+        //a->read(filename);
       } catch (WarningCoder &warning_) {
         cerr << warning_.what() << endl;
       }
@@ -161,70 +155,70 @@ namespace lib {
       if (nParam == 2) {
         DString magick;
         e->AssureScalarPar<DStringGDL>(1, magick);
-        if (a.magick() != magick) return new DLongGDL(0);
+        if (a->magick() != magick) return new DLongGDL(0);
       }
 
       int debug = 0;
       if (debug == 1) {
-        cout << "a.type()      :" << a.type() << endl;
-        cout << "a.classType() :" << a.classType() << endl;
-        cout << "a.matte()     :" << a.matte() << endl;
-        // no useful info here:cout << "a.colorSpace()     :" << a.colorSpace() << endl;
-        // Always 8:cout << "a.depth()     :" << a.depth() << endl;
-        // Always 1: cout << "a.colorSpace() :" << a.colorSpace() << endl;
+        cout << "a->type()      :" << a->type() << endl;
+        cout << "a->classType() :" << a->classType() << endl;
+        cout << "a->matte()     :" << a->matte() << endl;
+        // no useful info here:cout << "a->colorSpace()     :" << a->colorSpace() << endl;
+        // Always 8:cout << "a->depth()     :" << a->depth() << endl;
+        // Always 1: cout << "a->colorSpace() :" << a->colorSpace() << endl;
       }
 
       // AC 2012-May-10
       // http://www.graphicsmagick.org/Magick++/Image.html#type
       // relevant information that, in some cases, is provided after pinging:
-      // a.matte(), a.classType() [and a.type() for Palette info only]
+      // a->matte(), a->classType() [and a->type() for Palette info only]
       DLong channels = 0;
-      if (a.classType() == 1) channels = 3; // DirectColor
-      if (a.classType() == 2) channels = 1; // PseudoColor
+      if (a->classType() == 1) channels = 3; // DirectColor
+      if (a->classType() == 2) channels = 1; // PseudoColor
       if (channels == 0) cout << "no ClassType found for current Image" << endl;
 
-      // AC 2012-May-10 this is NOT working with only a a.ping()
-      // a.type() is FULLY reliable if and only if a.read() was done before !!! 
+      // AC 2012-May-10 this is NOT working with only a a->ping()
+      // a->type() is FULLY reliable if and only if a->read() was done before !!! 
       //http://www.graphicsmagick.org/Magick++/Image.html#type
-      // (should be OK with a a.read())
+      // (should be OK with a a->read())
       /*
-      channels = a.classType() == PseudoClass 
+      channels = a->classType() == PseudoClass 
         ? 1      // color palette
-        : a.type() == GrayscaleType 
+        : a->type() == GrayscaleType 
           ? 1    // greyscale
-          : a.type() == ColorSeparationType 
+          : a->type() == ColorSeparationType 
             ? 4  // CMYK
             : 3; // RGB
        */
 
       // AC 2012-May-10 this is OK (reliable), see example "589 Lavandula mono"
-      if (a.matte()) channels += 1;
+      if (a->matte()) channels += 1;
 
       // TODO! multiple images (using the Magick++ STL interface)
       DLong image_index, num_images;
       image_index = 0;
-      num_images = 1;
+      num_images = 1; //a->scene()+1;
 
       DInt pixel_type;
-      pixel_type = a.depth() == 16 ? 2 : 1;
+      pixel_type = a->depth() == 16 ? 2 : 1;
 
       // AC 2012-May-10 Palette only if type == 4 OR 5
-      // Despite Type is NOT useful without a a.read(), it is OK for Palette !
+      // Despite Type is NOT useful without a a->read(), it is OK for Palette !
       // This should be reliable (OK with ImageMagick AND GraphicsMagick)
       DInt has_palette = 0;
-      if ( (a.type() == PaletteType) || (a.type() == PaletteMatteType) ) has_palette = 1;
+      if ( (a->type() == PaletteType) || (a->type() == PaletteMatteType) ) has_palette = 1;
 
       DString type;
-      type = a.magick() == "PNM" ? "PPM" :
-        a.magick() == "PGM" ? "PPM" :
-        a.magick() == "DCM" ? "DICOM" :
-        a.magick() == "JPC" ? "JPEG2000" :
-        a.magick() == "JP2" ? "JPEG2000" :
-        a.magick() == "JNG" ? "JPEG2000" :
-        a.magick();
-
-      if (debug == 1) cout << "Type (via a.magick()) : " << type << endl;
+      type = a->magick() == "PNM" ? "PPM" :
+        a->magick() == "PGM" ? "PPM" :
+        a->magick() == "DCM" ? "DICOM" :
+        a->magick() == "JPC" ? "JPEG2000" :
+        a->magick() == "JP2" ? "JPEG2000" :
+        a->magick() == "JNG" ? "JPEG2000" :
+        a->magick();
 
+      if (debug == 1) cout << "Type (via a->magick()) : " << type << endl;
+      bool addSBit=(a->magick() == "PNG"); //only one type?
       static int infoIx = e->KeywordIx("INFO");
       if (e->KeywordPresent(infoIx)) {
         e->AssureGlobalKW(infoIx);
@@ -235,12 +229,14 @@ namespace lib {
         SpDLong aLong;
         SpDInt aInt;
         SpDLong aLongArr2(dimension(2));
+        SpDByte aByteArr2(dimension(5));
         info_desc->AddTag("CHANNELS", &aLong);
         info_desc->AddTag("DIMENSIONS", &aLongArr2);
         info_desc->AddTag("HAS_PALETTE", &aInt);
         info_desc->AddTag("IMAGE_INDEX", &aLong);
         info_desc->AddTag("NUM_IMAGES", &aLong);
         info_desc->AddTag("PIXEL_TYPE", &aInt);
+        if (addSBit) info_desc->AddTag("SBIT_VALUES", &aByteArr2);
         info_desc->AddTag("TYPE", &aString);
         DStructGDL* info = new DStructGDL(info_desc, dimension());
 
@@ -248,8 +244,8 @@ namespace lib {
         info->InitTag("CHANNELS", DLongGDL(channels));
         {
           DLongGDL dims(dimension(2));
-          dims[0] = a.columns();
-          dims[1] = a.rows();
+          dims[0] = a->columns();
+          dims[1] = a->rows();
           info->InitTag("DIMENSIONS", dims);
         }
         info->InitTag("HAS_PALETTE", DIntGDL(has_palette));
@@ -270,8 +266,8 @@ namespace lib {
       if (e->KeywordPresent(dimensionsIx)) {
         e->AssureGlobalKW(dimensionsIx);
         DLongGDL *dims = new DLongGDL(dimension(2));
-        (*dims)[0] = a.columns();
-        (*dims)[1] = a.rows();
+        (*dims)[0] = a->columns();
+        (*dims)[1] = a->rows();
         e->SetKW(dimensionsIx, dims);
       }
 
@@ -325,15 +321,15 @@ namespace lib {
       g.height(rows);
       if (nParam == 3) { //truecolor
         e->AssureScalarPar<DStringGDL>(2, col);
-        Image a(g, Color(col));
+        Image *a = new Image(g, Color(col));
         unsigned int mid;
-        a.matte(false);
+        a->matte(false);
         mid = magick_image(e, a);
         return new DUIntGDL(mid);
       } else {
-        Image a(g, Color("black"));
+        Image *a = new Image(g, Color("black"));
         unsigned int mid;
-        a.matte(false);
+        a->matte(false);
         mid = magick_image(e, a);
         return new DUIntGDL(mid);
       }
@@ -356,7 +352,7 @@ namespace lib {
       else if (gValid[mid] == 0) e->Throw("ID not used");
 
       gValid[mid] = 0;
-      gImage[mid] = NULL;
+      delete gImage[mid];
       if (gCount - 1 == mid) gCount--;
     } catch (Exception &error_) {
       e->Throw(error_.what());
@@ -370,46 +366,35 @@ namespace lib {
       DUInt mid;
       e->AssureScalarPar<DUIntGDL>(0, mid);
       unsigned int columns, rows;
-      Image image = magick_image(e, mid);
-      if (image.classType() == DirectClass)
-        e->Throw("Not an indexed image: " + e->GetParString(0));
+      Image *image = magick_image(e, mid);
+      if (image->classType() != PseudoClass) e->Throw("Not an indexed image: " + e->GetParString(0));
 
-      columns = image.columns();
-      rows = image.rows();
-
-      if (image.matte() == 0) {
-
-        SizeT c[2];
-        c[0] = columns;
-        c[1] = rows;
-        dimension dim(c, 2);
-        DByteGDL *bImage = new DByteGDL(dim, BaseGDL::NOZERO);
-
-//        const PixelPacket* pixel;
-        const IndexPacket* index;
-//        pixel = image.getPixels(0, 0, columns, rows);
-        index = image.getIndexes();
-
-        if (index == NULL) {
-          string txt = "Warning -- Magick's getIndexes() returned NULL for: ";
-          string txt2 = ", using unsafe patch.";
-          //PATCH to get something until we understand what's going on
-          cerr << (txt + e->GetParString(0) + txt2) << endl;
-          string map = "R";
-          image.write(0, 0, columns, rows, map, CharPixel, &(*bImage)[0]);
-          return bImage;
-        }
+      columns = image->columns();
+      rows = image->rows();
+      SizeT c[2];
+      c[0] = columns;
+      c[1] = rows;
+      dimension dim(c, 2);
+#ifdef USE_MAGICK6
+      if (image->modulusDepth() <= 8)
+#else
+      if (image->depth() <= 8)
+#endif
+      {
+        DByteGDL *bImage = new DByteGDL(dim, BaseGDL::ZERO);
+        PixelPacket *pixel_cache = image->getPixels(0,0,columns,rows); //magick command, without it writePixels do NOTHING!
+        image->writePixels(IndexQuantum,(unsigned char*)(bImage->DataAddr()));
         return bImage;
-      } else {
-        // we do have to manage an extra channel for transparency
-        string map = "RA";
-        SizeT c[3];
-        c[0] = map.length(); // see code "magick_read" below
-        c[1] = columns;
-        c[2] = rows;
-        dimension dim(c, 3);
-        DByteGDL *bImage = new DByteGDL(dim, BaseGDL::NOZERO);
-        image.write(0, 0, columns, rows, map, CharPixel, &(*bImage)[0]);
+      }
+#ifdef USE_MAGICK6
+      else if (image->modulusDepth() <= 16)
+#else     
+      else if (image->depth() <= 16)
+#endif
+      {
+        DUIntGDL *bImage = new DUIntGDL(dim, BaseGDL::NOZERO);
+        PixelPacket *pixel_cache = image->getPixels(0,0,columns,rows); //magick command, without it writePixels do NOTHING!
+        image->writePixels(IndexQuantum,(unsigned char*)(bImage->DataAddr())); //probably not good, impossible to test yet.
         return bImage;
       }
     } catch (Exception &error_) {
@@ -425,64 +410,65 @@ namespace lib {
       size_t nParam = e->NParam(1);
       DUInt mid;
       e->AssureScalarPar<DUIntGDL>(0, mid);
-      Image image = magick_image(e, mid);
-      if (image.classType() == DirectClass)
-        e->Throw("Not an indexed image: " + e->GetParString(0));
-
-      if (image.classType() == PseudoClass) {
-        unsigned int Quant, scale, i;
-        if (QuantumDepth == 16) Quant = 65535;
-        if (QuantumDepth == 8) Quant = 255;
-
-        unsigned int cmapsize = image.colorMapSize();
+      Image* image = magick_image(e, mid);
+      bool bgnotfound=true;
+      
+      if (image->classType() == PseudoClass) {
+        //retrieve background color
+        Color bg=image->backgroundColor();
+        unsigned int cmapsize = image->colorMapSize();
         dimension cmap(cmapsize, 1);
         Color col;
 
 #ifdef USE_MAGICK6
-        if (image.modulusDepth() <= 8)
+        if (image->modulusDepth() <= 8)
 #else
-        if (image.depth() <= 8)
+        if (image->depth() <= 8)
 #endif
         {
+          DByteGDL* bgIndex=new DByteGDL(0);
 
-          scale = MaxRGB;
           DByteGDL *R, *G, *B;
 
           R = new DByteGDL(cmap, BaseGDL::NOZERO);
           G = new DByteGDL(cmap, BaseGDL::NOZERO);
           B = new DByteGDL(cmap, BaseGDL::NOZERO);
 
-          for (i = 0; i < cmapsize; ++i) {
-            col = image.colorMap(i);
-            (*R)[i] = (col.redQuantum()) ; //* scale / Quant;
-            (*G)[i] = (col.greenQuantum()) ; // * scale / Quant;
-            (*B)[i] = (col.blueQuantum()) ; //* scale / Quant;
+          for (int i = 0; i < cmapsize; ++i) {
+            col = image->colorMap(i);
+            (*R)[i] = (col.redQuantum()) ;
+            (*G)[i] = (col.greenQuantum()) ;
+            (*B)[i] = (col.blueQuantum()) ;
+            if (bgnotfound && (col==bg) ) {bgnotfound=false; (*bgIndex)[0]=i;}
           }
           if (nParam > 1) e->SetPar(1, R);
           if (nParam > 2) e->SetPar(2, G);
           if (nParam > 3) e->SetPar(3, B);
+          if (e->KeywordPresent(0)) {e->AssureGlobalKW(0); e->SetKW(0, bgIndex);}
         }
 #ifdef USE_MAGICK6
-        else if (image.modulusDepth() <= 16)
+        else if (image->modulusDepth() <= 16)
 #else     
-        else if (image.depth() <= 16)
+        else if (image->depth() <= 16)
 #endif
         {
-          scale = MaxRGB;
+          DUIntGDL* bgIndex=new DUIntGDL(0);
           DUIntGDL *R, *G, *B;
           R = new DUIntGDL(cmap, BaseGDL::NOZERO);
           G = new DUIntGDL(cmap, BaseGDL::NOZERO);
           B = new DUIntGDL(cmap, BaseGDL::NOZERO);
 
-          for (i = 0; i < cmapsize; ++i) {
-            col = image.colorMap(i);
-            (*R)[i] = (col.redQuantum()) ; //* scale / Quant;
-            (*G)[i] = (col.greenQuantum()) ; //* scale / Quant;
-            (*B)[i] = (col.blueQuantum()) ; //* scale / Quant;
+          for (int i = 0; i < cmapsize; ++i) {
+            col = image->colorMap(i);
+            (*R)[i] = (col.redQuantum()) ; 
+            (*G)[i] = (col.greenQuantum()) ;
+            (*B)[i] = (col.blueQuantum()) ; 
+            if (bgnotfound && col==bg ) {bgnotfound=false; (*bgIndex)[0]=i;}
           }
           if (nParam > 1) e->SetPar(1, R);
           if (nParam > 2) e->SetPar(2, G);
           if (nParam > 3) e->SetPar(3, B);
+          if (e->KeywordPresent(0)) {e->AssureGlobalKW(0); e->SetKW(0, bgIndex);}
         } else {
           e->Throw("Unknown Image type, too many colors");
         }
@@ -504,18 +490,18 @@ namespace lib {
       DUInt mid;
       e->AssureScalarPar<DUIntGDL>(0, mid);
       unsigned int columns, rows, lx, ly, wx, wy;
-      Image image = magick_image(e, mid);
+      Image* image = magick_image(e, mid);
 
-      columns = image.columns();
-      rows = image.rows();
+      columns = image->columns();
+      rows = image->rows();
       if ((rows * columns) == 0) e->Throw("Error reading image dimensions!");
       string map = "RGB";
-      if (image.matte()) map = map + "A";
+      if (image->matte()) map = map + "A";
 
       if (e->GetKW(0) != NULL)//RGB
       {
         DInt rgb;
-        e->AssureScalarKW<DIntGDL>(0, rgb);
+        e->ProvideScalarKW<DIntGDL>(0, rgb);
         if (rgb == 0) map = "BGR";
         else if (rgb == 1) map = "RGB";
         else if (rgb == 2) map = "RBG";
@@ -560,13 +546,13 @@ namespace lib {
       dim << wx;
       dim << wy;
 
-      if (image.depth() == 8) {
+      if (image->depth() == 8) {
         DByteGDL *bImage = new DByteGDL(dim, BaseGDL::NOZERO);
-        image.write(lx, ly, wx, wy, map, CharPixel, &(*bImage)[0]);
+        image->write(lx, ly, wx, wy, map, CharPixel, &(*bImage)[0]);
         return bImage;
-      } else if (image.depth() == 16) {
+      } else if (image->depth() == 16) {
         DUIntGDL* iImage = new DUIntGDL(dim, BaseGDL::NOZERO);
-        image.write(lx, ly, wx, wy, map, ShortPixel, &(*iImage)[0]);
+        image->write(lx, ly, wx, wy, map, ShortPixel, &(*iImage)[0]);
         return iImage;
       } else {
         e->Throw("Unsupported bit depth");
@@ -587,7 +573,7 @@ namespace lib {
 
       DUInt mid;
       e->AssureScalarPar<DUIntGDL>(0, mid);
-      Image image = magick_image(e, mid);
+      Image* image = magick_image(e, mid);
       int columns, rows, planes;
       //	StorageType ty;
       //	ty=CharPixel;
@@ -604,7 +590,7 @@ namespace lib {
           if (e->GetKW(0) != NULL)//RGB
           {
             DInt rgb;
-            e->AssureScalarKW<DIntGDL>(0, rgb);
+            e->ProvideScalarKW<DIntGDL>(0, rgb);
 
             if (rgb == 0) map = "BGR";
             else if (rgb == 1) map = "RGB";
@@ -619,28 +605,20 @@ namespace lib {
               Message(s);
               map = "BGR";
             }
-            if (image.matte()) map = map + "A";
+            if (image->matte()) map = map + "A";
           }
         }
-
-        DByteGDL * bImage =
-          static_cast<DByteGDL*> (GDLimage->Convert2(GDL_BYTE, BaseGDL::COPY));
-        Guard<DByteGDL> bImageGuard(bImage);
-
-        image.read(columns, rows, map, CharPixel, &(*bImage)[0]);
-        /*	      }
-        else if(image.depth() == 16)
-          {
-        DUIntGDL * iImage=
-          static_cast<DUIntGDL*>(GDLimage->Convert2(GDL_UINT,BaseGDL::COPY));
-
-        image.read(columns,rows,map, ShortPixel,&(*iImage)[0]);
-          }
-        else
-          {
-        e->Throw("Unsupported bit depth");
-        }*/
-
+        if (image->depth() == 8) {
+          DByteGDL * bImage = static_cast<DByteGDL*> (GDLimage->Convert2(GDL_BYTE, BaseGDL::COPY));
+          Guard<DByteGDL> bImageGuard(bImage);
+          image->read(columns, rows, map, CharPixel, &(*bImage)[0]);
+        } else if (image->depth() == 16) {
+          DUIntGDL * iImage = static_cast<DUIntGDL*> (GDLimage->Convert2(GDL_UINT, BaseGDL::COPY));
+          Guard<DUIntGDL> iImageGuard(iImage);
+          image->read(columns, rows, map, ShortPixel, &(*iImage)[0]);
+        } else {
+          e->Throw("Unsupported bit depth");
+        }
       } else {
         columns = GDLimage->Dim(0);
         rows = GDLimage->Dim(1);
@@ -648,18 +626,18 @@ namespace lib {
           static_cast<DByteGDL*> (GDLimage->Convert2(GDL_BYTE, BaseGDL::COPY));
         Guard<DByteGDL> bImageGuard(bImage);
         // Ensure that there are no other references to this image.
-        image.modifyImage();
+        image->modifyImage();
         // Set the image type to Palette.
-        image.type(PaletteType);
+        image->type(PaletteType);
 //
-        if (image.colorMapSize() < 1) e->Throw("GDL internal: destination image has no colormap!");
-        image.size(Geometry(columns, rows));
-        image.setPixels(0,0,columns, rows);
-        image.readPixels(IndexQuantum,(unsigned char*)bImage->DataAddr());
-        image.syncPixels();
+        if (image->colorMapSize() < 1) e->Throw("GDL internal: destination image has no colormap!");
+        image->size(Geometry(columns, rows));
+        image->setPixels(0,0,columns, rows);
+        image->readPixels(IndexQuantum,(unsigned char*)bImage->DataAddr());
+        image->syncPixels();
       }
-//      image.flip();
-      magick_replace(e, mid, image);
+      image->flip();
+      //magick_replace(e, mid, image);
     } catch (Exception &error_) {
       e->Throw(error_.what());
     }
@@ -672,7 +650,7 @@ namespace lib {
       size_t nParam = e->NParam(2);
       DUInt mid;
       e->AssureScalarPar<DUIntGDL>(0, mid);
-      Image image = magick_image(e, mid);
+      Image* image = magick_image(e, mid);
 
       DString filename;
       e->AssureScalarPar<DStringGDL>(1, filename);
@@ -681,10 +659,10 @@ namespace lib {
       if (nParam == 3) {
         DString imagetype;
         e->AssureScalarPar<DStringGDL>(2, imagetype);
-        image.magick(imagetype);
+        image->magick(imagetype);
       }
-      image.write(filename);
-      magick_replace(e, mid, image);
+      image->write(filename);
+      //magick_replace(e, mid, image);
     } catch (Exception &error_) {
       e->Throw(error_.what());
     }
@@ -698,16 +676,16 @@ namespace lib {
     try {
       DUInt mid;
       e->AssureScalarPar<DUIntGDL>(0, mid);
-      Image image = magick_image(e, mid);
+      Image* image = magick_image(e, mid);
       size_t nParam = e->NParam(1);
-      if (nParam == 2) {
+      if (nParam == 2) { //to be tested, works?
         DUInt ncol;
         e->AssureScalarPar<DUIntGDL>(1, ncol);
-        image.colorMapSize(ncol);
-        magick_replace(e, mid, image);
+        image->colorMapSize(ncol);
+        //magick_replace(e, mid, image);
       }
 
-      return new DLongGDL(image.colorMapSize());
+      return new DLongGDL(image->colorMapSize());
     } catch (Exception &error_) {
       return new DLongGDL(-1);
     }
@@ -720,16 +698,16 @@ namespace lib {
     try {
       DUInt mid;
       e->AssureScalarPar<DUIntGDL>(0, mid);
-      Image image = magick_image(e, mid);
+      Image* image = magick_image(e, mid);
       size_t nParam = e->NParam(1);
       if (nParam == 2) {
         DString format;
         e->AssureScalarPar<DStringGDL>(1, format);
-        image.magick(format);
-        magick_replace(e, mid, image);
+        image->magick(format);
+        //magick_replace(e, mid, image);
       }
 
-      return new DStringGDL(image.magick());
+      return new DStringGDL(image->magick());
     } catch (Exception &error_) {
       e->Throw(error_.what());
     }
@@ -742,8 +720,8 @@ namespace lib {
     try {
       DUInt mid;
       e->AssureScalarPar<DUIntGDL>(0, mid);
-      Image image = magick_image(e, mid);
-      return new DLongGDL(image.rows());
+      Image* image = magick_image(e, mid);
+      return new DLongGDL(image->rows());
     } catch (Exception &error_) {
       e->Throw(error_.what());
     }
@@ -756,8 +734,8 @@ namespace lib {
     try {
       DUInt mid;
       e->AssureScalarPar<DUIntGDL>(0, mid);
-      Image image = magick_image(e, mid);
-      return new DLongGDL(image.columns());
+      Image* image = magick_image(e, mid);
+      return new DLongGDL(image->columns());
     } catch (Exception &error_) {
       e->Throw(error_.what());
     }
@@ -769,11 +747,11 @@ namespace lib {
     try {
       DUInt mid;
       e->AssureScalarPar<DUIntGDL>(0, mid);
-      Image image = magick_image(e, mid);
+      Image* image = magick_image(e, mid);
 
-      if (image.classType() == DirectClass)
+      if (image->classType() == DirectClass)
         return new DIntGDL(0);
-      if (image.classType() == PseudoClass)
+      if (image->classType() == PseudoClass)
         return new DIntGDL(1);
     } catch (Exception &error_) {
       e->Throw(error_.what());
@@ -793,9 +771,9 @@ namespace lib {
       if (nParam == 2) q = 75;
       //check before we do anything.
 
-      Image image = magick_image(e, mid);
-      image.quality(q);
-      magick_replace(e, mid, image);
+      Image* image = magick_image(e, mid);
+      image->quality(q);
+      //magick_replace(e, mid, image);
     } catch (Exception &error_) {
       e->Throw(error_.what());
     }
@@ -809,9 +787,9 @@ namespace lib {
     try {
       DUInt mid;
       e->AssureScalarPar<DUIntGDL>(0, mid);
-      Image image = magick_image(e, mid);
-      image.flip();
-      magick_replace(e, mid, image);
+      Image* image = magick_image(e, mid);
+      image->flip();
+      //magick_replace(e, mid, image);
     } catch (Exception &error_) {
       e->Throw(error_.what());
     }
@@ -822,10 +800,10 @@ namespace lib {
     try {
       DUInt mid;
       e->AssureScalarPar<DUIntGDL>(0, mid);
-      Image image = magick_image(e, mid);
-      image.matte(true);
+      Image* image = magick_image(e, mid);
+      image->matte(true);
 
-      magick_replace(e, mid, image);
+      //magick_replace(e, mid, image);
     } catch (Exception &error_) {
       e->Throw(error_.what());
     }
@@ -836,12 +814,12 @@ namespace lib {
     try {
       DUInt mid, index;
       e->AssureScalarPar<DUIntGDL>(0, mid);
-      Image image = magick_image(e, mid);
+      Image* image = magick_image(e, mid);
       e->AssureScalarPar<DUIntGDL>(1, index);
-      if (index >= 0 && index < image.colorMapSize()) {
-        image.transparent(image.colorMap(index));
+      if (index >= 0 && index < image->colorMapSize()) {
+        image->transparent(image->colorMap(index));
 
-        magick_replace(e, mid, image);
+        //magick_replace(e, mid, image);
       }
     } catch (Exception &error_) {
       e->Throw(error_.what());
@@ -853,7 +831,7 @@ namespace lib {
     try {
       DUInt mid;
       e->AssureScalarPar<DUIntGDL>(0, mid);
-      Image image = magick_image(e, mid);
+      Image* image = magick_image(e, mid);
 
 
       //NoInterlace.......Don't interlace image (RGBRGBRGBRGBRGBRGB...)
@@ -861,13 +839,13 @@ namespace lib {
       //PlaneInterlace....Use plane interlacing (RRRRRR...GGGGGG...BBBBBB...)
 
       if (e->KeywordSet(0))//NoInterlace
-        image.interlaceType(NoInterlace);
+        image->interlaceType(NoInterlace);
       else if (e->KeywordSet(1))//LineInterlace
-        image.interlaceType(LineInterlace);
+        image->interlaceType(LineInterlace);
       else if (e->KeywordSet(2))//PlaneInterlace
-        image.interlaceType(PlaneInterlace);
+        image->interlaceType(PlaneInterlace);
 
-      magick_replace(e, mid, image);
+      //magick_replace(e, mid, image);
     } catch (Exception &error_) {
       e->Throw(error_.what());
     }
@@ -879,42 +857,42 @@ namespace lib {
     try {
       DUInt mid;
       e->AssureScalarPar<DUIntGDL>(0, mid);
-      Image image = magick_image(e, mid);
+      Image* image = magick_image(e, mid);
 
       if (e->KeywordSet(0))//Uniform noise
-        image.addNoise(UniformNoise);
+        image->addNoise(UniformNoise);
       else if (e->KeywordSet(1))//Gaussian noise
-        image.addNoise(GaussianNoise);
+        image->addNoise(GaussianNoise);
       else if (e->KeywordSet(2))//Multiplicative Gaussian noise
-        image.addNoise(MultiplicativeGaussianNoise);
+        image->addNoise(MultiplicativeGaussianNoise);
       else if (e->KeywordSet(3))//Impulse noise
-        image.addNoise(ImpulseNoise);
+        image->addNoise(ImpulseNoise);
       else if (e->KeywordSet(4))//Laplacian noise
-        image.addNoise(LaplacianNoise);
+        image->addNoise(LaplacianNoise);
       else if (e->KeywordSet(5))//Poisson noise
-        image.addNoise(PoissonNoise);
+        image->addNoise(PoissonNoise);
       else if (e->GetKW(6) != NULL) {
         DInt noise;
-        e->AssureScalarKW<DIntGDL>(6, noise);
+        e->ProvideScalarKW<DIntGDL>(6, noise);
 
         if (noise == 0)//Uniform noise
-          image.addNoise(UniformNoise);
+          image->addNoise(UniformNoise);
         else if (noise == 1)//Gaussian noise
-          image.addNoise(GaussianNoise);
+          image->addNoise(GaussianNoise);
         else if (noise == 2)//Multiplicative Gaussian noise
-          image.addNoise(MultiplicativeGaussianNoise);
+          image->addNoise(MultiplicativeGaussianNoise);
         else if (noise == 3)//Impulse noise
-          image.addNoise(ImpulseNoise);
+          image->addNoise(ImpulseNoise);
         else if (noise == 4)//Laplacian noise
-          image.addNoise(LaplacianNoise);
+          image->addNoise(LaplacianNoise);
         else if (noise == 5)//Poisson noise
-          image.addNoise(PoissonNoise);
+          image->addNoise(PoissonNoise);
         else
           e->Throw("Unknown noise type requested.");
       } else//no keyword
-        image.addNoise(UniformNoise);
+        image->addNoise(UniformNoise);
 
-      magick_replace(e, mid, image);
+      //magick_replace(e, mid, image);
     } catch (Exception &error_) {
       e->Throw(error_.what());
     }
@@ -927,9 +905,10 @@ namespace lib {
 
       DUInt mid;
       e->AssureScalarPar<DUIntGDL>(0, mid);
-      Image image = magick_image(e, mid);
+      Image* image = magick_image(e, mid);
+      image->modifyImage();
       //set the number of colors;
-      DLong ncol = 256;
+      DLong ncol = (image->depth()<16)?256:65635;
       if (nParam > 1) e->AssureLongScalarPar(1, ncol);
 
       static int TRUECOLORIx = e->KeywordIx("TRUECOLOR");
@@ -940,24 +919,37 @@ namespace lib {
       static int GRAYSCALEIx = e->KeywordIx("GRAYSCALE");
       
         if (e->KeywordSet(YUVIx)) //YUV
-          image.quantizeColorSpace(YUVColorspace);
+          image->quantizeColorSpace(YUVColorspace);
         else if (e->KeywordSet(GRAYSCALEIx)) //Grayscale
-          image.quantizeColorSpace(GRAYColorspace);
+          image->quantizeColorSpace(GRAYColorspace);
         else
-          image.quantizeColorSpace(RGBColorspace);
+          image->quantizeColorSpace(RGBColorspace);
 
-//      image.colorMapSize(ncol);
-//      image.classType(PseudoClass);
-//      image.type(PaletteType);
-      image.quantizeColors(ncol);
-      image.quantizeDither(dither);
-      image.quantize();
-//      image.syncPixels();
-      magick_replace(e, mid, image);
+      image->quantizeColors(ncol);
+      image->quantizeDither(dither);
+      image->quantize();
+      //magick_replace(e, mid, image);
     } catch (Exception &error_) {
       e->Throw(error_.what());
     }
   }
+  
+  //return type
+  BaseGDL* magick_type(EnvT* e){
+    START_MAGICK;
+    DUInt mid;
+    e->AssureScalarPar<DUIntGDL>(0, mid);
+    Image* image = magick_image(e, mid);
+    return new DStringGDL(imageTypeList[image->type()]);
+  }
+  //return class
+  BaseGDL* magick_class(EnvT* e){
+    START_MAGICK;
+    DUInt mid;
+    e->AssureScalarPar<DUIntGDL>(0, mid);
+    Image* image = magick_image(e, mid);
+    return new DStringGDL(imageClassList[image->classType()]);
+  }
 
   //MAGICK_DISPLAY,mid
 
@@ -965,8 +957,8 @@ namespace lib {
     START_MAGICK;
     DUInt mid;
     e->AssureScalarPar<DUIntGDL>(0, mid);
-    Image image = magick_image(e, mid);
-    image.display();
+    Image* image = magick_image(e, mid);
+    image->display();
 
   }
 
@@ -978,17 +970,17 @@ namespace lib {
       BaseGDL* GDLimage = e->GetParDefined(1);
       DByteGDL * bImage = static_cast<DByteGDL*> (GDLimage->Convert2(GDL_BYTE, BaseGDL::COPY));
 
-      Image image = magick_image(e, mid);
+      Image* image = magick_image(e, mid);
 
 //      const PixelPacket* pixels;
       IndexPacket* index;
 
       unsigned int columns, rows;
-      columns = image.columns();
-      rows = image.rows();
+      columns = image->columns();
+      rows = image->rows();
 
-//      pixels = image.setPixels(0, 0, columns, rows);
-      index = image.getIndexes();
+//      pixels = image->setPixels(0, 0, columns, rows);
+      index = image->getIndexes();
 
       SizeT nEl = columns*rows;
       // #pragma omp parallel if (nEl >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nEl))
@@ -1000,9 +992,9 @@ namespace lib {
                   index++;*/
         }
       }
-      image.syncPixels();
+      image->syncPixels();
 
-      magick_replace(e, mid, image);
+      //magick_replace(e, mid, image);
     } catch (Exception &error_) {
       e->Throw(error_.what());
     }
@@ -1015,13 +1007,13 @@ namespace lib {
     try {
       DUInt mid;
       e->AssureScalarPar<DUIntGDL>(0, mid);
-      Image image = magick_image(e, mid);
+      Image* image = magick_image(e, mid);
       SizeT nparam = e->NParam();
       if (nparam != 1 && nparam != 4) e->Throw("invalid number of parameters for MAGICK_WRITECOLORTABLE Procedure.");
 //      unsigned int scale;
 //      scale = 255;
       //these would be Palette type images I bet.
-      image.type(PaletteType);
+      image->type(PaletteType);
 
       if (nparam == 4) { //get passed LUT
         BaseGDL* GDLCol = e->GetParDefined(1);
@@ -1040,21 +1032,21 @@ namespace lib {
         if (Red->N_Elements() == Green->N_Elements() &&
           Red->N_Elements() == Blue->N_Elements()) {
           unsigned long n = Red->N_Elements();
-          image.colorSpace(RGBColorspace);
-          image.colorMapSize(n);
-          image.quantize(n);
-          for (unsigned long c = 0; c < n; ++c) image.colorMap(c, ColorRGB((double)(*Red)[c] / 255., (double)(*Green)[c] / 255., (double)(*Blue)[c] / 255.));
+          image->colorSpace(RGBColorspace);
+          image->colorMapSize(n);
+          image->quantize(n);
+          for (unsigned long c = 0; c < n; ++c) image->colorMap(c, ColorRGB((double)(*Red)[c] / 255., (double)(*Green)[c] / 255., (double)(*Blue)[c] / 255.));
           }
       } else { //GET current LOADCT LUT
         PLINT r[ctSize], g[ctSize], b[ctSize];
         GraphicsDevice::GetDevice()->GetCT()->Get(r, g, b);
         unsigned long n = ctSize;
-        image.colorSpace(RGBColorspace);
-        image.colorMapSize(n);
-        image.quantize(n);
-        for (unsigned long c = 0; c < n; ++c) image.colorMap(c, ColorRGB((double)r[c] / 255., (double)g[c] / 255., (double)b[c] / 255.));
+        image->colorSpace(RGBColorspace);
+        image->colorMapSize(n);
+        image->quantize(n);
+        for (unsigned long c = 0; c < n; ++c) image->colorMap(c, ColorRGB((double)r[c] / 255., (double)g[c] / 255., (double)b[c] / 255.));
       }
-      magick_replace(e, mid, image);
+      //magick_replace(e, mid, image);
     } catch (Exception &error_) {
       e->Throw(error_.what());
     }
