@@ -18,6 +18,11 @@
 #include "datatypes.hpp"
 #include "envt.hpp"
 
+#if defined(USE_EIGEN)
+#include <Eigen/Dense>
+//using namespace Eigen; // never in header files!
+#endif
+
 namespace lib {
 
   BaseGDL* beseli_fun( EnvT* e);
@@ -32,5 +37,10 @@ namespace lib {
   BaseGDL* roberts_fun( EnvT* e);
   BaseGDL* prewitt_fun( EnvT* e);
 
+  BaseGDL* erode_fun( EnvT* e);
+  BaseGDL* dilate_fun( EnvT* e);
+
+  BaseGDL* matrix_multiply( EnvT* e);
+
 } // namespace
 
