@@ -85,28 +85,35 @@
 #define GM_2P0(a)							\
   e->NParam(a);								\
   									\
-  DIntGDL* p0 = e->GetParAs<DIntGDL>(0);				\
+  DIntGDL* p0 = e->IfDefGetParAs<DIntGDL>(0);				\
   SizeT nElp0 = p0->N_Elements();					\
   									\
-  if (nElp0 == 0)							\
+  if (nElp0 == 0 || p0 == NULL)							\
     throw GDLException(e->CallingNode(), "Variable is undefined: "+e->GetParString(0));	\
 
 #define GM_5P0(a)							\
-  e->NParam(a);								\
-  									\
-  DDoubleGDL* p0 = e->GetParAs<DDoubleGDL>(0);				\
+  e->NParam(a);													\
+  DDoubleGDL* p0 = e->IfDefGetParAs<DDoubleGDL>(0);				\
   SizeT nElp0 = p0->N_Elements();					\
   									\
-  if (nElp0 == 0)							\
+  if (nElp0 == 0 || p0 == NULL)							\
     throw GDLException(e->CallingNode(), "Variable is undefined: "+e->GetParString(0)); \
 
 #define GM_5P1()							\
-  DDoubleGDL* p1 = e->GetParAs<DDoubleGDL>(1);				\
+  DDoubleGDL* p1 = e->IfDefGetParAs<DDoubleGDL>(1);				\
   SizeT nElp1 = p1->N_Elements();					\
   									\
-  if (nElp1 == 0)							\
+  if (nElp1 == 0 || p1 == NULL)							\
     throw GDLException(e->CallingNode(), "Variable is undefined: "+e->GetParString(1));	\
 
+#define GM_5P2()							\
+  DDoubleGDL* p2 = e->IfDefGetParAs<DDoubleGDL>(2);				\
+  SizeT nElp2 = p2->N_Elements();					\
+  									\
+  if (nElp2 == 0 || p2 == NULL)							\
+    throw GDLException(e->CallingNode(), "Variable is undefined: "+e->GetParString(2));	\
+
+
 #define GM_CheckComplex_P0(flag)						\
   DType t0 = e->GetParDefined(0)->Type();				\
   if (flag == 1 && (t0 == GDL_COMPLEX || t0 == GDL_COMPLEXDBL))		\
@@ -117,6 +124,11 @@
   if (flag == 1 && (t1 == GDL_COMPLEX || t1 == GDL_COMPLEXDBL))		\
     e->Throw("Complex not implemented (GSL limitation). ");		\
 
+#define GM_CheckComplex_P2(flag)						\
+  DType t2 = e->GetParDefined(2)->Type();				\
+  if (flag == 1 && (t2 == GDL_COMPLEX || t2 == GDL_COMPLEXDBL))		\
+    e->Throw("Complex not implemented (GSL limitation). ");		\
+
 // Use this macro to define Inf and NaN, number of elements and result in a function with one parameter.
 // Note no STATIC for SysVar::, due to .reset 
 #define GM_NaN_Inf()							\
@@ -127,6 +139,28 @@
 #define GM_DF1()							\
   DDoubleGDL* res = new DDoubleGDL(p0->Dim(), BaseGDL::NOZERO);
 
+// Use this macro to define Inf and NaN, number of elements and result in a function with three parameters.
+#define GM_DF3()							\
+  DDoubleGDL* res;							    \
+  bool p0_isScalar = p0->Rank() == 0;     \
+  bool p1_isScalar = p1->Rank() == 0;     \
+  bool p2_isScalar = p2->Rank() == 0;    \
+  \
+  if (p0_isScalar && p1_isScalar && p2_isScalar)          \
+    res=new DDoubleGDL(p0->Dim(), BaseGDL::NOZERO);    \
+  else {\
+    int maxListNEl = max( max(p0->N_Elements(), p1->N_Elements() ), p2->N_Elements());  \
+    if( !p0_isScalar && ( p1_isScalar || p0->N_Elements() <= p1->N_Elements()) && ( p2_isScalar || p0->N_Elements() <= p2->N_Elements())) \
+      res=new DDoubleGDL(p0->Dim(), BaseGDL::NOZERO);    \
+    else if (!p1_isScalar && ( p0_isScalar || p1->N_Elements() <= p0->N_Elements()) && ( p2_isScalar || p1->N_Elements() <= p2->N_Elements())) \
+      res=new DDoubleGDL(p1->Dim(), BaseGDL::NOZERO);    \
+    else \
+      res=new DDoubleGDL(p2->Dim(), BaseGDL::NOZERO);    \
+  }\
+                      \
+  SizeT nElp = res->N_Elements();					\
+
+
 // Use this macro to define Inf and NaN, number of elements and result in a function with two parameters.
 #define GM_DF2()							\
   DDoubleGDL* res;							\
@@ -249,7 +283,7 @@ using std::isnan;
     if (nParam == 2) {
       GM_CheckComplex_P1(0);
       p1 = e->GetParAs<DDoubleGDL>(1);
-      SizeT nElp1 = p1->N_Elements();
+//      SizeT nElp1 = p1->N_Elements();
       tmp1=new DDoubleGDL(p1->Dim(), BaseGDL::NOZERO);
       t1bis = e->GetParDefined(1)->Type();
       //cout << "type : "<< t1 << endl;
@@ -519,6 +553,36 @@ using std::isnan;
     GM_CV2();
   } // beta_fun
 
+  BaseGDL* ibeta_fun(EnvT* e)
+  {
+    GM_CheckComplex_P0(1);
+    GM_CheckComplex_P1(1);
+    GM_CheckComplex_P2(1);
+    GM_5P0(3);
+    GM_5P1();
+    GM_5P2();
+    GM_DF3();
+    GM_NaN_Inf();
+
+      for (SizeT c = 0; c < nElp; ++c)
+      {
+        double a = p0_isScalar ? (*p0)[0] : (*p0)[c];
+        double b = p1_isScalar ? (*p1)[0] : (*p1)[c];
+        double z = p2_isScalar ? (*p2)[0] : (*p2)[c];
+
+        if(isfinite(z) && ( z < 0 || z > 1))
+            e->Throw("Argument Z must be in the range [0,1]");
+        else if (!isfinite(a)|| !isfinite(b)|| !isfinite(z))
+            (*res)[c] = d_nan;
+        else if ( ( (int) a == a && a <= 0.0) || ((int) b == b && b <= 0.0))
+            (*res)[c] = d_infinity;
+        else
+            (*res)[c] = gsl_sf_beta_inc(a, b, z);
+      }
+
+    GM_CV2();
+  } // beta_fun
+
   BaseGDL* expint_fun(EnvT* e)
   {
     // real part of Complex inputs are used
