@@ -40,7 +40,9 @@ const std::string GDL_OBJECT_NAME("GDL_OBJECT");
 const std::string GDL_CONTAINER_NAME("GDL_CONTAINER");
 
 extern VarListT      sysVarList;
+extern VarListT      obsoleteSysVarList;
 extern VarListT      sysVarRdOnlyList;
+extern VarListT      sysVarNoSaveList;
 
 extern FunListT      funList;
 extern ProListT      proList;
@@ -65,12 +67,16 @@ extern DLong64 CpuTPOOL_MAX_ELTS;
 // signals if control-c was pressed
 extern volatile bool sigControlC;
 
+extern volatile bool iAmANotebook;
+// tells if wxwidgets is to be used at all...
+extern volatile bool useWxWidgets;
 // tells if wxwidgets backend for graphics is to be used...
 extern volatile bool useWxWidgetsForGraphics;
-
+// do we force fonts to be the ugly IDL fonts?
+extern volatile bool forceWxWidgetsUglyFonts;
 //do we favor SIMD-accelerated random number generation?
 extern volatile bool useDSFMTAcceleration;
-
+extern volatile bool usePlatformDeviceName;
 extern          int  debugMode;
 
 enum DebugCode {
