@@ -71,16 +71,16 @@ namespace lib {
     }
     
     // MARGIN
-    gdlGetDesiredAxisMargin(e, "X",xMarginL, xMarginR);
-    gdlGetDesiredAxisMargin(e, "Y",yMarginB, yMarginT);
+    gdlGetDesiredAxisMargin(e, XAXIS,xMarginL, xMarginR);
+    gdlGetDesiredAxisMargin(e, YAXIS,yMarginB, yMarginT);
 
     // will handle axis logness..
     bool xLog, yLog, zLog;
     // is current box log or not?
     bool xAxisWasLog, yAxisWasLog, zAxisWasLog;
-    gdlGetAxisType("X", xAxisWasLog);
-    gdlGetAxisType("Y", yAxisWasLog);
-    gdlGetAxisType("Z", zAxisWasLog);
+    gdlGetAxisType(XAXIS, xAxisWasLog);
+    gdlGetAxisType(YAXIS, yAxisWasLog);
+    gdlGetAxisType(ZAXIS, zAxisWasLog);
     xLog=xAxisWasLog;
     yLog=yAxisWasLog; //by default logness is similar until another option is set
     zLog=zAxisWasLog;
@@ -102,8 +102,8 @@ namespace lib {
 
     // get viewport coordinates in normalised units
     PLFLT ovpXL, ovpXR, ovpYB, ovpYT;
-    gdlGetCurrentAxisWindow("X", ovpXL, ovpXR);
-    gdlGetCurrentAxisWindow("Y", ovpYB, ovpYT);
+    gdlGetCurrentAxisWindow(XAXIS, ovpXL, ovpXR);
+    gdlGetCurrentAxisWindow(YAXIS, ovpYB, ovpYT);
     //undefined or null previous viewport, seems IDL returns without complain:
     if ((ovpXL==ovpXR) || (ovpYB==ovpYT)) return;
 
@@ -113,8 +113,8 @@ namespace lib {
     DDouble oyStart, oyEnd;
 
     // get ![XY].CRANGE
-    gdlGetCurrentAxisRange("X", oxStart, oxEnd, FALSE); //ignore projection limits, convert to linear values if necessary.
-    gdlGetCurrentAxisRange("Y", oyStart, oyEnd, FALSE);
+    gdlGetCurrentAxisRange(XAXIS, oxStart, oxEnd, FALSE); //ignore projection limits, convert to linear values if necessary.
+    gdlGetCurrentAxisRange(YAXIS, oyStart, oyEnd, FALSE);
 
     if ((oyStart == oyEnd) || (oxStart == oxEnd))
     {
@@ -162,8 +162,8 @@ namespace lib {
 
     //XRANGE and YRANGE overrides all that, but  Start/End should be recomputed accordingly
     DDouble xAxisStart, xAxisEnd, yAxisStart, yAxisEnd;
-    bool setx=gdlGetDesiredAxisRange(e, "X", xAxisStart, xAxisEnd);
-    bool sety=gdlGetDesiredAxisRange(e, "Y", yAxisStart, yAxisEnd);
+    bool setx=gdlGetDesiredAxisRange(e, XAXIS, xAxisStart, xAxisEnd);
+    bool sety=gdlGetDesiredAxisRange(e, YAXIS, yAxisStart, yAxisEnd);
     if (sety)
     {
       yStart=yAxisStart;
@@ -182,28 +182,33 @@ namespace lib {
 
     // [XY]STYLE
     DLong xStyle=0, yStyle=0;
-    gdlGetDesiredAxisStyle(e, "X", xStyle);
-    gdlGetDesiredAxisStyle(e, "Y", yStyle);
+    gdlGetDesiredAxisStyle(e, XAXIS, xStyle);
+    gdlGetDesiredAxisStyle(e, YAXIS, yStyle);
 
      //xStyle and yStyle apply on range values
     if ((xStyle & 1) != 1) {
-      PLFLT intv = gdlAdjustAxisRange(e, "X", xStart, xEnd, xLog);
+      PLFLT intv = gdlAdjustAxisRange(e, XAXIS, xStart, xEnd, xLog);
     }
     if ((yStyle & 1) != 1) {
-      PLFLT intv = gdlAdjustAxisRange(e, "Y", yStart, yEnd, yLog);
+      PLFLT intv = gdlAdjustAxisRange(e, YAXIS, yStart, yEnd, yLog);
     }
     
     DDouble yVal, xVal;
     //in absence of arguments we will have:
+    bool ynodef=true;
+    bool xnodef=true;
     yVal=(standardNumPos)?oyStart:oyEnd;
     xVal=(standardNumPos)?oxStart:oxEnd;
     //read arguments 
     if (nParam() == 1) {
       e->AssureDoubleScalarPar( 0, xVal);
+      xnodef=false;
     }
     if (nParam() == 2) {
       e->AssureDoubleScalarPar( 0, xVal);
+      xnodef=false;
       e->AssureDoubleScalarPar( 1, yVal);
+      ynodef=false;
     }
     if (nParam() == 3) {
       e->Throw( "Sorry, we do not yet support the 3D case");
@@ -229,7 +234,7 @@ namespace lib {
     else //DATA
     {
 
-#ifdef USE_LIBPROJ4
+#ifdef USE_LIBPROJ
       // Map Stuff (xtype = 3)
       LPTYPE idata;
       XYTYPE odata;
@@ -243,11 +248,19 @@ namespace lib {
         {
           e->Throw("Projection initialization failed.");
         }
+#if PROJ_VERSION_MAJOR >= 5
+        idata.lam=xVal * DEG_TO_RAD;
+        idata.phi=yVal * DEG_TO_RAD;
+        odata=protect_proj_fwd_lp(idata, ref);
+        xVal=odata.x;
+        yVal=odata.y;
+#else
         idata.u=xVal * DEG_TO_RAD;
         idata.v=yVal * DEG_TO_RAD;
         odata=PJ_FWD(idata, ref);
         xVal=odata.u;
         yVal=odata.v;
+#endif
         DDouble *sx, *sy;
         GetSFromPlotStructs( &sx, &sy );
 
@@ -269,17 +282,17 @@ namespace lib {
 #define ADDEPSILON 0.1
     if ( standardNumPos )
     {
-      vpXL=(xAxis)?ovpXL       :vpX;
-      vpXR=(xAxis)?ovpXR       :vpX+ovpSizeY;
-      vpYB=(xAxis)?vpY         :ovpYB;
-      vpYT=(xAxis)?vpY+ovpSizeX:ovpYT;
+      vpXL=(xAxis || xnodef )?ovpXL       :vpX;
+      vpXR=(xAxis || xnodef )?ovpXR       :vpX+ovpSizeY;
+      vpYB=(yAxis || ynodef )?ovpYB       :vpY         ;
+      vpYT=(yAxis || ynodef )?ovpYT       :vpY+ovpSizeX;
     }
     else
     {
-      vpXL=(xAxis)?ovpXL:vpX-ovpSizeY;
-      vpXR=(xAxis)?ovpXR:vpX;
-      vpYB=(xAxis)?vpY-ovpSizeX:ovpYB;
-      vpYT=(xAxis)?vpY:ovpYT;
+      vpXL=(xAxis || xnodef )?ovpXL  :vpX-ovpSizeY;
+      vpXR=(xAxis || xnodef )?ovpXR  :vpX;
+      vpYB=(yAxis || ynodef )?ovpYB  :vpY-ovpSizeX;
+      vpYT=(yAxis || ynodef )?ovpYT  :vpY;
     }
      
     actStream->OnePageSaveLayout(); // one page
@@ -287,29 +300,31 @@ namespace lib {
     actStream->vpor(vpXL, vpXR, vpYB, vpYT);
     if (xLog) {xStart=log10(xStart);xEnd=log10(xEnd);}
     if (yLog) {yStart=log10(yStart);yEnd=log10(yEnd);}
+    
+    //insure 'wind' arguments are 
     actStream->wind(xStart, xEnd, yStart, yEnd);
 
     if ( xAxis )
-    { //special name "axisX" needed because we artificially changed size of box
-      gdlAxis(e, actStream, "axisX", xStart, xEnd, xLog, standardNumPos?1:2, ovpSizeY);
+    { //special ID "XAXIS2" needed because we artificially changed size of box
+      gdlAxis(e, actStream, XAXIS2, xStart, xEnd, xLog, standardNumPos?1:2, ovpSizeY);
 
       if ( e->KeywordSet(SAVEIx) )
       {
-        gdlStoreAxisCRANGE("X", xStart, xEnd, xLog);
-        gdlStoreAxisType("X", xLog);
-        gdlStoreAxisSandWINDOW(actStream, "X", xStart, xEnd, xLog);
+        gdlStoreAxisCRANGE(XAXIS, xStart, xEnd, xLog);
+        gdlStoreAxisType(XAXIS, xLog);
+        gdlStoreAxisSandWINDOW(actStream,XAXIS, xStart, xEnd, xLog);
       }
     }
 
     if ( yAxis )
-    {//special name "axisY" needed because we artificially changed size of box
-      gdlAxis(e, actStream, "axisY", yStart, yEnd, yLog, standardNumPos?1:2, ovpSizeX);
+    {//special id "YAXIS2" needed because we artificially changed size of box
+      gdlAxis(e, actStream, YAXIS2, yStart, yEnd, yLog, standardNumPos?1:2, ovpSizeX);
 
       if ( e->KeywordSet(SAVEIx) )
       {
-        gdlStoreAxisCRANGE("Y", yStart, yEnd, yLog);
-        gdlStoreAxisType("Y", yLog);
-        gdlStoreAxisSandWINDOW(actStream, "Y", yStart, yEnd, yLog);
+        gdlStoreAxisCRANGE(YAXIS, yStart, yEnd, yLog);
+        gdlStoreAxisType(YAXIS, yLog);
+        gdlStoreAxisSandWINDOW(actStream,YAXIS, yStart, yEnd, yLog);
       }
     }
     // reset the viewport and world coordinates to the original values
