@@ -88,9 +88,9 @@ namespace lib {
     GetSFromPlotStructs(&sx, &sy, &sz);
     
     bool xLog, yLog, zLog;
-    gdlGetAxisType("X", xLog);
-    gdlGetAxisType("Y", yLog);
-    gdlGetAxisType("Z", zLog);
+    gdlGetAxisType(XAXIS, xLog);
+    gdlGetAxisType(YAXIS, yLog);
+    gdlGetAxisType(ZAXIS, zLog);
         
     int xSize, ySize;
     //give default values
@@ -106,7 +106,7 @@ namespace lib {
     
     //projection?
       bool mapSet=false;
-#ifdef USE_LIBPROJ4
+#ifdef USE_LIBPROJ
       static LPTYPE idata;
       static XYTYPE odata;
       get_mapset ( mapSet );
@@ -122,20 +122,32 @@ namespace lib {
     {
       case DATA:
         // to u,v
-#ifdef USE_LIBPROJ4
+#ifdef USE_LIBPROJ
       if ( mapSet )
       {
+#ifdef PROJ_IS_THREADSAFE
 #pragma omp parallel if (nrows >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nrows))
         {
 #pragma omp for private(idata,odata)
+#endif
             for (SizeT i = 0; i < nrows; i++) {
+#if LIBPROJ_MAJOR_VERSION >= 5
+              idata.lam = (*xVal)[i] * DEG_TO_RAD;
+              idata.phi = (*yVal)[i] * DEG_TO_RAD;
+              odata = protect_proj_fwd_lp(idata, ref);
+              (*xVal)[i] = odata.x;
+              (*yVal)[i] = odata.y;
+#else
               idata.u = (*xVal)[i] * DEG_TO_RAD;
               idata.v = (*yVal)[i] * DEG_TO_RAD;
               odata = PJ_FWD(idata, ref);
               (*xVal)[i] = odata.u;
               (*yVal)[i] = odata.v;
+#endif
             }
+#ifdef PROJ_IS_THREADSAFE
           }
+#endif
         }
 #endif
         // to norm:
@@ -179,20 +191,32 @@ namespace lib {
       }
       
       // from u,v
-#ifdef USE_LIBPROJ4
+#ifdef USE_LIBPROJ
       if ( mapSet )
       {
+#ifdef PROJ_IS_THREADSAFE
 #pragma omp parallel if (nrows >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nrows))
         {
 #pragma omp for private(idata,odata)
+#endif
             for (SizeT i = 0; i < nrows; i++) {
+#if LIBPROJ_MAJOR_VERSION >= 5
+              odata.x = (*xVal)[i];
+              odata.y = (*yVal)[i];
+              idata = protect_proj_inv_xy(odata, ref);
+              (*xVal)[i] = idata.lam * RAD_TO_DEG;
+              (*yVal)[i] = idata.phi * RAD_TO_DEG;
+#else
               odata.u = (*xVal)[i];
               odata.v = (*yVal)[i];
               idata = PJ_INV(odata, ref);
               (*xVal)[i] = idata.u * RAD_TO_DEG;
               (*yVal)[i] = idata.v * RAD_TO_DEG;
+#endif
             }
+#ifdef PROJ_IS_THREADSAFE
           }
+#endif
         }
 #endif
 
