@@ -38,46 +38,43 @@ namespace lib {
 
     private: void call_plplot(EnvT* e, GDLGStream* actStream) // {{{
     {
-      DLong chan=0;
-      static int chanIx = e->KeywordIx("CHANNEL");
-      if (e->KeywordPresent(chanIx))  {
-        e->AssureLongScalarKWIfPresent(chanIx, chan);
-	      if ((chan > 3) || (chan < 0) )  e->Throw("Value of Channel is out of allowed range.");
-      } 
-      
-	  static DStructGDL* dStruct = SysVar::D();
-      DLong MaxColorIdx;
-	  MaxColorIdx = (*static_cast<DLongGDL*>
-		       (dStruct->GetTag(dStruct->Desc()->TagIndex("N_COLORS"),0)))[0];
-      
-      DLong bColor=-1;
-      static int bColorIx = e->KeywordIx("COLOR");
+      DLong chan = 0;
+      static int chanIx = e->KeywordIx( "CHANNEL" );
+      if ( e->KeywordPresent( chanIx ) ) {
+        e->AssureLongScalarKWIfPresent( chanIx, chan );
+        if ( (chan > 3) || (chan < 0) ) e->Throw( "Value of Channel is out of allowed range." );
+      } else { //get !P.CHANNEL value
+          DStructGDL* pStruct = SysVar::P( );   //MUST NOT BE STATIC, due to .reset 
+          chan = (*static_cast<DLongGDL*>
+          (pStruct->GetTag( pStruct->Desc( )->TagIndex( "CHANNEL" ), 0 )))[0];
+      }
 
-      if (nParam() == 0)
- 	{
-	  if (e->KeywordPresent(e->KeywordIx("COLOR")))
-	    {
-	      e->AssureLongScalarKWIfPresent(bColorIx, bColor);
-	      if (bColor > MaxColorIdx) bColor = MaxColorIdx;
-	      if (bColor < 0)   bColor = 0;
-	    }
-	  else
-	    // we have to read back !p.background value
-	    {
-	      static DStructGDL* pStruct = SysVar::P();
-	      bColor =(*static_cast<DLongGDL*>
-		       (pStruct->GetTag(pStruct->Desc()->TagIndex("BACKGROUND"),0)))[0];
-	    }
-	}
-      else 
-	{
-	  e->AssureLongScalarPar(0, bColor);	
-	  if (bColor > MaxColorIdx) bColor = MaxColorIdx;
-	  if (bColor < 0)   bColor = 0;
-	}
-    DLong decomposed=GraphicsDevice::GetDevice()->GetDecomposed();
-    actStream->Background( bColor, decomposed);
-    if (chan > 0) actStream->Clear(chan-1); else actStream->Clear();      
+      DStructGDL* dStruct = SysVar::D( );   //MUST NOT BE STATIC, due to .reset 
+      DLong MaxColorIdx;
+      MaxColorIdx = (*static_cast<DLongGDL*>
+      (dStruct->GetTag( dStruct->Desc( )->TagIndex( "N_COLORS" ), 0 )))[0];
+
+      DLong bColor = -1;
+      static int bColorIx = e->KeywordIx( "COLOR" );
+
+      if ( nParam( ) == 0 ) {
+        if ( e->KeywordPresent( bColorIx ) ) {
+          e->AssureLongScalarKWIfPresent( bColorIx, bColor );
+        } else
+          // we have to read back !p.background value
+        {
+          DStructGDL* pStruct = SysVar::P( );   //MUST NOT BE STATIC, due to .reset 
+          bColor = (*static_cast<DLongGDL*>
+          (pStruct->GetTag( pStruct->Desc( )->TagIndex( "BACKGROUND" ), 0 )))[0];
+        }
+      } else {
+        e->AssureLongScalarPar( 0, bColor );
+      }
+      if ( bColor > MaxColorIdx ) bColor = MaxColorIdx;
+      if ( bColor < 0 ) bColor = 0;
+      DLong decomposed = GraphicsDevice::GetDevice( )->GetDecomposed( );
+      actStream->Background( bColor, decomposed );
+      if ( chan > 0 ) actStream->Clear( chan - 1 ); else actStream->Clear( );      
     }
 
     private: virtual void post_call(EnvT*, GDLGStream*) // {{{
