@@ -215,14 +215,14 @@ namespace lib
       if (coordinateSystem == DEVICE) doT3d =false;
       
       // get_axis_type
-      gdlGetAxisType("X", xLog);
-      gdlGetAxisType("Y", yLog);
-      gdlGetAxisType("Z", zLog);
+      gdlGetAxisType(XAXIS, xLog);
+      gdlGetAxisType(YAXIS, yLog);
+      gdlGetAxisType(ZAXIS, zLog);
       
       //get DATA limits (not necessary CRANGE, see AXIS / SAVE behaviour!)
       GetCurrentUserLimits(actStream, xStart, xEnd, yStart, yEnd);
       // get !Z.CRANGE
-      gdlGetCurrentAxisRange("Z", zStart, zEnd);
+      gdlGetCurrentAxisRange(ZAXIS, zStart, zEnd);
 
       if (zStart != 0.0 && zStart == zEnd)
       {
@@ -240,7 +240,7 @@ namespace lib
       restorelayout=true;
 
       mapSet=false;
-#ifdef USE_LIBPROJ4
+#ifdef USE_LIBPROJ
       get_mapset(mapSet);
       mapSet=(mapSet && coordinateSystem==DATA);
       if ( mapSet )
@@ -400,7 +400,7 @@ namespace lib
         
         ///TODO: Get proper USerSymSize in 3D.
         
-#ifdef USE_LIBPROJ4
+#ifdef USE_LIBPROJ
         if ( mapSet && psym < 1) {
           GDLgrProjectedPolygonPlot(actStream, ref, NULL, xValou, yValou, false, false, NULL);
           psym=-psym;
