@@ -183,14 +183,14 @@ namespace lib
       if (coordinateSystem == DEVICE) doT3d =false;
       
       // get_axis_type
-      gdlGetAxisType("X", xLog);
-      gdlGetAxisType("Y", yLog);
-      gdlGetAxisType("Z", zLog);
+      gdlGetAxisType(XAXIS, xLog);
+      gdlGetAxisType(YAXIS, yLog);
+      gdlGetAxisType(ZAXIS, zLog);
 
       //get DATA limits (not necessary CRANGE, see AXIS / SAVE behaviour!)
       GetCurrentUserLimits(actStream, xStart, xEnd, yStart, yEnd);
       // get !Z.CRANGE
-      gdlGetCurrentAxisRange("Z", zStart, zEnd);
+      gdlGetCurrentAxisRange(ZAXIS, zStart, zEnd);
 
       if (zStart != 0.0 && zStart == zEnd)
       {
@@ -206,7 +206,7 @@ namespace lib
       restorelayout=true;
 
       mapSet=false;
-#ifdef USE_LIBPROJ4
+#ifdef USE_LIBPROJ
       get_mapset(mapSet);
       mapSet=(mapSet && coordinateSystem==DATA);
       if ( mapSet )
@@ -380,18 +380,45 @@ namespace lib
         
         ///TODO: Get proper USerSymSize in 3D.
         
-#ifdef USE_LIBPROJ4
+#ifdef USE_LIBPROJ
         if (mapSet) 
           GDLgrProjectedPolygonPlot(actStream, ref, NULL, xVal, yVal, false, true, NULL);
         else  actStream->fill(xEl, static_cast<PLFLT*>(&(*xValou)[0]), static_cast<PLFLT*>(&(*yValou)[0]));
-      } else {  
-        if (mapSet) GDLgrProjectedPolygonPlot(actStream, ref, NULL, xVal, yVal, false, true, NULL);
-        else actStream->fill(xEl, static_cast<PLFLT*>(&(*xVal)[0]), static_cast<PLFLT*>(&(*yVal)[0]));
+      } else { 
+        Guard<BaseGDL> xval_guard, yval_guard;
+        DDoubleGDL *xValou=xVal;
+        DDoubleGDL *yValou=yVal;
+        if (xLog) {
+            xValou=new DDoubleGDL(dimension(xEl));
+            xval_guard.reset(xValou);
+            for (SizeT i=0; i<xEl; ++i) (*xValou)[i]=log10((*xVal)[i]);
+          }
+        if (yLog) {
+            yValou=new DDoubleGDL(dimension(yEl));
+            yval_guard.reset(yValou);
+            for (SizeT i=0; i<yEl; ++i) (*yValou)[i]=log10((*yVal)[i]);
+          }
+
+        if (mapSet) GDLgrProjectedPolygonPlot(actStream, ref, NULL, xValou, yValou, false, true, NULL);
+        else actStream->fill(xEl, static_cast<PLFLT*>(&(*xValou)[0]), static_cast<PLFLT*>(&(*yValou)[0]));
       }
 #else
         actStream->fill(xEl, static_cast<PLFLT*>(&(*xValou)[0]), static_cast<PLFLT*>(&(*yValou)[0]));
-      } else {  
-        actStream->fill(xEl, static_cast<PLFLT*>(&(*xVal)[0]), static_cast<PLFLT*>(&(*yVal)[0]));
+      } else { 
+        Guard<BaseGDL> xval_guard, yval_guard;
+        DDoubleGDL *xValou=xVal;
+        DDoubleGDL *yValou=yVal;
+        if (xLog) {
+            xValou=new DDoubleGDL(dimension(xEl));
+            xval_guard.reset(xValou);
+            for (SizeT i=0; i<xEl; ++i) (*xValou)[i]=log10((*xVal)[i]);
+          }
+        if (yLog) {
+            yValou=new DDoubleGDL(dimension(yEl));
+            yval_guard.reset(yValou);
+            for (SizeT i=0; i<yEl; ++i) (*yValou)[i]=log10((*yVal)[i]);
+          }
+        actStream->fill(xEl, static_cast<PLFLT*>(&(*xValou)[0]), static_cast<PLFLT*>(&(*yValou)[0]));
       }
 #endif
       if (stopClip) stopClipping(actStream);
