@@ -86,80 +86,94 @@ namespace lib
 	// should be OK ...
 	    if ( (e->GetNumericArrayParDefined ( 0 ))->EquivalentRank ( )!=2 ) e->Throw ( "Array must have 2 dimensions: "+e->GetParString ( 0 ) );
     }
-    if ( nParam ( )==1 )
-      {
-        BaseGDL* p0=e->GetNumericArrayParDefined ( 0 )->Transpose ( NULL );
-        p0_guard.Init ( p0 ); // delete upon exit
+    if ( nParam ( )==1) {
+        BaseGDL* p0 = e->GetNumericArrayParDefined(0)->Transpose(NULL);
+        p0_guard.Init(p0); // delete upon exit
 
-        zVal=static_cast<DDoubleGDL*>( p0->Convert2 ( GDL_DOUBLE, BaseGDL::COPY ) );
-        zval_guard.Init ( zVal ); // delete upon exit
+        zVal = static_cast<DDoubleGDL*> (p0->Convert2(GDL_DOUBLE, BaseGDL::COPY));
+        zval_guard.Init(zVal); // delete upon exit
 
-        xEl=zVal->Dim ( 1 );
-        yEl=zVal->Dim ( 0 );
+        xEl = zVal->Dim(1);
+        yEl = zVal->Dim(0);
 
-        xVal=new DDoubleGDL ( dimension ( xEl ), BaseGDL::INDGEN );
-        xval_guard.Init ( xVal ); // delete upon exit
+        xVal = new DDoubleGDL(dimension(xEl), BaseGDL::INDGEN);
+        xval_guard.Init(xVal); // delete upon exit
         if (xLog) xVal->Inc();
-        yVal=new DDoubleGDL ( dimension ( yEl ), BaseGDL::INDGEN );
-        yval_guard.Init ( yVal ); // delete upon exit
+        yVal = new DDoubleGDL(dimension(yEl), BaseGDL::INDGEN);
+        yval_guard.Init(yVal); // delete upon exit
         if (yLog) yVal->Inc();
-      }
-      else
-      {
-        BaseGDL* p0=e->GetNumericArrayParDefined ( 0 )->Transpose ( NULL );
-        p0_guard.Init ( p0 ); // delete upon exit
+      } else {
+        BaseGDL* p0 = e->GetNumericArrayParDefined(0)->Transpose(NULL);
+        p0_guard.Init(p0); // delete upon exit
+
+        zVal = static_cast<DDoubleGDL*> (p0->Convert2(GDL_DOUBLE, BaseGDL::COPY));
+        zval_guard.Init(zVal); // delete upon exit
+
+        xVal = e->GetParAs< DDoubleGDL>(1);
+        yVal = e->GetParAs< DDoubleGDL>(2);
+
+        if (xVal->Rank() > 2)
+          e->Throw("X, Y, or Z array dimensions are incompatible.");
+
+        if (yVal->Rank() > 2)
+          e->Throw("X, Y, or Z array dimensions are incompatible.");
+        if (xVal->Rank() == 0 || yVal->Rank() == 0)
+          e->Throw("X, Y, or Z array dimensions are incompatible.");
+
+        if (xVal->Rank() == 1) {
+          xEl = xVal->Dim(0);
+
+          if (xEl != zVal->Dim(1))
+            e->Throw("X, Y, or Z array dimensions are incompatible.");
+        }
+
+        if (yVal->Rank() == 1) {
+          yEl = yVal->Dim(0);
+
+          if (yEl != zVal->Dim(0))
+            e->Throw("X, Y, or Z array dimensions are incompatible.");
+        }
+
+        if (xVal->Rank() == 2) {
+          //plplot is unable to handle such subtetlies, better to throw?
+          e->Throw("Sorry, plplot cannot handle 2D X coordinates in its 3D plots.");
+          xEl = xVal->Dim(0);
+          if ((xVal->Dim(0) != zVal->Dim(1))&&(xVal->Dim(1) != zVal->Dim(0)))
+            e->Throw("X, Y, or Z array dimensions are incompatible.");
+        }
+
+        if (yVal->Rank() == 2) {
+        //plplot is unable to handle such subtetlies, better to throw?
+          e->Throw("Sorry, plplot cannot handle 2D Y coordinates in its 3D plots.");
+          yEl = yVal->Dim(1);
+          if ((yVal->Dim(0) != zVal->Dim(1))&&(yVal->Dim(1) != zVal->Dim(0)))
+            e->Throw("X, Y, or Z array dimensions are incompatible.");
+        }
         
-        zVal=static_cast<DDoubleGDL*>( p0->Convert2 ( GDL_DOUBLE, BaseGDL::COPY ) );
-        zval_guard.Init ( zVal ); // delete upon exit
-
-        xVal=e->GetParAs< DDoubleGDL>( 1 );
-        yVal=e->GetParAs< DDoubleGDL>( 2 );
-
-	    if ( xVal->Rank ( )>2 )
-	      e->Throw ( "X, Y, or Z array dimensions are incompatible." );
-
-	    if ( yVal->Rank ( )>2 )
-	      e->Throw ( "X, Y, or Z array dimensions are incompatible." );
-	    if ( xVal->Rank ( )==0 || yVal->Rank ( )==0 )
-	      e->Throw ( "X, Y, or Z array dimensions are incompatible." );
-
-        if ( xVal->Rank ( )==1 )
-        {
-          xEl=xVal->Dim ( 0 );
-
-          if ( xEl!=zVal->Dim ( 1 ) )
-            e->Throw ( "X, Y, or Z array dimensions are incompatible." );
-        }
-
-        if ( yVal->Rank ( )==1 )
-        {
-          yEl=yVal->Dim ( 0 );
-
-          if ( yEl!=zVal->Dim ( 0 ) )
-            e->Throw ( "X, Y, or Z array dimensions are incompatible." );
-        }
-
-	    if ( xVal->Rank ( )==2 )
-	      {
-		xEl=xVal->Dim ( 0 );
-		if ( ( xVal->Dim ( 0 )!=zVal->Dim ( 1 ) )&&( xVal->Dim ( 1 )!=zVal->Dim ( 0 ) ) )
-		  e->Throw ( "X, Y, or Z array dimensions are incompatible." );
+        //plplot is unable to handle such subtetlies, better to throw?
+        
+//        // But if X is 2D and Y is 1D (or reciprocally), we need to promote the 1D to 2D since this is supported by IDL
+//        if (xVal->Rank() == 1 && yVal->Rank() == 2) {
+//          DDoubleGDL* xValExpanded = new DDoubleGDL(zVal->Dim(), BaseGDL::NOZERO);
+//          SizeT k = 0;
+//          for (SizeT j = 0; j < zVal->Dim(1); ++j) for (SizeT i = 0; i < zVal->Dim(0); ++i) (*xValExpanded)[k++] = (*xVal)[i];
+//          xval_guard.Init(xValExpanded); // delete upon exit
+//          xVal = xValExpanded;
+//        } else if (xVal->Rank() == 2 && yVal->Rank() == 1) {
+//          DDoubleGDL* yValExpanded = new DDoubleGDL(zVal->Dim(), BaseGDL::NOZERO);
+//          SizeT k = 0;
+//          for (SizeT j = 0; j < zVal->Dim(1); ++j) for (SizeT i = 0; i < zVal->Dim(0); ++i) (*yValExpanded)[k++] = (*yVal)[j];
+//          xval_guard.Init(yValExpanded); // delete upon exit
+//          yVal = yValExpanded;
+//        }
       }
-
-	    if ( yVal->Rank ( )==2 )
-	      {
-		yEl=yVal->Dim ( 1 );
-		if ( ( yVal->Dim ( 0 )!=zVal->Dim ( 1 ) )&&( yVal->Dim ( 1 )!=zVal->Dim ( 0 ) ) )
-		  e->Throw ( "X, Y, or Z array dimensions are incompatible." );
-	      }
-	  }
       
       GetMinMaxVal ( xVal, &xStart, &xEnd );
       GetMinMaxVal ( yVal, &yStart, &yEnd );
       //XRANGE and YRANGE overrides all that, but  Start/End should be recomputed accordingly
       DDouble xAxisStart, xAxisEnd, yAxisStart, yAxisEnd;
-      bool setx=gdlGetDesiredAxisRange(e, "X", xAxisStart, xAxisEnd);
-      bool sety=gdlGetDesiredAxisRange(e, "Y", yAxisStart, yAxisEnd);
+      bool setx=gdlGetDesiredAxisRange(e, XAXIS, xAxisStart, xAxisEnd);
+      bool sety=gdlGetDesiredAxisRange(e, YAXIS, yAxisStart, yAxisEnd);
       if (setx) {
         xStart=xAxisStart;
         xEnd=xAxisEnd;
@@ -175,7 +189,7 @@ namespace lib
       GetMinMaxVal ( zVal, &datamin, &datamax );
       zStart=datamin;
       zEnd=datamax;
-      setZrange = gdlGetDesiredAxisRange(e, "Z", zStart, zEnd);
+      setZrange = gdlGetDesiredAxisRange(e, ZAXIS, zStart, zEnd);
 
         return false;
     } 
@@ -205,9 +219,9 @@ namespace lib
 
       // [XYZ]STYLE
       DLong xStyle=0, yStyle=0, zStyle=0; ;
-      gdlGetDesiredAxisStyle(e, "X", xStyle);
-      gdlGetDesiredAxisStyle(e, "Y", yStyle);
-      gdlGetDesiredAxisStyle(e, "Z", zStyle);
+      gdlGetDesiredAxisStyle(e, XAXIS, xStyle);
+      gdlGetDesiredAxisStyle(e, YAXIS, yStyle);
+      gdlGetDesiredAxisStyle(e, ZAXIS, zStyle);
 
       //check here since after AutoIntvAC values will be good but arrays passed
       //to plplot will be bad...
@@ -226,12 +240,12 @@ namespace lib
 
       if ( ( xStyle&1 )!=1 )
       {
-        PLFLT intv=gdlAdjustAxisRange (e,"X",  xStart, xEnd, xLog );
+        PLFLT intv=gdlAdjustAxisRange (e,XAXIS,  xStart, xEnd, xLog );
       }
 
       if ( ( yStyle&1 )!=1 )
       {
-        PLFLT intv=gdlAdjustAxisRange (e, "Y", yStart, yEnd, yLog );
+        PLFLT intv=gdlAdjustAxisRange (e, YAXIS, yStart, yEnd, yLog );
       }
 
       static int MIN_VALUEIx = e->KeywordIx( "MIN_VALUE");
@@ -252,7 +266,7 @@ namespace lib
       // then only apply expansion  of axes:
       if ( ( zStyle&1 )!=1 )
       {
-        PLFLT intv=gdlAdjustAxisRange (e, "Z", zStart, zEnd, zLog );
+        PLFLT intv=gdlAdjustAxisRange (e, ZAXIS, zStart, zEnd, zLog );
       }
 
       // background BEFORE next plot since it is the only place plplot may redraw the background...
