@@ -21,10 +21,10 @@
 SizeT w1 = width[0] / 2;
 SizeT w2 = width[1] / 2;
 SizeT nEl = dimx*dimy;
-SMOOTH_Ty tmp[nEl];
-#pragma omp parallel if (nEl >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nEl))
+SMOOTH_Ty* tmp=(SMOOTH_Ty*)malloc(nEl*sizeof(SMOOTH_Ty));
+//parallelizing all that stuff is more complicated than a single pragma...
+//#pragma omp parallel if (nEl >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nEl))
 {
-#pragma omp for
  for (SizeT j = 0; j < dimy; ++j) {
   DDouble z;
   DDouble n = 0;
@@ -141,7 +141,6 @@ SMOOTH_Ty tmp[nEl];
 #endif
  }
 
-#pragma omp for
  for (SizeT j = 0; j < dimx; ++j) {
   DDouble z;
   DDouble n = 0;
@@ -258,4 +257,6 @@ SMOOTH_Ty tmp[nEl];
 #endif
  }
 }
+free(tmp);
+
 #endif
