@@ -16,7 +16,8 @@
  *                                                                         *
  ***************************************************************************/
 
-// basic_op.cpp
+#ifndef SPECIALIZATIONS_DATATYPES_HPP_
+#define SPECIALIZATIONS_DATATYPES_HPP_
 
 template<>
 SizeT Data_<SpDObj>::N_Elements() const; 
@@ -363,14 +364,14 @@ Data_<SpDByte>* Data_<SpDComplexDbl>::LogNeg();
 
 // template<class Sp>
 // bool Data_<Sp>::OutOfRangeOfInt() const 
-template<>
-bool Data_<SpDString>::OutOfRangeOfInt() const; 
-template<>
-bool Data_<SpDByte>::OutOfRangeOfInt() const;
-template<>
-bool Data_<SpDComplex>::OutOfRangeOfInt() const;
-template<>
-bool Data_<SpDComplexDbl>::OutOfRangeOfInt() const; 
+//template<>
+//bool Data_<SpDString>::OutOfRangeOfInt() const; 
+//template<>
+//bool Data_<SpDByte>::OutOfRangeOfInt() const;
+//template<>
+//bool Data_<SpDComplex>::OutOfRangeOfInt() const;
+//template<>
+//bool Data_<SpDComplexDbl>::OutOfRangeOfInt() const; 
 
 /*template<> 
 void* Data_<SpDString>::DataAddr();//SizeT);*/
@@ -697,3 +698,5 @@ PyObject* Data_<SpDComplexDbl>::ToPythonScalar();
 template<> 
 PyObject* Data_<SpDString>::ToPythonScalar();
 #endif
+
+#endif
