@@ -61,6 +61,7 @@ bool CompleteFileName(std::string& fn);
 // Strip whitespace from the start and end of a string.
 inline void StrTrim(std::string& s)
 {
+  if (s.length() == 0) return;
   unsigned long first=s.find_first_not_of(" \t");
   if( first == s.npos)
     {
@@ -68,6 +69,11 @@ inline void StrTrim(std::string& s)
       return;
     }
   long last=s.find_last_not_of(" \t");
+  if( last == s.npos)
+    {
+      s=s.substr(first);
+      return;
+    }
   s=s.substr(first,last-first+1);
 }
 
@@ -117,6 +123,13 @@ long int Str2L( const char* c, int base=10);
 long int Str2L( const std::string& s, int base=10);
 unsigned long int Str2UL( const char* c, int base=10);
 unsigned long int Str2UL( const std::string& s, int base=10);
-
+namespace lib {
+#ifdef _WIN32
+  extern bool posixpaths;
+#endif
+  std::string PathSeparator();
+  std::string SearchPathSeparator();
+  std::string ParentDirectoryIndicator();
+}
 
 #endif
