@@ -29,7 +29,7 @@ namespace lib {
   void GDL_Triangulate(EnvT* e);
   BaseGDL* trigrid_fun( EnvT* e);
   void grid_input (EnvT* e);
-#ifdef PL_HAVE_QHULL
+#ifdef HAVE_QHULL
   void qhull ( EnvT* e);
   BaseGDL* qgrid3_fun ( EnvT* e);
 #endif
